<?php

namespace Sapienttree\Luckydraw\Helper;

use \Magento\Framework\App\Helper\AbstractHelper;

class Data extends AbstractHelper {

    const ENROLL_PATH_EMAIL_TEMPLATE_FIELD = 'luckydraw/general/enroll_mail_template';
    const WINNER_PATH_EMAIL_TEMPLATE_FIELD = 'luckydraw/general/winner_mail_template';

    /**
     * Store manager
     *
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Translate\Inline\StateInterface
     */
    protected $inlineTranslation;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $_transportBuilder;

    /**
     * @var string
     */
    protected $temp_id;
    protected $_luckydrawmodel;
    protected $_objectManager;

    public function __construct(\Magento\Framework\App\Helper\Context $context, \Magento\Store\Model\StoreManagerInterface $storeManager, \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation, \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder) {
        parent::__construct($context);
        $this->_storeManager = $storeManager;
        $this->inlineTranslation = $inlineTranslation;
        $this->_transportBuilder = $transportBuilder;
        $this->_objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $luckydrawmodel = $this->_objectManager->create('Sapienttree\Luckydraw\Model\Luckydraw');
        $this->_luckydrawmodel = $luckydrawmodel;
    }

    public function confirmLuckyWinner($id) {
        $lucky_draw = $this->_luckydrawmodel->load($id);
        $lucky_draw_details = $lucky_draw->getData();
        /* Receiver Detail  */
        $receiverInfo = [
            'name' => $lucky_draw_details['customer_name'],
            'email' => $lucky_draw_details['customer_email']
        ];
        /* Sender Detail  */
        $sender_name = $this->scopeConfig->getValue('trans_email/ident_general/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $sender_email = $this->scopeConfig->getValue('trans_email/ident_general/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $senderInfo = [
            'name' => $sender_name,
            'email' => $sender_email,
        ];
        $emailTempVariables = array();
        $emailTemplateVariables = $lucky_draw_details;
        $emailTemplateVariables['ticketDetails'] = "<tr><td>{$lucky_draw_details['ticket']}</td></tr>";
        $this->SendEmailOnWinnerConfirmation($emailTemplateVariables, $senderInfo, $receiverInfo);
        $lucky_draw->setIsWinner(1);
        $lucky_draw->save();
        unset($tickets);
        unset($data);
        unset($emailTemplateVariables);
    }

    public function enrollCustomerToLuckyDraw($_order) {
        $data = $_order->getData();
        $shipping_address = $_order->getShippingAddress()->getData();
        $flag = 1;
        $is_active = $this->scopeConfig->getValue('luckydraw/general/enable', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if (!$is_active) {
            $flag = 0;
            return;
        }
        $start_date = $this->scopeConfig->getValue('luckydraw/general/start_date', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $end_date = $this->scopeConfig->getValue('luckydraw/general/end_date', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $contest_name = $this->scopeConfig->getValue('luckydraw/general/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $order_date = date('Y-m-d', strtotime($data['created_at']));
        if ($start_date || $end_date) {
            if ($start_date && $end_date) {
                $contest_name .= " between $start_date and $end_date";
                $flag = (($order_date >= $start_date && $order_date <= $end_date) && $flag) ? 1 : 0;
            } elseif ($start_date && !$end_date) {
                $flag = ($order_date >= $start_date && $flag) ? 1 : 0;
                $contest_name .= " between $start_date till Today";
            } elseif (!$start_date && $end_date) {
                $contest_name .= " till $end_date";
                $flag = ($order_date <= $end_date && $flag) ? 1 : 0;
            }
        }

        //Here to check the order status for enrolling orders
        $status = $data['status'];
        $allowed_statues = $this->scopeConfig->getValue('luckydraw/general/order_status', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if ($allowed_statues) {
            $expld = explode(',', $allowed_statues);
            $flag = (in_array($status, $expld) && $flag) ? 1 : 0;
        }


        //Here to set customer name if guest
        if(isset($data['customer_firstname']) && !$data['customer_firstname']){
            $data['customer_firstname'] = $shipping_address['firstname'];
        }
        if(isset($data['customer_lastname']) && !$data['customer_lastname']){
            $data['customer_lastname'] = $shipping_address['lastname'];
        }

        //Here if lucky draw is active
        if ($flag) {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $luckydrawCollection = $this->_objectManager->create('\Sapienttree\Luckydraw\Model\ResourceModel\Luckydraw\Collection');
            $luckydrawCollection->addFieldToFilter('order_number', array('eq' => $data['increment_id']));

            if (!$luckydrawCollection->getSize()) {
                //Here let's check if a customer already enrolled for an order then do not allow further
                $tickets = array();
                $currency = $this->_objectManager->get('\Magento\Directory\Model\Currency');
                $order_base_total = $data['base_grand_total'];
                $freq = $this->getEnrolledFrequency($order_base_total);
                $dis_order_base_total = $currency->format($order_base_total, ['display' => \Zend_Currency::NO_SYMBOL], false);
                $grand_total = $currency->format($data['grand_total'], ['display' => \Zend_Currency::NO_SYMBOL], false);

                $enroll_data = array();
                for ($i = 0; $i < $freq; $i++) {
                    $enroll_data['contest_name'] = $contest_name;
                    $enroll_data['customer_id'] = isset($data['customer_id']) ? $data['customer_id'] : '';
                    $enroll_data['customer_name'] = $data['customer_firstname'] . " " . $data['customer_lastname'];
                    $enroll_data['customer_email'] = $data['customer_email'];
                    $enroll_data['customer_phone'] = (!empty($shipping_address) && $shipping_address['telephone']) ? $shipping_address['telephone'] : '';
                    $enroll_data['order_id'] = isset($data['id']) ? $data['id'] : '';
                    $enroll_data['order_number'] = $data['increment_id'];
                    $enroll_data['order_total'] = $order_base_total;
                    $enroll_data['ticket'] = uniqid();
                    $this->_luckydrawmodel->setData($enroll_data);
                    $this->_luckydrawmodel->save();
                    $tickets[] = $enroll_data['ticket'];
                    $enroll_data = array();
                }
                if ($freq) {
                    /* Receiver Detail  */
                    $receiverInfo = [
                        'name' => $data['customer_firstname'] . " " . $data['customer_lastname'],
                        'email' => $data['customer_email']
                    ];
                    /* Sender Detail  */
                    $sender_name = $this->scopeConfig->getValue('trans_email/ident_general/name', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
                    $sender_email = $this->scopeConfig->getValue('trans_email/ident_general/email', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
                    $senderInfo = [
                        'name' => $sender_name,
                        'email' => $sender_email,
                    ];
                    $emailTempVariables = array();
                    $emailTemplateVariables = $data;
                    $emailTemplateVariables['base_grand_total'] = $dis_order_base_total;
                    $emailTemplateVariables['grand_total'] = $grand_total;
                    $emailTemplateVariables['customer_name'] = $data['customer_firstname'] . " " . $data['customer_lastname'];
                    $ticketDetails = "";
                    foreach ($tickets as $k => $ticket) {
                        $sl_no = ++$k;
                        $ticketDetails .= "<tr><td>$sl_no</td><td>$ticket</td></tr>";
                    }
                    $emailTemplateVariables['ticketDetails'] = $ticketDetails;
                    $emailTemplateVariables['frequency'] = $freq;
                    $this->SendEmailAfterEnroll($emailTemplateVariables, $senderInfo, $receiverInfo);
                    unset($tickets);
                    unset($data);
                    unset($emailTemplateVariables);
                }
            }
        }
    }

    /**
     * Return store configuration value of your template field that which id you set for template
     *
     * @param string $path
     * @param int $storeId
     * @return mixed
     */
    protected function getConfigValue($path, $storeId) {
        return $this->scopeConfig->getValue(
                        $path, \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $storeId
        );
    }

    /**
     * Return store 
     *
     * @return Store
     */
    public function getStore() {
        return $this->_storeManager->getStore();
    }

    /**
     * Return template id according to store
     *
     * @return mixed
     */
    public function getTemplateId($xmlPath) {
        return $this->getConfigValue($xmlPath, $this->getStore()->getStoreId());
    }

    /**
     * [generateTemplate description]  with template file and tempaltes variables values                
     * @param  Mixed $emailTemplateVariables 
     * @param  Mixed $senderInfo             
     * @param  Mixed $receiverInfo           
     * @return void
     */
    public function generateTemplate($emailTemplateVariables, $senderInfo, $receiverInfo) {
        $template = $this->_transportBuilder->setTemplateIdentifier($this->temp_id)
                ->setTemplateOptions(
                        [
                            'area' => \Magento\Framework\App\Area::AREA_ADMINHTML, /* here you can defile area and
                              store of template for which you prepare it */
                            'store' => $this->_storeManager->getStore()->getId(),
                        ]
                )
                ->setTemplateVars($emailTemplateVariables)
                ->setFrom($senderInfo)
                ->addTo($receiverInfo['email'], $receiverInfo['name']);
        return $this;
    }

    /**
     * [sendInvoicedOrderEmail description]                  
     * @param  Mixed $emailTemplateVariables 
     * @param  Mixed $senderInfo             
     * @param  Mixed $receiverInfo           
     * @return void
     */
    /* your send mail method */
    public function SendEmailAfterEnroll($emailTemplateVariables, $senderInfo, $receiverInfo) {
        $this->temp_id = $this->getTemplateId(self::ENROLL_PATH_EMAIL_TEMPLATE_FIELD);
        $this->inlineTranslation->suspend();
        $this->generateTemplate($emailTemplateVariables, $senderInfo, $receiverInfo);
        $transport = $this->_transportBuilder->getTransport();
        $transport->sendMessage();
        $this->inlineTranslation->resume();
    }

    /**
     * [sendInvoicedOrderEmail description]                  
     * @param  Mixed $emailTemplateVariables 
     * @param  Mixed $senderInfo             
     * @param  Mixed $receiverInfo           
     * @return void
     */
    /* your send mail method */
    public function SendEmailOnWinnerConfirmation($emailTemplateVariables, $senderInfo, $receiverInfo) {
        $this->temp_id = $this->getTemplateId(self::WINNER_PATH_EMAIL_TEMPLATE_FIELD);
        $this->inlineTranslation->suspend();
        $this->generateTemplate($emailTemplateVariables, $senderInfo, $receiverInfo);
        $transport = $this->_transportBuilder->getTransport();
        $transport->sendMessage();
        $this->inlineTranslation->resume();
    }

    public function getEnrolledFrequency($order_base_total) {
        $freq = 0;
        $min_order_amount = $this->getLuckydrawAmount();
        $min_order_amount = $min_order_amount ? $min_order_amount : 0;
        if ($order_base_total > $min_order_amount) {
            $freq = floor($order_base_total / $min_order_amount);
        }
        return $freq;
    }

    public function getLuckydrawAmount() {
        $order_amount = $this->scopeConfig->getValue('luckydraw/general/order_amount', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        return $order_amount;
    }

    public function getRandomLuckyWinner() {
        $luckydrawCollection = $this->_objectManager->create('\Sapienttree\Luckydraw\Model\ResourceModel\Luckydraw\Collection');
        $start_date = $this->scopeConfig->getValue('luckydraw/general/start_date', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        $end_date = $this->scopeConfig->getValue('luckydraw/general/end_date', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if ($start_date && $end_date) {
            $luckydrawCollection->addFieldToFilter('created_at', array('gt' => $start_date));
            $luckydrawCollection->addFieldToFilter('created_at', array('lt' => $end_date));
        } elseif ($start_date && !$end_date) {
            $luckydrawCollection->addFieldToFilter('created_at', array('gt' => $start_date));
        } elseif (!$start_date && $end_date) {
            $luckydrawCollection->addFieldToFilter('created_at', array('lt' => $end_date));
        }
        $luckydrawCollection->getSelect()->orderRand()->limit(1);
        return $luckydrawCollection->getData();
    }

    public function getLuckyCusHtml() {
        $lucky_cus = $this->getRandomLuckyWinner();
        $info = $lucky_cus[0];
        $html = '<table class="luckydraw_winner_tbl">
                <tbody>';
        $html .= "<tr><td>" . __('Customer Name') . "</td><td>" . $info['customer_name'] . "</td></tr>";
        $html .= "<tr><td>" . __('Customer Email') . "</td><td>" . $info['customer_email'] . "</td></tr>";
        $html .= "<tr><td>" . __('Customer Phone') . "</td><td>" . $info['customer_phone'] . "</td></tr>";
        $html .= "<tr><td>" . __('Order Number') . "</td><td>" . $info['order_number'] . "</td></tr>";
        $html .= "<tr><td>" . __('Order Date') . "</td><td>" . date('Y-m-d', strtotime($info['created_at'])) . "</td></tr>";
        $html .= "<tr><td>" . __('Order Total') . "</td><td>" . $info['order_total'] . "</td></tr>";
        $html .= "<tr><td>" . __('Ticket') . "</td><td>" . $info['ticket'] . "</td></tr>";
        $html .= '</tbody><table>';
        $html .= '<br/><div class="confirm-winner">
                <button class="primary" data-id="' . $info['lucky_draw_id'] . '">' . __('Confirm Winner') . '</button>
            </div>';
        return $html;
    }

}