<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Catalog;

    class GetAllReviews extends AbstractCatalog   {

        public function execute()   {
            $returnArray                               = [];
            $returnArray["id"]                         = 0;
            $returnArray["reviewList"]                 = [];
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId      = $wholeData["storeId"]    ?? 1;
                        $productId    = $wholeData["productId"]  ?? 0;
                        $environment  = $this->_emulate->startEnvironmentEmulation($storeId);
// getting review list //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $returnArray["id"] = $productId;
                        $reviewList = [];
                        $reviewCollection = $this->_review
                            ->getResourceCollection()
                            ->addStoreFilter($storeId)
                            ->addEntityFilter("product", $productId)
                            ->addStatusFilter(\Magento\Review\Model\Review::STATUS_APPROVED)
                            ->setDateOrder()
                            ->addRateVotes();
                        $returnArray["reviewCount"] = $reviewCollection->getSize();
                        foreach ($reviewCollection as $review) {
                            $oneReview            = [];
                            $ratings              = [];
                            $oneReview["title"]   = $this->_helperCatalog->stripTags($review->getTitle());
                            $oneReview["details"] = $this->_helperCatalog->stripTags($review->getDetail());
                            $votes                = $review->getRatingVotes();
                            if (count($votes)) {
                                foreach ($votes as $_vote) {
                                    $oneVote          = [];
                                    $oneVote["label"] = $this->_helperCatalog->stripTags($_vote->getRatingCode());
                                    $oneVote["value"] = number_format($_vote->getValue(), 2, ".", "");
                                    $ratings[]        = $oneVote;
                                }
                            }
                            $oneReview["ratings"]  = $ratings;
                            $oneReview["reviewBy"] = __("Review by %1", $this->_helperCatalog->stripTags($review->getNickname()));
                            $oneReview["reviewOn"] = __("(Posted on %1)", $this->_helperCatalog->formatDate($review->getCreatedAt()), "long");
                            $reviewList[]          = $oneReview;
                        }

                        $returnArray["reviewList"] = $reviewList;
                        $returnArray["success"]       = true;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }
