<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Contact;

    class Post extends \Webkul\Mobikul\Controller\ApiController    {

        protected $_emulate;
        protected $_stateInterface;
        protected $_scopeInterface;
        protected $_transportBuilder;

        public function __construct(
            \Webkul\Mobikul\Helper\Data $helper,
            \Magento\Store\Model\App\Emulation $emulate,
            \Webkul\Mobikul\Helper\Catalog $helperCatalog,
            \Magento\Framework\App\Action\Context $context,
            \Magento\Framework\App\Config\ScopeConfigInterface $scopeInterface,
            \Magento\Framework\Translate\Inline\StateInterface $stateInterface,
            \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
        ) {
            $this->_emulate          = $emulate;
            $this->_stateInterface   = $stateInterface;
            $this->_scopeInterface   = $scopeInterface;
            $this->_transportBuilder = $transportBuilder;
            parent::__construct($helper, $context);
        }

        public function execute()   {
            $returnArray                 = [];
            $returnArray["success"]      = false;
            $returnArray["message"]      = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $name        = $wholeData["name"]      ?? "";
                        $email       = $wholeData["email"]     ?? "";
                        $comment     = $wholeData["comment"]   ?? "";
                        $storeId     = $wholeData["storeId"]   ?? 1;
                        $telephone   = $wholeData["telephone"] ?? "";
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $this->_stateInterface->suspend();
                        $postObject = new \Magento\Framework\DataObject();
                        $postObject->setData($wholeData);
                        $error      = false;
                        if (!\Zend_Validate::is(trim($wholeData["name"]), "NotEmpty"))
                            $error = true;
                        if (!\Zend_Validate::is(trim($wholeData["comment"]), "NotEmpty"))
                            $error = true;
                        if (!\Zend_Validate::is(trim($wholeData["email"]), "EmailAddress"))
                            $error = true;
                        if ($error)
                            throw new \Exception();
                        $storeScope  = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
                        $scopeConfig = $this->_scopeInterface;
                        $transport   = $this->_transportBuilder
                            ->setTemplateIdentifier($scopeConfig->getValue("contact/email/email_template", $storeScope))
                            ->setTemplateOptions([
                                "area"  => \Magento\Backend\App\Area\FrontNameResolver::AREA_CODE,
                                "store" => \Magento\Store\Model\Store::DEFAULT_STORE_ID,
                            ])
                            ->setTemplateVars(["data"=>$postObject])
                            ->setFrom($scopeConfig->getValue("contact/email/sender_email_identity", $storeScope))
                            ->addTo($scopeConfig->getValue("contact/email/recipient_email", $storeScope))
                            ->setReplyTo($email)
                            ->getTransport();
                        $transport->sendMessage();
                        $this->_stateInterface->resume();
                        $returnArray["message"] = __("Thanks for contacting us with your comments and questions. We'll respond to you very soon.");
                        $returnArray["success"] = true;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $this->_stateInterface->resume();
                $returnArray["message"] = __("We can't process your request right now. Sorry, that's all we know.");
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }