<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class RemoveFromWishlist extends AbstractCustomer   {

        public function execute()   {
            $returnArray                   = [];
            $returnArray["success"]        = false;
            $returnArray["message"]        = "";
            $returnArray["alreadyDeleted"] = false;
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $itemId      = $wholeData["itemId"]     ?? 0;
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $customerId  = $wholeData["customerId"] ?? 0;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $item        = $this->_wishlistItem->load($itemId);
                        if (!$item->getId()){
                            $returnArray["alreadyDeleted"] = true;
                            return $this->getJsonResponse($returnArray);
                        }
                        $error = false;
                        $wishlist = $this->_wishlistProvider->loadByCustomerId($customerId, true);
                        if (!$wishlist)
                            $error = true;
                        $item->delete();
                        $wishlist->save();
                        if ($error){
                            $returnArray["message"] = __("An error occurred while deleting the item from wishlist.");
                            return $this->getJsonResponse($returnArray);
                        }
                        $returnArray["success"] = true;
                        $returnArray["message"] = __("Item successfully deleted from wishlist.");
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }