<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;
    use Magento\Framework\App\ObjectManager;

    class SaveAccountInfo extends AbstractCustomer     {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["success"]      = false;
            $returnArray["message"]      = "";
            $returnArray["customerName"] = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $dob              = $wholeData["dob"]              ?? "";
                        $email            = $wholeData["email"]            ?? "";
                        $mobile           = $wholeData["mobile"]           ?? "";
                        $prefix           = $wholeData["prefix"]           ?? "";
                        $suffix           = $wholeData["suffix"]           ?? "";
                        $taxvat           = $wholeData["taxvat"]           ?? "";
                        $gender           = $wholeData["gender"]           ?? "";
                        $storeId          = $wholeData["storeId"]          ?? 1;
                        $lastName         = $wholeData["lastName"]         ?? "";
                        $firstName        = $wholeData["firstName"]        ?? "";
                        $customerId       = $wholeData["customerId"]       ?? 0;
                        $middleName       = $wholeData["middleName"]       ?? "";
                        $newPassword      = $wholeData["newPassword"]      ?? "";
                        $doChangeEmail    = $wholeData["doChangeEmail"]    ?? 0;
                        $confirmPassword  = $wholeData["confirmPassword"]  ?? "";
                        $currentPassword  = $wholeData["currentPassword"]  ?? "";
                        $doChangePassword = $wholeData["doChangePassword"] ?? 0;
                        $environment      = $this->_emulate->startEnvironmentEmulation($storeId);
                        $currentCustomerDataObject = $this->_customerRepositoryInterface->getById($customerId);
                        $inputData = [
                            "dob"                   => $dob,
                            "email"                 => $email,
                            "prefix"                => $prefix,
                            "suffix"                => $suffix,
                            "taxvat"                => $taxvat,
                            "gender"                => $gender,
                            "lastname"              => $lastName,
                            "firstname"             => $firstName,
                            "middlename"            => $middleName,
                            "password"              => $newPassword,
                            "password_confirmation" => $confirmPassword,
                            "current_password"      => $currentPassword
                        ];
                        $this->_request->setParams($inputData);
                        $storeManager   = $this->_storeManager;
                        $customerCheck  = $this->_customerFactory->create()->setWebsiteId($storeManager->getStore()->getWebsiteId())->loadByEmail($email);
                        $checkCustomerId = $customerCheck->getId();
                        if ($checkCustomerId > 0 && $checkCustomerId != $customerId) {
                            $returnArray["message"] = __("A customer with the same email already exists in an associated website.");
                            return $this->getJsonResponse($returnArray);
                        }
                        $inputData = $this->_request;
                        $customerCandidateDataObject = $this->populateNewCustomerDataObject(
                            $inputData,
                            $currentCustomerDataObject,
                            $doChangeEmail
                        );
                        if($doChangeEmail == 1){
                            try {
                                $this->getAuthentication()->authenticate($currentCustomerDataObject->getId(), $currentPassword);
                            } catch (InvalidEmailOrPasswordException $e) {
                                throw new InvalidEmailOrPasswordException(__("The password doesn't match this account."));
                            }
                            $this->_customerRepositoryInterface->save($customerCandidateDataObject);
                        }
                        $isPasswordChanged = false;
                        if($doChangePassword == 1){
                            if ($newPassword != $confirmPassword)
                                throw new InputException(__("Password confirmation doesn't match entered password."));
                            $isPasswordChanged = $this->_accountManagement->changePassword($email, $currentPassword, $newPassword);
                            $this->_customerRepositoryInterface->save($customerCandidateDataObject);
                        }
                        $this->_customerRepositoryInterface->save($customerCandidateDataObject);
                        $this->getEmailNotification()->credentialsChanged(
                            $customerCandidateDataObject,
                            $currentCustomerDataObject->getEmail(),
                            $isPasswordChanged
                        );
                        $customer = $this->_customerFactory->create()->load($customerId);
                        $returnArray["success"]      = true;
                        $returnArray["message"]      = __("You saved the account information.");
                        $returnArray["customerName"] = $customer->getName();
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (InvalidEmailOrPasswordException $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (UserLockedException $e) {
                $returnArray["message"] = __("The account is locked. Please wait and try again or contact %1.", $this->getScopeConfig()->getValue("contact/email/recipient_email"));
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (InputException $e) {
                $message = [];
                $message[] = $e->getMessage();
                foreach ($e->getErrors() as $error)
                    $message[] = $error->getMessage();
                $returnArray["message"] = implode(",", $message);
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (\Magento\Framework\Exception\LocalizedException $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            } catch (\Exception $e) {
                $returnArray["message"] = __("We can't save the customer.");
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

        private function populateNewCustomerDataObject(
            \Magento\Framework\App\RequestInterface $inputData,
            \Magento\Customer\Api\Data\CustomerInterface $currentCustomerData,
            $doChangeEmail
        ) {
            $attributeValues = $this->getCustomerMapper()->toFlatArray($currentCustomerData);
            $customerDto     = $this->_customerExtractor->extract(
                "customer_account_edit",
                $inputData,
                $attributeValues
            );
            $customerDto->setId($currentCustomerData->getId());
            if (!$customerDto->getAddresses()) {
                $customerDto->setAddresses($currentCustomerData->getAddresses());
            }
            if (!$doChangeEmail) {
                $customerDto->setEmail($currentCustomerData->getEmail());
            }
            return $customerDto;
        }

        private function getScopeConfig()   {
            if (!($this->_scopeConfig instanceof \Magento\Framework\App\Config\ScopeConfigInterface))
                return ObjectManager::getInstance()->get(\Magento\Framework\App\Config\ScopeConfigInterface::class);
            else
                return $this->_scopeConfig;
        }

        private function dispatchSuccessEvent(\Magento\Customer\Api\Data\CustomerInterface $customerCandidateDataObject)    {
            $this->_eventManager->dispatch("customer_account_edited", ["email"=>$customerCandidateDataObject->getEmail()]);
        }

        private function getCustomerMapper()    {
            if ($this->_customerMapper === null)
                $this->_customerMapper = ObjectManager::getInstance()->get("Magento\Customer\Model\Customer\Mapper");
            return $this->_customerMapper;
        }

        private function getAuthentication()    {
            if (!($this->_authentication instanceof \Magento\Customer\Model\AuthenticationInterface))
                return ObjectManager::getInstance()->get(\Magento\Customer\Model\AuthenticationInterface::class);
            else
                return $this->_authentication;
        }

        private function getEmailNotification()     {
            if (!($this->_emailNotification instanceof \Magento\Customer\Model\EmailNotificationInterface))
                return ObjectManager::getInstance()->get(\Magento\Customer\Model\EmailNotificationInterface::class);
            else
                return $this->_emailNotification;
        }

    }