<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Ui\Component\Listing\Columns;
    use Magento\Framework\UrlInterface;
    use Magento\Framework\View\Element\UiComponent\ContextInterface;
    use Magento\Framework\View\Element\UiComponentFactory;
    use Magento\Ui\Component\Listing\Columns\Column;

    /**
    * Class ViewAction.
    */
    class ViewAction extends Column     {
        /**
        * @var UrlInterface
        */
        protected $_urlBuilder;

        /**
        * Constructor.
        *
        * @param ContextInterface   $context
        * @param UiComponentFactory $uiComponentFactory
        * @param UrlInterface       $urlBuilder
        * @param array              $components
        * @param array              $data
        */
        public function __construct(
            ContextInterface $context,
            UiComponentFactory $uiComponentFactory,
            UrlInterface $urlBuilder,
            array $components = [],
            array $data = []) {
            $this->_urlBuilder = $urlBuilder;
            parent::__construct($context, $uiComponentFactory, $components, $data);
        }

        /**
        * Prepare Data Source.
        *
        * @param array $dataSource
        *
        * @return array
        */
        public function prepareDataSource(array $dataSource)    {
            if (isset($dataSource["data"]["items"])) {
                foreach ($dataSource["data"]["items"] as &$item) {
                    if (isset($item["id"])) {
                        $viewUrlPath = $this->getData("config/viewUrlPath") ? : "#";
                        $urlEntityParamName = $this->getData("config/urlEntityParamName") ? : "id";
                        $item[$this->getData("name")] = [
                            "view" => [
                                "href"  => $this->_urlBuilder->getUrl($viewUrlPath, [$urlEntityParamName=>$item["id"]]),
                                "label" => __("Edit")
                            ]
                        ];
                    }
                }
            }
            return $dataSource;
        }

    }