<?php


namespace Natz\AccountVerify\Model;

use Magento\Framework\Api\SortOrder;
use Magento\Framework\Exception\CouldNotSaveException;
use Natz\AccountVerify\Api\Data\SmslogsInterfaceFactory;
use Magento\Store\Model\StoreManagerInterface;
use Natz\AccountVerify\Api\SmslogsRepositoryInterface;
use Natz\AccountVerify\Api\Data\SmslogsSearchResultsInterfaceFactory;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Framework\Exception\CouldNotDeleteException;
use Natz\AccountVerify\Model\ResourceModel\Smslogs\CollectionFactory as SmslogsCollectionFactory;
use Natz\AccountVerify\Model\ResourceModel\Smslogs as ResourceSmslogs;
use Magento\Framework\Api\DataObjectHelper;

class SmslogsRepository implements smslogsRepositoryInterface
{

    protected $smslogsCollectionFactory;

    private $storeManager;

    protected $dataObjectHelper;

    protected $dataObjectProcessor;

    protected $searchResultsFactory;

    protected $smslogsFactory;

    protected $dataSmslogsFactory;

    protected $resource;


    /**
     * @param ResourceSmslogs $resource
     * @param SmslogsFactory $smslogsFactory
     * @param SmslogsInterfaceFactory $dataSmslogsFactory
     * @param SmslogsCollectionFactory $smslogsCollectionFactory
     * @param SmslogsSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ResourceSmslogs $resource,
        SmslogsFactory $smslogsFactory,
        SmslogsInterfaceFactory $dataSmslogsFactory,
        SmslogsCollectionFactory $smslogsCollectionFactory,
        SmslogsSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager
    ) {
        $this->resource = $resource;
        $this->smslogsFactory = $smslogsFactory;
        $this->smslogsCollectionFactory = $smslogsCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataSmslogsFactory = $dataSmslogsFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \Natz\AccountVerify\Api\Data\SmslogsInterface $smslogs
    ) {
        /* if (empty($smslogs->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $smslogs->setStoreId($storeId);
        } */
        try {
            $smslogs->getResource()->save($smslogs);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the smslogs: %1',
                $exception->getMessage()
            ));
        }
        return $smslogs;
    }

    /**
     * {@inheritdoc}
     */
    public function getById($smslogsId)
    {
        $smslogs = $this->smslogsFactory->create();
        $smslogs->getResource()->load($smslogs, $smslogsId);
        if (!$smslogs->getId()) {
            throw new NoSuchEntityException(__('smslogs with id "%1" does not exist.', $smslogsId));
        }
        return $smslogs;
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->smslogsCollectionFactory->create();
        foreach ($criteria->getFilterGroups() as $filterGroup) {
            foreach ($filterGroup->getFilters() as $filter) {
                if ($filter->getField() === 'store_id') {
                    $collection->addStoreFilter($filter->getValue(), false);
                    continue;
                }
                $condition = $filter->getConditionType() ?: 'eq';
                $collection->addFieldToFilter($filter->getField(), [$condition => $filter->getValue()]);
            }
        }
        
        $sortOrders = $criteria->getSortOrders();
        if ($sortOrders) {
            /** @var SortOrder $sortOrder */
            foreach ($sortOrders as $sortOrder) {
                $collection->addOrder(
                    $sortOrder->getField(),
                    ($sortOrder->getDirection() == SortOrder::SORT_ASC) ? 'ASC' : 'DESC'
                );
            }
        }
        $collection->setCurPage($criteria->getCurrentPage());
        $collection->setPageSize($criteria->getPageSize());
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        $searchResults->setTotalCount($collection->getSize());
        $searchResults->setItems($collection->getItems());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \Natz\AccountVerify\Api\Data\SmslogsInterface $smslogs
    ) {
        try {
            $smslogs->getResource()->delete($smslogs);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the smslogs: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($smslogsId)
    {
        return $this->delete($this->getById($smslogsId));
    }
}
