<?php

namespace Potato\PerformanceMonitoring\Block\Adminhtml\Form\Tab;

use Magento\Backend\Block\Template;
use Potato\PerformanceMonitoring\Model\System;
use Potato\PerformanceMonitoring\Model\Chart;
use Potato\PerformanceMonitoring\Model\ResourceModel\Cpu\CollectionFactory as CpuCollectionFactory;
use Magento\Backend\Block\Widget\Tab\TabInterface;

/**
 * Class Cpu
 */
class Cpu extends \Magento\Backend\Block\Widget implements TabInterface
{
    protected $_template = 'Potato_PerformanceMonitoring::tab_cpu.phtml';

    /** @var Chart  */
    protected $chart;
    
    /** @var System  */
    protected $system;

    /** @var CpuCollectionFactory  */
    protected $cpuCollectionFactory;

    /**
     * Cpu constructor.
     * @param Template\Context $context
     * @param System $system
     * @param Chart $chart
     * @param CpuCollectionFactory $cpuCollectionFactory
     * @param array $data
     */
    public function __construct(
        Template\Context $context,
        System $system,
        Chart $chart,
        CpuCollectionFactory $cpuCollectionFactory, 
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->chart = $chart;
        $this->system = $system;
        $this->cpuCollectionFactory = $cpuCollectionFactory;
    }

    /**
     * Memory Chart Data
     *
     * @return mixed
     */
    public function getDataJson()
    {
        $groupedDataFlag = false;
        if ($this->chart->canGroup()) {
            $groupedDataFlag = true;
        }
        $data = [
            [
                [
                    'label' => __('DateTime'),
                    'type' => 'datetime'
                ],
                [
                    'label' => __('Load Average'),
                    'type' => 'number'
                ]
            ]
        ];

        /** @var \Potato\PerformanceMonitoring\Model\ResourceModel\Cpu\Collection $collection */
        $collection = $this->cpuCollectionFactory->create();
        $collection->setDateFilter(
            $this->chart->getDateFrom(),
            $this->chart->getDateTo()
        )->orderByDate();
        if ($groupedDataFlag) {
            $collection->groupByHour();
            $data = [
                [
                    [
                        'label' => __('DateTime'),
                        'type' => 'datetime'
                    ],
                    [
                        'label' => __('(Max) Load Average'),
                        'type' => 'number'
                    ],
                    [
                        'label' => __('(Avg) Load Average'),
                        'type' => 'number'
                    ]
                ]
            ];
        }
        $zendJson = new \Zend_Json();
        if (!count($collection->getItems())) {
            return $zendJson::encode([], false, ['enableJsonExprFinder' => true]);
        }
        foreach ($collection as $row) {
            if ($groupedDataFlag) {
                $data[] = [
                    $this->chart->formatDateToJs($row->getCreatedAt()),
                    (float)$row->getMax(),
                    (float)$row->getAvg(),
                ];
                continue;
            }
            $data[] = [
                $this->chart->formatDateToJs($row->getCreatedAt()),
                (float)$row->getValue()
            ];
        }
        return $zendJson::encode($data, false, ['enableJsonExprFinder' => true]);
    }

    /**
     * Top Website Transactions Chart Data
     *
     * @return mixed
     */
    public function getWebTransactionDataJson()
    {
        /** @var \Potato\PerformanceMonitoring\Model\ResourceModel\TransactionWeb\Collection $collection */
        $collection = $this->chart->getWebTransactionCollection();
        $collection->setMaxCpuFilter()
            ->setOrderByCpu()
        ;
        $data = $collection->toChartArray('cpu');
        return json_encode($data);
    }

    /**
     * System Transactions Chart Data
     *
     * @return mixed
     */
    public function getTransactionDataJson()
    {
        $result = $this->chart->getSystemTransactionsChartData('cpu');
        return $result;
    }
    
    public function getAcceptableLoadAverage()
    {
        return $this->system->getAcceptableLoadAverage();
    }

    /**
     * Tab settings
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabLabel()
    {
        return __('CPU Usage');
    }

    /**
     * @return \Magento\Framework\Phrase
     */
    public function getTabTitle()
    {
        return __('CPU Usage');
    }

    /**
     * @return bool
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return false;
    }
}