<?php

namespace Potato\PerformanceMonitoring\Block\Adminhtml\Form\Tab;

use Magento\Backend\Block\Template;
use Potato\PerformanceMonitoring\Model\System;
use Magento\Backend\Block\Widget\Tab\TabInterface;

/**
 * Class Overview
 */
class Overview extends \Magento\Backend\Block\Widget implements TabInterface
{
    protected $_template = 'Potato_PerformanceMonitoring::tab_overview.phtml';
    
    protected $memory = null;
    protected $swap = null;
    protected $os = null;
    protected $cpuModel = null;
    protected $hdu = null;
    protected $cores = null;

    /** @var System  */
    protected $system;

    /**
     * Overview constructor.
     * @param Template\Context $context
     * @param System $system
     * @param array $data
     */
    public function __construct(
        Template\Context $context,
        System $system,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->system = $system;
    }
    
    protected function _construct()
    {
        $this->initMemoryInfo();
        parent::_construct();
    }

    /**
     * Init memory Info
     *
     * @return $this
     */
    protected function initMemoryInfo()
    {
        $result = [];
        $status = [];
        exec('free -m 2>&1', $result, $status);
        if ($status != 0) {
            $this->memory = __('Error code %s: %s', $status, implode(' ', $result));
        } else {
            $memoryLine = array_values(
                array_filter(
                    explode(" ", $result[1]),
                    function($value) {
                        return $value !== '';
                    }
                )
            );
            $this->memory = [
                'total' => $memoryLine[1],
                'used'  => $memoryLine[2],
                'free'  => $memoryLine[1] - $memoryLine[2]
            ];
            $swapLine = array_values(
                array_filter(
                    explode(" ", $result[2]),
                    function($value) {
                        return $value !== '';
                    }
                )
            );
            if ($swapLine[1] == 0) {
                $this->swap = __('Not used');
            } else {
                $this->swap = [
                    'total' => $swapLine[1],
                    'used'  => $swapLine[2],
                    'free'  => $swapLine[3]
                ];
            }
        }
        return $this;
    }

    /**
     * @return array
     */
    public function getLoadAvg()
    {
        return sys_getloadavg();
    }

    /**
     * @return null
     */
    public function getMemoryInfo()
    {
        return $this->memory;
    }

    /**
     * @return null
     */
    public function getSwapInfo()
    {
        return $this->swap;
    }

    /**
     * @return \Magento\Framework\Phrase|null
     */
    public function getCpuModel()
    {
        if (null === $this->cpuModel) {
            $result = [];
            $status = [];
            exec('less /proc/cpuinfo 2>&1', $result, $status);
            if ($status != 0) {
                $this->cpuModel = __('Error code %s: %s', $status, implode(' ', $result));
            } else {
                $modelLine = array_values(
                    array_filter(
                        explode(":", $result[4]),
                        function($value) {
                            return $value !== '';
                        }
                    )
                );
                $this->cpuModel = $modelLine[1];
            }
        }
        return $this->cpuModel;
    }

    /**
     * @return \Magento\Framework\Phrase|mixed|null
     */
    public function getOsInfo()
    {
        if (null === $this->os) {
            $result = [];
            $status = [];
            exec('uname -a 2>&1', $result, $status);
            if ($status != 0) {
                $this->os = __('Error code %s: %s', $status, implode(' ', $result));
            } else {
                $this->os = $result[0];
            }
        }
        return $this->os;
    }

    /**
     * @return array|\Magento\Framework\Phrase|null
     */
    public function getHduInfo()
    {
        if (null === $this->hdu) {
            $result = [];
            $status = [];
            exec('df -h 2>&1', $result, $status);
            if ($status != 0) {
                $this->hdu = __('Error code %s: %s', $status, implode(' ', $result));
            } else {
                $info = [
                    'header' => [],
                    'body'   => []
                ];
                foreach ($result as $key => $_line) {
                    $line = array_values(array_diff(explode(" ", $_line), ['']));
                    if ($key == 0) {
                        $info['header'][0] = $line[0];
                        $info['header'][1] = $line[1];
                        $info['header'][2] = $line[2];
                        $info['header'][3] = $line[3];
                        $info['header'][4] = $line[4];
                        $info['header'][5] = $line[5] . ' ' . $line[6];
                        continue;
                    }
                    $info['body'][] = [
                        $line[0], $line[1], $line[2], $line[3], $line[4], $line[5]
                    ];
                }
                $this->hdu = $info;
            }
        }
        return $this->hdu;
    }

    /**
     * @return \Magento\Framework\Phrase|mixed
     */
    public function getCpuCoresNumber()
    {
        return $this->system->getCpuCoresNumber();
    }

    /**
     * Tab settings
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabLabel()
    {
        return __('Overview');
    }

    /**
     * @return \Magento\Framework\Phrase
     */
    public function getTabTitle()
    {
        return __('Overview');
    }

    /**
     * @return bool
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return false;
    }
}