<?php

namespace Potato\PerformanceMonitoring\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Config
 */
class Config
{
    const GENERAL_ENABLED                = 'potato_performance_monitoring/general/is_enabled';
    const GENERAL_WEB_TRANSACTIONS_LIMIT = 'potato_performance_monitoring/general/web_transactions_limit';
    const GENERAL_HISTORY_LIFETIME       = 'potato_performance_monitoring/general/data_lifetime';
    const GENERAL_APPLICATION            = 'potato_performance_monitoring/general/applications';
    const ALERT_ENABLED                  = 'potato_performance_monitoring/alert/is_enabled';
    const ALERT_CONDITION                = 'potato_performance_monitoring/alert/condition';
    const ALERT_SENDER                   = 'potato_performance_monitoring/alert/sender';
    const ALERT_RECIPIENT                = 'potato_performance_monitoring/alert/recipient';
    const ALERT_TEMPLATE                 = 'potato_performance_monitoring/alert/template';
    const ALERT_BCC                      = 'potato_performance_monitoring/alert/bcc';
    const ALERT_FREQUENCY                = 'potato_performance_monitoring/alert/frequency';

    /** @var ScopeConfigInterface  */
    protected $scopeConfig;

    /** @var StoreManagerInterface  */
    protected $storeManager;
    
    /**
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager
    ) {
        $this->scopeConfig = $scopeConfig;
        $this->storeManager = $storeManager;
    }

    /**
     * @return array
     */
    public function getUnusedApplications()
    {
        $value = $this->scopeConfig->getValue(self::GENERAL_APPLICATION);
        $result = [];
        if (null !== $value && false !== $value) {
            $result = explode(',', $value);
        }
        return $result;
    }

    /**
     * @return int
     */
    public function getAlertFrequency()
    {
        return (int)$this->scopeConfig->getValue(self::ALERT_FREQUENCY) * 60;
    }

    /**
     * @return bool
     */
    public function isEnabled()
    {
        return (bool)$this->scopeConfig->getValue(self::GENERAL_ENABLED);
    }

    /**
     * @return int
     */
    public function getWebTransactionsLimit()
    {
        return (int)$this->scopeConfig->getValue(self::GENERAL_WEB_TRANSACTIONS_LIMIT);
    }

    /**
     * @return int
     */
    public function getHistoryLifetime()
    {
        return (int)$this->scopeConfig->getValue(self::GENERAL_WEB_TRANSACTIONS_LIMIT);
    }

    /**
     * @return array
     */
    public function getBCC()
    {
        $sendTo = trim($this->scopeConfig->getValue(self::ALERT_BCC));
        if ($sendTo) {
            return explode(',', $sendTo);
        }
        return [];
    }

    /**
     * @return string
     */
    public function getEmailSender()
    {
        return (string)$this->scopeConfig->getValue(self::ALERT_SENDER);
    }

    /**
     * @return int
     */
    public function getEmailRecipient()
    {
        return (int)$this->scopeConfig->getValue(self::ALERT_RECIPIENT);
    }

    /**
     * @return mixed
     */
    public function getEmailTemplate()
    {
        return $this->scopeConfig->getValue(self::ALERT_TEMPLATE);
    }

    /**
     * @return bool
     */
    public function canUseAlert()
    {
        return (bool)$this->scopeConfig->getValue(self::ALERT_ENABLED);
    }

    /**
     * @return float
     */
    public function getAlertCondition()
    {
        return (float)$this->scopeConfig->getValue(self::ALERT_CONDITION);
    }

    /**
     * @return bool
     */
    public function isWin()
    {
        if (stristr(PHP_OS, 'win')) {
            return true;
        }
        return false;
    }

    /**
     * @return bool
     */
    public function enabledExec()
    {
        return function_exists('exec');
    }

    /**
     * @return bool
     */
    public function canUseModule()
    {
        return $this->enabledExec() && !$this->isWin() && $this->isEnabled();
    }

    /**
     * @return string
     */
    public function getTopStatVariable()
    {
        $result = [];
        $status= [];
        exec('top -b -n1 2>&1', $result, $status);

        if ($status != 0) {
            return implode(" ", $result);
        }
        $top = '<table style="border-collapse: collapse;width: 100%;"><tr><td style="text-align: left; padding: 8px;">';
        $i = 0;
        foreach ($result as $row) {
            $rowCss = '';
            if ($i++%2 == 0) {
                $rowCss = 'style="background-color: #f2f2f2"';
            }
            $top .= '<tr ' . $rowCss . '><td style="text-align: left; padding: 8px;">' . $row . '</td></tr>';
        }
        $top .= '</table>';
        return $top;
    }
}