<?php

namespace Potato\PerformanceMonitoring\Model;

use Magento\Framework\Exception\NoSuchEntityException;
use Potato\PerformanceMonitoring\Api\Data\CpuInterfaceFactory;

/**
 * Class Cpu
 */
class CpuRegistry
{
    /** @var CpuFactory  */
    private $cpuFactory;

    /** @var array  */
    private $cpuRegistryById = [];

    /** @var ResourceModel\Cpu  */
    private $cpuResource;

    /** @var CpuInterfaceFactory  */
    private $dataFactory;

    /**
     * CpuRegistry constructor.
     * @param CpuFactory $cpuFactory
     * @param ResourceModel\Cpu $cpuResource
     * @param CpuInterfaceFactory $dataFactory
     */
    public function __construct(
        CpuFactory $cpuFactory,
        ResourceModel\Cpu $cpuResource,
        CpuInterfaceFactory $dataFactory
    ) {
        $this->cpuResource = $cpuResource;
        $this->cpuFactory = $cpuFactory;
        $this->dataFactory = $dataFactory;
    }

    /**
     * @param int $cpuId
     * @return Cpu
     * @throws NoSuchEntityException
     */
    public function retrieve($cpuId)
    {
        if (!isset($this->cpuRegistryById[$cpuId])) {
            /** @var Cpu $cpu */
            $cpu = $this->cpuFactory->create();
            $this->cpuResource->load($cpu, $cpuId);
            if (!$cpu->getId()) {
                throw NoSuchEntityException::singleField('cpuId', $cpuId);
            } else {
                $this->cpuRegistryById[$cpuId] = $cpu;
            }
        }
        return $this->cpuRegistryById[$cpuId];
    }

    /**
     * @param int $cpuId
     * @return void
     */
    public function remove($cpuId)
    {
        if (isset($this->cpuRegistryById[$cpuId])) {
            unset($this->cpuRegistryById[$cpuId]);
        }
    }

    /**
     * @param Cpu $cpu
     * @return $this
     */
    public function push(Cpu $cpu)
    {
        $this->cpuRegistryById[$cpu->getId()] = $cpu;
        return $this;
    }

    /**
     * @return \Potato\PerformanceMonitoring\Api\Data\CpuInterface
     */
    public function create()
    {
        return $this->dataFactory->create();
    }
}
