<?php

namespace Potato\PerformanceMonitoring\Model;

use Magento\Framework\Exception\NoSuchEntityException;
use Potato\PerformanceMonitoring\Api\Data\MemoryInterfaceFactory;

/**
 * Class Memory
 */
class MemoryRegistry
{
    /** @var MemoryFactory  */
    private $memoryFactory;

    /** @var array  */
    private $memoryRegistryById = [];

    /** @var ResourceModel\Memory  */
    private $memoryResource;

    /** @var MemoryInterfaceFactory  */
    private $dataFactory;

    /**
     * MemoryRegistry constructor.
     * @param MemoryFactory $memoryFactory
     * @param ResourceModel\Memory $memoryResource
     * @param MemoryInterfaceFactory $dataFactory
     */
    public function __construct(
        MemoryFactory $memoryFactory,
        ResourceModel\Memory $memoryResource,
        MemoryInterfaceFactory $dataFactory
    ) {
        $this->memoryResource = $memoryResource;
        $this->memoryFactory = $memoryFactory;
        $this->dataFactory = $dataFactory;
    }

    /**
     * @param int $memoryId
     * @return Memory
     * @throws NoSuchEntityException
     */
    public function retrieve($memoryId)
    {
        if (!isset($this->memoryRegistryById[$memoryId])) {
            /** @var Memory $memory */
            $memory = $this->memoryFactory->create();
            $this->memoryResource->load($memory, $memoryId);
            if (!$memory->getId()) {
                throw NoSuchEntityException::singleField('memoryId', $memoryId);
            } else {
                $this->memoryRegistryById[$memoryId] = $memory;
            }
        }
        return $this->memoryRegistryById[$memoryId];
    }

    /**
     * @param int $memoryId
     * @return void
     */
    public function remove($memoryId)
    {
        if (isset($this->memoryRegistryById[$memoryId])) {
            unset($this->memoryRegistryById[$memoryId]);
        }
    }

    /**
     * @param Memory $memory
     * @return $this
     */
    public function push(Memory $memory)
    {
        $this->memoryRegistryById[$memory->getId()] = $memory;
        return $this;
    }

    /**
     * @return \Potato\PerformanceMonitoring\Api\Data\MemoryInterface
     */
    public function create()
    {
        return $this->dataFactory->create();
    }
}
