<?php

namespace Potato\PerformanceMonitoring\Model\ResourceModel\TransactionWeb;

use Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection;
use Potato\PerformanceMonitoring\Model;

/**
 * Class Collection
 */
class Collection extends AbstractCollection
{
    /**
     * Init collection and determine table names
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(
            Model\TransactionWeb::class,
            Model\ResourceModel\TransactionWeb::class
        );
    }

    /**
     * @param string $from
     * @param string|null $to
     * @return $this
     */
    public function setDateFilter($from, $to = null)
    {
        $this->addFieldToFilter('created_at', ['gt' => $from]);
        if ($to) {
            $this->addFieldToFilter('created_at', ['lt' => $to]);
        }
        return $this;
    }

    /**
     * @param int $pid
     * @return $this
     */
    public function addFilterByPid($pid)
    {
        return $this->addFieldToFilter('pid', $pid);
    }

    /**
     * @return $this
     */
    public function setMaxCpuFilter()
    {
        $this->getSelect()->columns(['cpu' => 'MAX(main_table.cpu)']);
        $this->getSelect()->group(['url']);
        return $this;
    }

    /**
     * @return $this
     */
    public function setMaxMemoryFilter()
    {
        $this->getSelect()->columns(['memory' => 'MAX(main_table.memory)']);
        $this->getSelect()->group(['url']);
        return $this;
    }

    /**
     * @return $this
     */
    public function setOrderByCpu()
    {
        $this->getSelect()->order('cpu ' . \Zend_Db_Select::SQL_DESC);
        return $this;
    }

    /**
     * @return $this
     */
    public function setOrderByMemory()
    {
        $this->getSelect()->order('memory ' . \Zend_Db_Select::SQL_DESC);
        return $this;
    }

    /**
     * @param int $limit
     * @return $this
     */
    public function setLimit($limit)
    {
        $this->getSelect()->limit($limit);
        return $this;
    }

    /**
     * @param string $fieldName
     * @return array
     */
    public function toChartArray($fieldName)
    {
        $data = [];
        $result = $this->toArray();
        foreach ($result['items'] as $row) {
            $data[] = [
                "<a title='" . $row['url'] . "' href='" . $row['url'] . "'>" . $row['url'] . "</a>",
                (float)$row[$fieldName],
                (float)$row['time'] / 1000
            ];
        }
        return $data;
    }
}
