<?php
namespace Potato\PerformanceMonitoring\Model\Source;

use Magento\Framework\Data\OptionSourceInterface;
use Magento\User\Model\ResourceModel\User\CollectionFactory as UserCollectionFactory;
use Magento\User\Model\ResourceModel\User as UserResourceModel;
use Magento\User\Model\UserFactory;

/**
 * Class Recipient
 */
class Recipient implements OptionSourceInterface
{
    /** @var UserCollectionFactory  */
    protected $userCollectionFactory;
    
    /** @var UserResourceModel  */
    protected $userResourceModel;
    
    /** @var UserFactory  */
    protected $userFactory;
    
    /**
     * Recipient constructor.
     * @param UserCollectionFactory $userCollectionFactory
     * @param UserResourceModel $userResourceModel
     * @param UserFactory $userFactory
     */
    public function __construct(
        UserCollectionFactory $userCollectionFactory,
        UserResourceModel $userResourceModel,
        UserFactory $userFactory
    ) {
        $this->userCollectionFactory = $userCollectionFactory;
        $this->userResourceModel = $userResourceModel;
        $this->userFactory = $userFactory;
    }

    /**
     * @return array
     */
    public function toOptionArray()
    {
        /** @var \Magento\User\Model\ResourceModel\User\Collection $collection */
        $collection = $this->userCollectionFactory->create();
        $options = [];
        foreach ($collection as $user) {
            $options[] = [
                'value' => $user->getId(),
                'label' => $user->getName() . '<' . $user->getEmail() . '>'
            ];
        }
        return $options;
    }

    /**
     * @param int $id
     * @return \Magento\User\Model\User|null
     */
    public function getUserByOption($id)
    {
        $user = $this->userFactory->create();
        $this->userResourceModel->load($user, $id);
        return $user;
    }
}