<?php

namespace Potato\PerformanceMonitoring\Model;

use Potato\PerformanceMonitoring\Api\Data;
use Magento\Framework;
use Magento\Framework\Stdlib\DateTime\DateTime;

/**
 * Class TransactionWeb
 */
class TransactionWeb extends Framework\Model\AbstractModel
{
    /** @var Data\TransactionWebInterfaceFactory  */
    private $transactionWebDataFactory;

    /** @var Framework\Api\DataObjectHelper  */
    private $dataObjectHelper;

    /** @var DateTime  */
    protected $date;

    /**
     * TransactionWeb constructor.
     * @param Framework\Model\Context $context
     * @param Framework\Registry $registry
     * @param ResourceModel\TransactionWeb $resource
     * @param ResourceModel\TransactionWeb\Collection $resourceCollection
     * @param Data\TransactionWebInterfaceFactory $transactionWebDataFactory
     * @param Framework\Api\DataObjectHelper $dataObjectHelper
     * @param array $data
     */
    public function __construct(
        Framework\Model\Context $context,
        Framework\Registry $registry,
        ResourceModel\TransactionWeb $resource,
        ResourceModel\TransactionWeb\Collection $resourceCollection,
        Data\TransactionWebInterfaceFactory $transactionWebDataFactory,
        Framework\Api\DataObjectHelper $dataObjectHelper,
        DateTime $dateTime,
        array $data = []
    ) {
        $this->transactionWebDataFactory = $transactionWebDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->date = $dateTime;
        parent::__construct(
            $context,
            $registry,
            $resource,
            $resourceCollection,
            $data
        );
    }

    /**
     * Initialize resource mode
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(ResourceModel\TransactionWeb::class);
    }

    /**
     * Retrieve TransactionWeb model with data
     *
     * @return \Potato\PerformanceMonitoring\Api\Data\TransactionWebInterface
     */
    public function getDataModel()
    {
        $data = $this->getData();
        $dataObject = $this->transactionWebDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $dataObject,
            $data,
            Data\TransactionWebInterface::class
        );
        return $dataObject;
    }

    /**
     * @return $this
     */
    public function beforeSave()
    {
        //set current date before save
        $this->setCreatedAt($this->date->gmtDate());
        return parent::beforeSave();
    }
}
