<?php

namespace Potato\PerformanceMonitoring\Observer;

use Magento\Framework\Event\ObserverInterface;

use Potato\PerformanceMonitoring\Model\Config;
use Potato\PerformanceMonitoring\Api\TransactionWebRepositoryInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\UrlInterface;

/**
 * Class TrackWeb
 */
class TrackWeb implements ObserverInterface
{
    /** @var Config  */
    protected $config;

    /** @var TransactionWebRepositoryInterface  */
    protected $transactionWebRepository;

    /** @var LoggerInterface  */
    protected $logger;

    /** @var UrlInterface  */
    protected $urlBuilder;

    /**
     * TrackWeb constructor.
     * @param Config $config
     * @param TransactionWebRepositoryInterface $transactionWebRepository
     * @param LoggerInterface $logger
     * @param UrlInterface $urlBuilder
     */
    public function __construct(
        Config $config,
        TransactionWebRepositoryInterface $transactionWebRepository,
        LoggerInterface $logger,
        UrlInterface $urlBuilder
    ) {
        $this->config = $config;
        $this->transactionWebRepository = $transactionWebRepository;
        $this->logger = $logger;
        $this->urlBuilder = $urlBuilder;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if (!$this->config->canUseModule()) {
            return $this;
        }
        try {
            $result = [];
            $status= [];
            exec('ps -o %cpu,%mem,time -p ' . getmypid() . ' 2>&1', $result, $status);
            if ($status != 0) {
                return $this;
            }
            $line = array_values(array_diff(explode(" ", $result[1]), ['']));
            $executeTime = strtotime($line[2]) - strtotime('TODAY');

            $transaction = $this->transactionWebRepository->create();
            $transaction
                ->setPid(getmypid())
                ->setCpu($line[0])
                ->setMemory($line[1])
                ->setTime($executeTime)
                ->setCreatedAt(gmdate('Y-m-d H:i:s'))
                ->setUrl($this->urlBuilder->getCurrentUrl())
            ;
            $this->transactionWebRepository->save($transaction);
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
        }
        return $this;
    }
}
