<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_Reports
 */

namespace Amasty\Reports\Helper;

use Magento\Framework\App\Helper\AbstractHelper;

class Data extends AbstractHelper
{

    /**
     * @var \Magento\Backend\Model\Session
     */
    private $session;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\TimezoneInterface
     */
    private $localeDate;

    /**
     * Wrapped component
     *
     * @var \Magento\Ui\Component\Form\Element\DataType\Date
     */
    private $wrappedComponent;

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Backend\Model\Session $session,
        \Magento\Framework\Stdlib\DateTime\TimezoneInterface $localeDate,
        \Magento\Ui\Component\Form\Element\DataType\Date $wrappedComponent,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
    ) {
        parent::__construct($context);
        $this->session = $session;
        $this->localeDate = $localeDate;
        $this->wrappedComponent = $wrappedComponent;
        $this->scopeConfig = $scopeConfig;
    }

    public function getDefaultFromDate()
    {
        return strtotime('-7 day');
    }

    public function getDefaultToDate()
    {
        return time();
    }

    public function getCurrentStoreId()
    {
        return $this->session->getAmreportsStore();
    }

    public function setCurrentStore($store)
    {
        return $this->session->setAmreportsStore($store);
    }

    public function convertTime($date)
    {
        $dateForLocale = $this->getDateForLocale($date);

        return $dateForLocale->format('Y-m-d H:i:s');
    }

    public function addTimeForDay($date)
    {
        $dateForLocale = $this->getDateForLocale($date);

        return $dateForLocale->add(new \DateInterval('PT23H59M59S'))->format('Y-m-d H:i:s');
    }

    public function getDateForLocale($date)
    {
        $skipTimeZoneConversion = $this->scopeConfig->getValue(
            'config/skipTimeZoneConversion',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );

        return $this->wrappedComponent->convertDate(
            $date,
            0,
            0,
            0,
            !$skipTimeZoneConversion
        );
    }
}
