<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_Xlanding
 */


namespace Amasty\Xlanding\Setup;

use Magento\Framework\Setup\UpgradeSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\DB\Adapter\AdapterInterface;


class UpgradeSchema implements UpgradeSchemaInterface
{
    /**
     * @var \Magento\Store\Model\System\Store
     */
    private $storeManager;

    /**
     * @var \Amasty\Base\Model\Serializer
     */
    private $serializer;

    public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Amasty\Base\Model\Serializer $serializer
    ) {
        $this->storeManager = $storeManager;
        $this->serializer = $serializer;
    }

    /**
     * @param SchemaSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function upgrade(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $setup->startSetup();

        if (version_compare($context->getVersion(), '1.1.5', '<')) {
            $this->removeUrlKeyUniqueness($setup);
        }

        if (version_compare($context->getVersion(), '1.2.8', '<')) {
            $this->addIndexFields($setup);
        }

        if (version_compare($context->getVersion(), '1.3.0', '<')) {
            $this->changeMetaData($setup);
        }

        $setup->endSetup();
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function removeUrlKeyUniqueness(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $connection = $installer->getConnection();
        $connection->dropIndex(
            $setup->getTable('amasty_xlanding_page'),
            $installer->getIdxName(
                $installer->getTable('amasty_xlanding_page'),
                ['identifier'],
                AdapterInterface::INDEX_TYPE_UNIQUE
            )
        );
    }

    private function addIndexFields(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $connection = $installer->getConnection();
        $connection->addIndex(
            $installer->getTable('amasty_xlanding_page'),
            $setup->getIdxName(
                $installer->getTable('amasty_xlanding_page'),
                ['layout_bottom_description', 'layout_top_description'],
                AdapterInterface::INDEX_TYPE_FULLTEXT
            ),
            ['layout_bottom_description', 'layout_top_description'],
            AdapterInterface::INDEX_TYPE_FULLTEXT
        );
    }

    /**
     * @param SchemaSetupInterface $setup
     */
    protected function changeMetaData(SchemaSetupInterface $setup)
    {
        $installer = $setup;
        $connection = $installer->getConnection();
        $table = $setup->getTable('amasty_xlanding_page');

        $select = $connection->select()->from($table, ['page_id', 'meta_title', 'meta_keywords', 'meta_description']);

        $connection->addColumn(
            $setup->getTable('amasty_xlanding_page'),
            'meta_data',
            [
                'type' => \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                'nullable' => true,
                'comment' => 'Meta data'
            ]
        );

        foreach ($connection->fetchAll($select) as $page) {
            $metaData = [];
            foreach ($this->storeManager->getStores() as $store) {
                $metaData[$store->getId()] = [
                    'meta_title_' . $store->getId() => $page['meta_title'],
                    'meta_keywords_' . $store->getId() => $page['meta_keywords'],
                    'meta_description_' . $store->getId() => $page['meta_description']
                ];
            }
            $metaData = $this->serializer->serialize($metaData);
            $sql = 'UPDATE ' . $table . ' SET `meta_data` = :metaData WHERE `page_id` = :pageId;';
            $setup->getConnection()->query($sql, [
                'metaData' => $metaData,
                'pageId' => $page['page_id']
            ]);
        }

        $connection->dropColumn($setup->getTable('amasty_xlanding_page'), 'meta_title');
        $connection->dropColumn($setup->getTable('amasty_xlanding_page'), 'meta_keywords');
        $connection->dropColumn($setup->getTable('amasty_xlanding_page'), 'meta_description');
    }
}
