<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2018 Amasty (https://www.amasty.com)
 * @package Amasty_Reports
 */


namespace Amasty\Reports\Model\ResourceModel\Sales\Weekday;


//  use Magento\Framework\App\Bootstrap;
//  include 'app/bootstrap.php';

use Amasty\Reports\Traits\Filters;

class Collection extends \Magento\Sales\Model\ResourceModel\Order\Collection
{
    use Filters;

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $request;
    /**
     * @var \Amasty\Reports\Helper\Data
     */
    protected $helper;

    /**
     * Collection constructor.
     *
     * @param \Magento\Framework\Data\Collection\EntityFactory                  $entityFactory
     * @param \Psr\Log\LoggerInterface                                          $logger
     * @param \Magento\Framework\Data\Collection\Db\FetchStrategyInterface      $fetchStrategy
     * @param \Magento\Framework\Event\ManagerInterface                         $eventManager
     * @param \Magento\Framework\Model\ResourceModel\Db\VersionControl\Snapshot $entitySnapshot
     * @param \Magento\Framework\DB\Helper                                      $coreResourceHelper
     * @param \Magento\Framework\App\RequestInterface                           $request
     * @param \Magento\Framework\DB\Adapter\AdapterInterface|null               $connection
     * @param \Magento\Framework\Model\ResourceModel\Db\AbstractDb|null         $resource
     */
    public function __construct(
        \Magento\Framework\Data\Collection\EntityFactory $entityFactory,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Data\Collection\Db\FetchStrategyInterface $fetchStrategy,
        \Magento\Framework\Event\ManagerInterface $eventManager,
        \Magento\Framework\Model\ResourceModel\Db\VersionControl\Snapshot $entitySnapshot,
        \Magento\Framework\DB\Helper $coreResourceHelper,
        \Magento\Framework\App\RequestInterface $request, // TODO move it out of here
        \Amasty\Reports\Helper\Data $helper,
        \Magento\Framework\DB\Adapter\AdapterInterface $connection = null,
        \Magento\Framework\Model\ResourceModel\Db\AbstractDb $resource = null
    ) {
        parent::__construct(
            $entityFactory,
            $logger,
            $fetchStrategy,
            $eventManager,
            $entitySnapshot,
            $coreResourceHelper,
            $connection,
            $resource
        );
        $this->request = $request;
        $this->helper = $helper;
    }

    public function prepareCollection($collection)
    {
        $this->applyBaseFilters($collection);
        $this->applyToolbarFilters($collection);
        // print_r((string)$collection->getSelect());exit;
    }

    public function applyBaseFilters($collection)
    {
        $collection->getSelect()
            ->reset(\Zend_Db_Select::COLUMNS)
            ->columns([
                'period' =>'DAYNAME(main_table.created_at)',
                'total_orders' => 'COUNT(main_table.entity_id)',
                'total_items' => 'SUM(main_table.total_item_count)',
                'subtotal' => 'SUM(main_table.base_subtotal)',
                'tax' => 'SUM(main_table.base_tax_amount)',
                'status' => 'main_table.status',
                'shipping' => 'SUM(main_table.base_shipping_amount)',
                'discounts' => 'SUM(main_table.base_discount_amount)',
                'total' => 'SUM(main_table.base_grand_total)',
                'invoiced' => 'SUM(main_table.base_total_invoiced)',
                'refunded' => 'SUM(main_table.base_total_refunded)',
                'entity_id' => 'CONCAT(main_table.entity_id,\''.$this->createUniqueEntity().'\')'
            ])
        ;

        $collection->getSelect()
            ->joinLeft(
                ['status' => $this->getTable('sales_order_status_history')],
                'status.parent_id = main_table.entity_id'
            );
        $filters = $this->request->getParam('amreports');
        $date = isset($filters['to']) ? $filters['to'] : date('Y-m-d');
        $to = $this->helper->getDateForLocale($date);
        $from = $this->helper->getDateForLocale($date, 23, 59, 59);
        $collection->getSelect()->where('status.created_at > ?', $to);
        $collection->getSelect()->where('status.created_at < ?', $from);
        $collection->getSelect()->where('status.status = ?', 'processing_closed');
        
        
        
        //  $collection->addFieldToFilter('status', 'processing_closed');
        
        
        //  echo "<pre>";
        // //  var_dump($collection);
        // print_r($collection);
        // echo "</pre>";
       
       
// $bootstrap = Bootstrap::create(BP, $_SERVER);
// $objectManager = $bootstrap->getObjectManager();

// $state = $objectManager->get('Magento\Framework\App\State');
// $state->setAreaCode('frontend');


// $factoryCollection = $objectManager->get('\Magento\Sales\Model\ResourceModel\Order\CollectionFactory');
// $collection = $factoryCollection->create();       



        
// $date = date('Y-m-d H:i:s', strtotime("2018-04-10"));
// print_r($collection->getSelect());exit;
// $collection->getSelect()
//     ->reset(\Zend_Db_Select::FROM)
//     ->reset(\Zend_Db_Select::COLUMNS)
//     ->from(array('o' => $collection->getMainTable()), array('increment_id','base_grand_total','created_at'));
// $collection->getSelect()->joinRight( array('h'=> 'sales_order_status_history'), 'h.parent_id = o.entity_id', array('h.status','h.created_at'))->where('DATE(o.created_at) = ?')->where('DATE(h.created_at) = ?');;
// $collection->addFieldToFilter("h.status","processing_closed");




// echo $collection->getSelect();






 // $collection->getSelect();




                
    }

    public function applyToolbarFilters($collection)
    {
        // $this->addFromFilter($collection);
        // $this->addToFilter($collection);
        $this->addStoreFilter($collection);
        $this->addGroupBy($collection);
    }

    public function addGroupBy($collection)
    {
        $collection->getSelect()
            ->group("DAYNAME(main_table.created_at)")
        ;
    }
}
