<?php
/**
 * @author Azaleasoft Team
 * @copyright Copyright (c) 2016 Azaleasoft (http://www.azaleasoft.com)
 * @package Azaleasoft_Asstorelocator
 */
namespace Azaleasoft\Asstorelocator\Block\Adminhtml\Store\Edit;

class Form extends \Magento\Backend\Block\Widget\Form\Generic
{

    protected $systemStore;
    protected $helper;

    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Store\Model\System\Store $systemStore,
        \Azaleasoft\Asstorelocator\Helper\Data $dataHelper,
        array $data = []
    ) {
        $this->systemStore = $systemStore;
        $this->helper = $dataHelper;
        parent::__construct($context, $registry, $formFactory, $data);
    }


    public function _construct() {
        parent::_construct();
        $this->setId('azaleasoft_asstorelocator_store_edit_form');
        $this->setTitle(__('New Store'));
    }


    protected function _prepareForm()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $model = $this->_coreRegistry->registry('asstorelocator_store_data');

        $form = $this->_formFactory->create(
            ['data' => ['id' => 'edit_form', 'action' => $this->getData('action'), 'method' => 'post']]
        );

        $fieldset = $form->addFieldset(
            'store_fields',
            array(
                'legend' => __('Store Information'), 'class' => 'fieldset-wide'
            )
        );
        if ($model->getId()) {
            $form->addField('id', 'hidden', ['name' => 'id', 'value' => $model->getId()]);
        }
        $fieldset->addField('name', 'text',
            array(
                'label'     => __('Store Name'),
                'title'     => __('Store Name'),
                'name'      => 'name',
                'required'  => true,
                'class'     => 'required-entry validate-length maximum-length-255',
            )
        );
        $fieldset->addField('website_id', 'select',
            array(
                'label'     => __('Website'),
                'title'     => __('Website'),
                'name'      => 'website_id',
                'values'    => $this->systemStore->getWebsiteValuesForForm(),
                'required'  => true,
                'class'     => 'required-entry',
            )
        );
        $fieldset->addField('status', 'select',
            array(
                'label'     => __('Status'),
                'title'     => __('Status'),
                'name'      => 'status',
                'values'    => $this->helper->getStoreStatuses(),
                'required'  => true,
                'class'     => 'required-entry',
            )
        );
        $fieldset->addField('country', 'select',
            array(
                'label'     => __('Country'),
                'title'     => __('Country'),
                'name'      => 'country',
                'values'    => $objectManager->get('Magento\Directory\Model\Config\Source\Country\Full')->toOptionArray(),
                'required'  => true,
                'class'     => 'required-entry validate-no-empty',
            )
        );
        $fieldset->addField('state', 'text',
            array(
                'label'     => __('State'),
                'title'     => __('State'),
                'name'      => 'state',
                'required'  => true,
                'class'     => 'required-entry',
            )
        );
        $fieldset->addField('city', 'text',
            array(
                'label'     => __('City'),
                'title'     => __('City'),
                'name'      => 'city',
                'required'  => false,
                // 'class'     => 'required-entry',
            )
        );
        $fieldset->addField('street', 'text',
            array(
                'label'     => __('Street'),
                'title'     => __('Street'),
                'name'      => 'street',
                'required'  => false,
                // 'class'     => 'validate-street',
            )
        );
        $fieldset->addField('zipcode', 'text',
            array(
                'label'     => __('Zip Code'),
                'title'     => __('Zip Code'),
                'name'      => 'zipcode',
                'required'  => true,
                'class'     => 'required-entry',
            )
        );
        $fieldset->addField('lat', 'text',
            array(
                'label'     => __('Latitude'),
                'title'     => __('Latitude'),
                'name'      => 'lat',
                'required'  => false,
                // 'class'     => 'required-entry',
                'disabled'  => true,
            )
        );
        $fieldset->addField('lng', 'text',
            array(
                'label'     => __('Longitude'),
                'title'     => __('Longitude'),
                'name'      => 'lng',
                'required'  => false,
                // 'class'     => 'required-entry',
                'disabled'  => true,
            )
        );
        $fieldset->addField('phone', 'text',
            array(
                'label'     => __('Phone'),
                'title'     => __('Phone'),
                'name'      => 'phone',
                'required'  => false,
                // 'class'     => '',
            )
        );
        $fieldset->addField('fax', 'text',
            array(
                'label'     => __('Fax'),
                'title'     => __('Fax'),
                'name'      => 'fax',
                'required'  => false,
                // 'class'     => 'validate-fax',
            )
        );
        $fieldset->addField('site', 'text',
            array(
                'label'     => __('Portal Site'),
                'title'     => __('Portal Site'),
                'name'      => 'site',
                'required'  => false,
                'class'     => 'validate-url',
                'note'      => 'Example: http://example.com',
            )
        );
        $fieldset->addField('open_hours', 'textarea',
            array(
                'label'     => __('Open Hours'),
                'title'     => __('Open Hours'),
                'name'      => 'open_hours',
                'required'  => false,
                // 'class'     => '',
            )
        );
        $fieldset->addField('position', 'text',
            array(
                'label'     => __('Position'),
                'title'     => __('Position'),
                'name'      => 'position',
                'required'  => true,
                'class'     => 'validate-zero-or-greater',
                'note'      => 'Define display order.',
            )
        );

        $data = $model->getData();
        if (empty($data['id'])) {
            $data['status']   = 1;
            $data['country']  = 'US';
            $data['position'] = 0;
            $data['open_hours'] = 'Mon-Fri: 10:00AM - 8:00PM
Sat-Sun: 10:00AM - 10:00PM';
        }
        $form->setValues($data);
        $form->setUseContainer(true);
        $this->setForm($form);

        return parent::_prepareForm();
    }
}
