<?php
/**
 * @author Azaleasoft Team
 * @copyright Copyright (c) 2016 Azaleasoft (http://www.azaleasoft.com)
 * @package Azaleasoft_Asstorelocator
 */

namespace Azaleasoft\Asstorelocator\Block;

class Content extends \Magento\Framework\View\Element\Template
{
    protected $objectManager;
    protected $customerSession;
    protected $helper;

    protected $_storelocatorCollection;

    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
		\Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Customer\Model\Session $customerSession,
        array $data = []
    ) {
        parent::__construct($context, $data);

        $this->objectManager   = $objectManager;
        $this->customerSession = $customerSession;
        $this->helper          = $this->objectManager->get('Azaleasoft\Asstorelocator\Helper\Data');
    }


    public function _prepareLayout() {
        $this->pageConfig->getTitle()->set($this->helper->getPageTitle());
    }


    public function getStorelocatorCollection() {
        if ($this->_storelocatorCollection == null) {
            $this->_storelocatorCollection = $this->objectManager->get('Azaleasoft\Asstorelocator\Model\ResourceModel\Asstorelocator\Collection')
                ->addFieldToFilter('website_id', $this->_storeManager->getStore()->getWebsiteId())
                ->addFieldToFilter('status', 1)
                ->setOrder('position', 'asc');
        }
        return $this->_storelocatorCollection;
    }


    public function getStorelocatorList() {
        if ($this->_storelocatorCollection == null) {
            $this->getStorelocatorCollection();
        }

        $storelocators = array();
        foreach ($this->_storelocatorCollection as $row) {
            $storelocators[] = [
                'id'    => $row->getId(),
                'name'  => $row->getName(),
                'lat'   => $row->getLat(),
                'lng'   => $row->getLng(),
                'phone' => $row->getPhone(),
                'fax'   => $row->getFax(),
                'site'  => $row->getSite(),
                'open_hours' => $row->getOpenHours()
            ];
        }

        return $storelocators;
    }

    public function getDefaultLocation() {
        if ($this->customerSession->isLoggedIn()) {
            $shippingId = $this->customerSession->getCustomer()->getDefaultShipping();
            if (!empty($shippingId)) {
                $address = $this->objectManager->get('Magento\Customer\Model\Address')->load($shippingId);
                if (!empty($address)) {
                    return $address->getPostcode();
                }
            }
        }
        
        return '';
    }
}