<?php
/**
 * @author Azaleasoft Team
 * @copyright Copyright (c) 2016 Azaleasoft (http://www.azaleasoft.com)
 * @package Azaleasoft_Asstorelocator
 */

namespace Azaleasoft\Asstorelocator\Helper;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    // general settings
    const ASSTORELOCATOR_GENERAL_ENABLE		     = 'azaleasoft_asstorelocator/general/enable';
    const ASSTORELOCATOR_GENERAL_GOOGLE_KEY	     = 'azaleasoft_asstorelocator/general/google_key';
    const ASSTORELOCATOR_GENERAL_STORE_URL       = 'azaleasoft_asstorelocator/general/store_url';
    const ASSTORELOCATOR_GENERAL_LINK_NAME       = 'azaleasoft_asstorelocator/general/link_name';
	const ASSTORELOCATOR_GENERAL_PAGE_TITLE      = 'azaleasoft_asstorelocator/general/page_title';
	const ASSTORELOCATOR_GENERAL_DEFAULT_LAT     = 'azaleasoft_asstorelocator/general/default_lat';
	const ASSTORELOCATOR_GENERAL_DEFAULT_LNG     = 'azaleasoft_asstorelocator/general/default_lng';
    const ASSTORELOCATOR_GENERAL_TOPMENU         = 'azaleasoft_asstorelocator/general/topmenu';
    const ASSTORELOCATOR_GENERAL_RANGE           = 'azaleasoft_asstorelocator/general/range';
    const ASSTORELOCATOR_GENERAL_SECURE_URL      = 'azaleasoft_asstorelocator/general/secure_url';
    
    // google map settings	
	const ASSTORELOCATOR_GOOGLE_MAP_MARKER_ICON  = 'azaleasoft_asstorelocator/google_map/marker_icon';
	const ASSTORELOCATOR_GOOGLE_MAP_ZOOM         = 'azaleasoft_asstorelocator/google_map/zoom';
    const ASSTORELOCATOR_GOOGLE_DISTANCE_UNIT    = 'azaleasoft_asstorelocator/google_map/distance_unit';
    const ASSTORELOCATOR_GOOGLE_GEOLOCATION      = 'azaleasoft_asstorelocator/google_map/geolocation';

    // search option
	const ASSTORELOCATOR_SEARCH_OPTION_DISTANCE  = 'azaleasoft_asstorelocator/search_option/distance';
	const ASSTORELOCATOR_SEARCH_OPTION_NAME   = 'azaleasoft_asstorelocator/search_option/name';

    // meta
    const ASSTORELOCATOR_META_TITLE              = 'azaleasoft_asstorelocator/meta/title';
    const ASSTORELOCATOR_META_KEYWORDS           = 'azaleasoft_asstorelocator/meta/keywords';
    const ASSTORELOCATOR_META_DESCRIPTION        = 'azaleasoft_asstorelocator/meta/description';

    protected $_storeManager;
    protected $objectManager;

    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        parent::__construct($context);
        $this->_storeManager = $storeManager;
        $this->objectManager = \Magento\Framework\App\ObjectManager::getInstance();
    }


    public function isEnabled() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_ENABLE, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getGoogleKey() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_GOOGLE_KEY, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getStoreUrl() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_STORE_URL, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function isSecureUrl() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_SECURE_URL, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getFullUrl() {
        if ($this->isSecureUrl()) {
            return $this->_storeManager->getStore()->getUrl($this->getStoreUrl(), ['_secure' => true]);
        } else {
            return $this->_storeManager->getStore()->getUrl($this->getStoreUrl(), ['_secure' => false]); 
        }
    }


    public function getLinkName() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_LINK_NAME, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getPageTitle() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_PAGE_TITLE, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getDefaultLat() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_DEFAULT_LAT, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getDefaultLng() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_DEFAULT_LNG, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getTopmenuIsEnabled() {
        if (!$this->isEnabled()) {
            return false;
        }
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_TOPMENU, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getRange() {
        if (!$this->isEnabled()) {
            return false;
        }
        $range = $this->scopeConfig->getValue(self::ASSTORELOCATOR_GENERAL_RANGE, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
        if (!empty($range)) {
            return explode(',', $range);
        } else {
            return array();
        }
    }    


    public function getMapZoom() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GOOGLE_MAP_ZOOM, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getDistanceUnit() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GOOGLE_DISTANCE_UNIT, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    public function getMetaTitle() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_META_TITLE, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    public function getMetaKeywords() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_META_KEYWORDS, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    public function getMetaDescription() {
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_META_DESCRIPTION, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function geolocationIsEnabled() {
        if (!$this->isEnabled()) {
            return false;
        }

        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_GOOGLE_GEOLOCATION, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function searchByDistanceIsEnabled() {
        return true;
    }


    public function searchByNameIsEnabled() {
        if (!$this->isEnabled()) {
            return false;
        }
        
        return $this->scopeConfig->getValue(self::ASSTORELOCATOR_SEARCH_OPTION_NAME, \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }


    public function getStoreStatuses() {
        return [
            0 => __('Disabled'),
            1 => __('Enabled')
        ];
    }


    public function getLatLngByAddress($asstorelocator_id) {
        $url = 'https://maps.googleapis.com/maps/api/geocode/json?key=' . $this->getGoogleKey();

        $model = $this->objectManager->create('Azaleasoft\Asstorelocator\Model\Asstorelocator');
        $model->load($asstorelocator_id);

        $address = '';
        if (!empty($model->getStreet())) {
            $address = trim($model->getStreet());
        }
        if (!empty($model->getCity())) {
            $address .= ','.trim($model->getCity());
        }
        if (!empty($model->getState())) {
            $address .= ','.trim($model->getState());
        }
        if (!empty($model->getZipcode())) {
            $address .= ','.trim($model->getZipcode());
        }
        if (!empty($model->getCountry())) {
            $address .= ','.trim($model->getCountry());
        }
        
        $address = str_replace(' ', '+', $address);
        $url .= '&address='.$address;

        $client = new \Zend_Http_Client($url);
        $client->setMethod(\Zend_Http_Client::GET);
        
        try{
            $response = $client->request();
        } catch(\Exception $e) {
            return;
        }
        
        if (!empty($response->getStatus()) && $response->getStatus() == 200) {
            $body = json_decode($response->getBody());
            $results = $body->results;
            if (count($results)) {
                $result = $results[0];
                $location = $result->geometry->location;
                $model->setLat($location->lat)
                    ->setLng($location->lng)
                    ->save();
            }
        }
    }
}

