<?php

namespace Custom\ProductManagement\Model;

use Symfony\Component\Console\Output\ConsoleOutput as Output;

/**
 * Custom ProductManagement logger model.
 *
 * @category Custom
 * @package  Custom_ProductManagement
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class Logger extends \Monolog\Logger
{
    /**
     * @var Output
     */
    private $output;

    /**
     * Logger constructor.
     *
     * @param Output $output
     * @param string $name
     * @param array  $handlers
     * @param array  $processors
     */
    public function __construct(
        Output $output,
        $name,
        array $handlers = [],
        array $processors = []
    ) {
        $this->output = $output;

        parent::__construct(
            $name,
            $handlers,
            $processors
        );
    }

    /**
     * @param string $message
     *
     * @return Logger
     */
    public function logDebug($message)
    {
        $this->output->writeln('<info>' . $message . '</info>');
        $this->debug($message);

        return $this;
    }

    /**
     * @param string $message
     *
     * @return Logger
     */
    public function logInfo($message)
    {
        $this->output->writeln('<info>' . $message . '</info>');
        $this->info($message);

        return $this;
    }

    /**
     * @param string $message
     *
     * @return Logger
     */
    public function logError($message)
    {
        $this->output->writeln('<error>' . $message . '</error>');
        $this->error($message);

        return $this;
    }
}
