<?php

namespace Custom\SalesReport\Cron\Report;

use Custom\SalesReport\Model\Config as ModuleConfig;
use Custom\SalesReport\Model\Email as EmailProcessor;
use Magento\Framework\Stdlib\DateTime\DateTime;
use Psr\Log\LoggerInterface;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory;

/**
 * Custom SalesReport daily report cron.
 *
 * @category Custom
 * @package  Custom_SalesReport
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class Daily
{
    /**
     * @var ModuleConfig
     */
    private $moduleConfig;

    /**
     * @var DateTime
     */
    private $date;

    /**
     * @var EmailProcessor
     */
    private $emailProcessor;

    /**
     * @var CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * Process constructor.
     *
     * @param ModuleConfig    $moduleConfig
     * @param DateTime        $date
     * @param EmailProcessor  $emailProcessor
     * @param CollectionFactory $collectionFactory
     * @param LoggerInterface $logger
     */
    public function __construct(
        ModuleConfig $moduleConfig,
        DateTime $date,
        EmailProcessor $emailProcessor,
        CollectionFactory $collectionFactory,
        LoggerInterface $logger
    ) {
        $this->moduleConfig = $moduleConfig;
        $this->date = $date;
        $this->emailProcessor = $emailProcessor;
        $this->collectionFactory = $collectionFactory;
        $this->logger = $logger;
    }

    /**
     * @return Daily
     */
    public function execute()
    {
        if ($this->moduleConfig->isEnabled() === false) {
            return $this;
        }

        try {
            //$day = date('Y-m-d', strtotime('-1 day', $this->date->gmtTimestamp()));
	    $day = date('Y-m-d', $this->date->gmtTimestamp());

            $start = $day . ' 00:00:00';
            $end = $day . ' 23:59:59';

            $collection = $this->collectionFactory->create();
            $collection
                ->addFieldToFilter('status', ['in' => $this->moduleConfig->getOrderStatuses()])
                ->addFieldToFilter('updated_at', ['lteq' => $end])
                ->addFieldToFilter('updated_at', ['gteq' =>$start]);

	    $total = 0.0;
            foreach ($collection as $order) {
                $total += (float)$order->getBaseGrandTotal();
            }

            $templatesVars = [
		'orders_total' => number_format($total, 2, '.', ','),
                'orders_count' => $collection->getSize(),
                'date_range' => $start . ' - ' . $end,
                'order_statuses' => implode(', ', $this->moduleConfig->getOrderStatuses()),
            ];

            $this->emailProcessor->sendDailyReport(
                $this->moduleConfig->getEmails(),
                $templatesVars
            );
        } catch (\Exception $e) {
            $this->logger->error($e);
        }

        return $this;
    }
}
