<?php

namespace Custom\SalesReport\Model;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;

/**
 * Custom SalesReport config model.
 *
 * @category Custom
 * @package  Custom_SalesReport
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class Config
{
    const XML_PATH_GENERAL_ENABLE = 'custom_salesreport_configuration/general/enable';
    const XML_PATH_GENERAL_EMAILS = 'custom_salesreport_configuration/general/emails';
    const XML_PATH_GENERAL_ORDER_STATUSES = 'custom_salesreport_configuration/general/order_statuses';

    /**
     * @var null|int
     */
    private $storeId = null;

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    /**
     * Already fetched config values.
     *
     * @var array
     */
    private $config = [];

    /**
     * Object initialization.
     *
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    ) {
        $this->scopeConfig = $scopeConfig;
    }

    /**
     * Store id setter.
     *
     * @param   null|int $storeId
     *
     * @return  Config
     */
    public function setStoreId($storeId = null)
    {
        $this->storeId = $storeId;

        return $this;
    }

    /**
     * Return config field value.
     *
     * @param string $keyPath Key path.
     *
     * @return mixed
     */
    private function getConfigValue($keyPath)
    {
        if (isset($this->config[$keyPath]) === false) {
            $this->config[$keyPath] = $this->scopeConfig->getValue(
                $keyPath,
                ScopeInterface::SCOPE_STORE,
                $this->storeId
            );
        }

        return $this->config[$keyPath];
    }

    /**
     * @return bool
     */
    public function isEnabled()
    {
        return (bool)$this->getConfigValue(self::XML_PATH_GENERAL_ENABLE);
    }

    /**
     * @return array
     */
    public function getEmails()
    {
        if (!empty($this->config[self::XML_PATH_GENERAL_EMAILS])) {
            return $this->config[self::XML_PATH_GENERAL_EMAILS];
        }

        $emails = $this->getConfigValue(self::XML_PATH_GENERAL_EMAILS);
        if (empty($emails)) {
            return [];
        }

        $emails = explode("\n", trim($emails));
        foreach ($emails as $index => $email) {
            if (empty($email)) {
                unset($emails[$index]);
                continue;
            }

            $emails[$index] = trim($email);
        }
        $this->config[self::XML_PATH_GENERAL_EMAILS] = array_values($emails);

        return $emails;
    }

    /**
     * @return array
     */
    public function getOrderStatuses()
    {
        if (!empty($this->config[self::XML_PATH_GENERAL_ORDER_STATUSES])) {
            return $this->config[self::XML_PATH_GENERAL_ORDER_STATUSES];
        }

        $statuses = $this->getConfigValue(self::XML_PATH_GENERAL_ORDER_STATUSES);
        if (empty($statuses)) {
            return [];
        }

        $statuses = explode(',', $statuses);
        $this->config[self::XML_PATH_GENERAL_ORDER_STATUSES] = $statuses;

        return $statuses;
    }
}
