<?php

namespace Custom\SalesReport\Model\Config\Source\Order;

use Amasty\OrderStatus\Model\Status as AmastyStatus;
use Magento\Sales\Model\Order\Status as MagentoStatus;

/**
 * Custom SalesReport config source order statuses model.
 *
 * @category Custom
 * @package  Custom_SalesReport
 * @author   Piotr Pierzak <piotrek.pierzak@gmail.com>
 */
class Statuses
{
    /**
     * @var MagentoStatus
     */
    private $magentoStatus;

    /**
     * @var AmastyStatus
     */
    private $amastyStatus;

    /**
     * Statuses constructor.
     *
     * @param MagentoStatus $magentoStatus
     * @param AmastyStatus  $amastyStatus
     */
    public function __construct(
        MagentoStatus $magentoStatus,
        AmastyStatus $amastyStatus
    ) {
        $this->magentoStatus = $magentoStatus;
        $this->amastyStatus = $amastyStatus;
    }

    /**
     * @return array
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function toOptionArray()
    {
	/*$objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $object = $objectManager->get('Custom\SalesReport\Cron\Report\Daily');
        $object->execute();*/

        $statuses = [];

        $magentoStatuses = $this->magentoStatus->getResourceCollection()->getData();
        foreach ($magentoStatuses as $status) {
            $statuses[] = [
                'value' => $status['status'],
                'label' => $status['label'],
            ];
        }

        $amastyStatuses = $this->amastyStatus->getResourceCollection()->getData();
        foreach ($amastyStatuses as $status) {
            $statuses[] = [
                'value' => $status['parent_state'] . '_' . $status['alias'],
                'label' => $status['status'],
            ];
        }

        return $statuses;
    }
}

