<?php
namespace MageArray\SocialLogin\Controller\Facebook;

use Exception;
use Magento\Customer\Model\Url;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use MageArray\SocialLogin\Helper\Facebook;
use Magento\Backend\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Connect
 * @package MageArray\SocialLogin\Controller\Facebook
 */
class Connect extends Action
{
    /**
     * @var Facebook
     */
    protected $_fbHelper;
    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;
    /**
     * @var Session
     */
    protected $_session;
    /**
     * @var Url
     */
    protected $_customerHelperData;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var
     */
    protected $_flag;

    /**
     * Connect constructor.
     * @param Context $context
     * @param Session $session
     * @param \Magento\Customer\Model\Session $customerSession
     * @param Url $customerHelperData
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param Facebook $_fbHelper
     */
    public function __construct(
        Context $context,
        Session $session,
        \Magento\Customer\Model\Session $customerSession,
        Url $customerHelperData,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        Facebook $_fbHelper
    ) {
        $this->_fbHelper = $_fbHelper;
        $this->_customerSession = $customerSession;
        $this->_session = $session;
        $this->_customerHelperData = $customerHelperData;
        $this->_scopeConfig = $scopeConfig;
        $this->_storeManager = $storeManager;
        parent::__construct($context);
    }

    /**
     *
     */
    public function execute()
    {
        try {
			
            $this->_connectCallback();
        } catch (Exception $e) {
			//echo '<pre>'; print_r($e); exit();
            $this->messageManager->addNotice($e->getMessage());
        }

        $this->_sendResponse();
        $this->_windowCloseReload();

    }

    /**
     *
     */
    protected function _windowCloseReload()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">
                window.opener.location.reload(true);window.close();
            </script>'
        );
    }

    /**
     *
     */
    protected function _windowClose()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">window.close();</script>'
        );
    }

    /**
     * @return $this|void
     * @throws Exception
     */
    protected function _connectCallback()
    {
        $errorCode = $this->getRequest()->getParam('error');
        $code = $this->getRequest()->getParam('code');
        $state = $this->getRequest()->getParam('state');
        if (!($errorCode || $code) && !$state) {
            // Direct route access - deny
            return;
        }
        if (!$state || $state != $this->_session->getFacebookCsrf()) {
            return;
        }
        if ($errorCode) {
            // Facebook API read light - abort
            if ($errorCode === 'access_denied') {
                $this->_flag = "noaccess";
                $this->_windowClose();
            }
            return;
        }
        
        if ($code) {
            $attributeModel = $this->_objectManager
                ->get('Magento\Customer\Model\Customer')->getAttributes();
            $attributegId = false;
            $attributegtoken = false;
            foreach ($attributeModel as $cal => $valData) {
                if ($cal == 'ma_sociallogin_fid') {
                    $attributegId = true;
                }
                if ($cal == 'ma_sociallogin_ftoken') {
                    $attributegtoken = true;
                }

            }
            if ($attributegId == false || $attributegtoken == false) {
                $this->messageManager->addNotice(
                    __('Attribute `ma_sociallogin_fid` or
                 `ma_sociallogin_ftoken` not exist !')
                );
                $this->_windowCloseReload();
            }
			
			
            // Facebook API green light - proceed
            $userInfo = $this->_fbHelper->api('/me', 'GET',
                [
                    'fields' =>
                        'id,name,first_name,last_name,link,
                        birthday,gender,email,picture.type(large)'
                ]);
				
            $token = $this->_fbHelper->getAccessToken();
		
            $customersByFacebookId = $this->_fbHelper
                ->getCustomersByFacebookId($userInfo->id);
            if ($this->_customerSession->isLoggedIn()) {
                if ($customersByFacebookId->getSize()) {
                    $this->messageManager->addNotice(
                        __('Your Facebook account is already connected 
                        to one of our store accounts.')
                    );

                    return $this;
                }
                // Connect from account dashboard - attach
                $customerId = $this->_customerSession->getCustomerId();

                $this->_fbHelper->connectByFacebookId(
                    $userInfo->id,
                    $token,
                    $customerId
                );

                $this->messageManager->addSuccess(
                    __('Your Facebook account is now connected 
                        to your store account.
                        You can now login using our Facebook ' .
                        'Login button or using store account 
                        credentials you will receive to your email address.')
                );

                return $this;
            }

            if ($customersByFacebookId->getSize()) {
                // Existing connected user - login
                $customer = $customersByFacebookId->getFirstItem();
                /* @var $customer \Magento\Customer\Model\Customer */
                $this->_fbHelper->loginByCustomer($customer);
                $this->messageManager->addSuccess(
                    __('You have successfully logged in 
                    using your Facebook account.')
                );

                return $this;
            }

            $customersByEmail = $this->_fbHelper
                ->getCustomersByEmail($userInfo->email);
            if ($customersByEmail->getSize()) {
                // Email account already exists - attach, login
                $customerId = $customersByEmail->getFirstItem()->getId();

                $this->_fbHelper->connectByFacebookId(
                    $userInfo->id,
                    $token,
                    $customerId
                );

                $this->messageManager->addSuccess(
                    __('We have discovered you already 
                        have an account at our store. 
                        Your Facebook account is now connected ' .
                        'to your store account.')
                );

                return $this;
            }
            // New connection - create, attach, login
            $firstName = $userInfo->first_name;
            if (!$firstName) {
                throw new Exception(
                    __('Sorry, could not retrieve your 
                    Facebook first name. Please try again.')
                );
            }

            $lastName = $userInfo->last_name;
            if (!$lastName) {
                throw new Exception(
                    __('Sorry, could not retrieve your 
                    Facebook last name. Please try again.')
                );
            }

            $this->_fbHelper->connectByCreatingAccount(
                $userInfo->id,
                $token,
                $userInfo->email,
                $userInfo->first_name,
                $userInfo->last_name
            );

            $this->messageManager->addSuccess(
                __('Your Facebook account is now connected to 
                    your new user account at our store. 
                    Now you can login using ' .
                    'our Facebook Login button or using 
                    store account credentials you will 
                    receive to your email address.')
            );
        }
    }

    /**
     * @return mixed
     */
    protected function _sendResponse()
    {
        $return = [
            'redirect' => $this->_loginPostRedirect()
        ];

        return json_encode($return);
    }

    /**
     * @return mixed
     */
    protected function _loginPostRedirect()
    {
        $lastCustomerId = $this->_getSession()->getLastCustomerId();
        if (isset(
                $lastCustomerId
            ) && $this->_getSession()->isLoggedIn()
            && $lastCustomerId != $this->_getSession()->getId()
        ) {
            $this->_getSession()->unsBeforeAuthUrl()
                ->setLastCustomerId($this->_getSession()->getId());
        }
        if (!$this->_getSession()->getBeforeAuthUrl() ||
            $this->_getSession()
                ->getBeforeAuthUrl() == $this->_storeManager
                ->getStore()->getBaseUrl()
        ) {
            // Set default URL to redirect customer to
            $this->_getSession()
                ->setBeforeAuthUrl($this->_customerHelperData->getAccountUrl());
            // Redirect customer to the last page visited after logging in
            if ($this->_getSession()->isLoggedIn()) {
                if ($this->_getSession()->getAfterAuthUrl()) {
                    $this->_getSession()
                        ->setBeforeAuthUrl($this->_getSession()
                            ->getAfterAuthUrl(true));
                }
            } else {
                $this->_getSession()
                    ->setBeforeAuthUrl($this->_customerHelperData
                        ->getLoginUrl());
            }
        } elseif ($this->_getSession()
                ->getBeforeAuthUrl() == $this->_customerHelperData
                ->getLogoutUrl()
        ) {
            $this->_getSession()
                ->setBeforeAuthUrl($this->_customerHelperData
                    ->getDashboardUrl());
        } else {
            if (!$this->_getSession()->getAfterAuthUrl()) {
                $this->_getSession()
                    ->setAfterAuthUrl($this->_getSession()->getBeforeAuthUrl());
            }
            if ($this->_getSession()->isLoggedIn()) {
                $this->_getSession()
                    ->setBeforeAuthUrl($this->_getSession()
                        ->getAfterAuthUrl(true));
            }
        }
        return $this->_getSession()->getBeforeAuthUrl(true);
    }

    /**
     * @return \Magento\Customer\Model\Session
     */
    protected function _getSession()
    {
        return $this->_customerSession;
    }
}