<?php
namespace MageArray\SocialLogin\Controller\Linkedin;

use Magento\Customer\Model\Url;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use MageArray\SocialLogin\Helper\Linkedin;
use Magento\Backend\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Exception;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Connect
 * @package MageArray\SocialLogin\Controller\Linkedin
 */
class Connect extends Action
{
    /**
     * @var Linkedin
     */
    protected $_liHelper;
    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;
    /**
     * @var Session
     */
    protected $_session;
    /**
     * @var Url
     */
    protected $_customerHelperData;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var
     */
    protected $_flag;

    /**
     * Connect constructor.
     * @param Context $context
     * @param Session $session
     * @param \Magento\Customer\Model\Session $customerSession
     * @param Url $customerHelperData
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param Linkedin $_liHelper
     */
    public function __construct(
        Context $context,
        Session $session,
        \Magento\Customer\Model\Session $customerSession,
        Url $customerHelperData,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        Linkedin $_liHelper
    ) {
        $this->_liHelper = $_liHelper;
        $this->_customerSession = $customerSession;
        $this->_session = $session;
        $this->_customerHelperData = $customerHelperData;
        $this->_scopeConfig = $scopeConfig;
        $this->_storeManager = $storeManager;
        parent::__construct($context);
    }

    /**
     *
     */
    public function execute()
    {
        try {
            $this->_connectCallback();
        } catch (Exception $e) {
            $this->messageManager->addNotice($e->getMessage());
        }

        $this->_sendResponse();
        $this->_windowCloseReload();

    }

    /**
     *
     */
    protected function _windowCloseReload()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">
                window.opener.location.reload(true);window.close();
            </script>'
        );
    }

    /**
     *
     */
    protected function _windowClose()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">window.close();</script>'
        );
    }

    /**
     * @return $this|void
     * @throws Exception
     */
    protected function _connectCallback()
    {
        $errorCode = $this->getRequest()->getParam('error');
        $code = $this->getRequest()->getParam('code');
        $state = $this->getRequest()->getParam('state');
        if (!($errorCode || $code) && !$state) {
            // Direct route access - deny
            return;
        }
        if (!$state || $state != $this->_session->getLinkedinCsrf()) {
            return;
        }

        if ($errorCode) {
            // Linkedin API read light - abort
            if ($errorCode === 'access_denied') {
                $this->_flag = "noaccess";
                $this->_windowClose();
            }
            return;
        }

        if ($code) {
            $attributeModel = $this->_objectManager
                ->get('Magento\Customer\Model\Customer')->getAttributes();
            $attributegId = false;
            $attributegtoken = false;
            foreach ($attributeModel as $cal => $valData) {
                if ($cal == 'ma_sociallogin_lid') {
                    $attributegId = true;
                }
                if ($cal == 'ma_sociallogin_ltoken') {
                    $attributegtoken = true;
                }

            }
            if ($attributegId == false || $attributegtoken == false) {
                $this->messageManager->addNotice(
                    __('Attribute `ma_sociallogin_lid` or
                 `ma_sociallogin_ltoken` not exist !')
                );
                $this->_windowCloseReload();
            }
            // Linkedin API green light - proceed
            $userInfo = $this->_liHelper->api('/v1/people/~');
            $token = $this->_liHelper->getAccessToken();
            $customersByLinkedinId = $this->_liHelper
                ->getCustomersByLinkedinId($userInfo->id);
            if ($this->_customerSession->isLoggedIn()) {
                // Logged in user
                if ($customersByLinkedinId->getSize()) {
                    // Linkedin account already
                    // connected to other account - deny
                    $this->messageManager->addNotice(
                        __('Your Linkedin account is already 
                        connected to one of our store accounts.')
                    );

                    return $this;
                }
                // Connect from account dashboard - attach
                $customerId = $this->_customerSession->getCustomerId();

                $this->_liHelper->connectByLinkedinId(
                    $userInfo->id,
                    $token,
                    $customerId
                );

                $this->messageManager->addSuccess(
                    __('Your Linkedin account is now 
                    connected to your store account.
                    You can now login using our Linkedin 
                    Login button or using store account 
                    credentials you will receive 
                    to your email address.')
                );

                return $this;
            }

            if ($customersByLinkedinId->getSize()) {
                // Existing connected user - login
                $customer = $customersByLinkedinId->getFirstItem();
                /* @var $customer \Magento\Customer\Model\Customer */

                $this->_liHelper->loginByCustomer($customer);

                $this->messageManager->addSuccess(
                    __('You have successfully logged in 
                        using your Linkedin account.')
                );

                return $this;
            }
            $customersByEmail = $this->_liHelper
                ->getCustomersByEmail($userInfo->emailAddress);

            if ($customersByEmail->getSize()) {
                // Email account already exists - attach, login
                $customerId = $customersByEmail->getFirstItem()->getId();
                $this->_liHelper->connectByLinkedinId(
                    $userInfo->id,
                    $token,
                    $customerId
                );

                $this->messageManager->addSuccess(
                    __('We have discovered you already 
                        have an account at our store.
                        Your Linkedin account is now connected ' .
                        'to your store account.')
                );

                return $this;
            }
            // New connection - create, attach, login
            $firstName = $userInfo->firstName;
            if (!$firstName) {
                throw new Exception(
                    __('Sorry, could not retrieve your 
                    Linkedin first name. Please try again.')
                );
            }

            $lastName = $userInfo->lastName;
            if (!$lastName) {
                throw new Exception(
                    __('Sorry, could not retrieve your 
                    Linkedin last name. Please try again.')
                );
            }
            $this->_liHelper->connectByCreatingAccount(
                $userInfo->id,
                $token,
                $userInfo->emailAddress,
                $userInfo->firstName,
                $userInfo->lastName
            );

            $this->messageManager->addSuccess(
                __('Your Linkedin account is now connected to 
                    your new user account at our store.
                    Now you can login using ' .
                    'our Linkedin Login button or 
                    using store account credentials 
                    you will receive to your email address.')
            );
        }
    }

    /**
     * @return mixed
     */
    protected function _sendResponse()
    {
        $return = [
            'redirect' => $this->_loginPostRedirect()
        ];

        return json_encode($return);
    }

    /**
     * @return mixed
     */
    protected function _loginPostRedirect()
    {
        $lastCustomerId = $this->_getSession()
            ->getLastCustomerId();
        if (isset(
                $lastCustomerId
            ) && $this->_getSession()->isLoggedIn()
            && $lastCustomerId != $this->_getSession()->getId()
        ) {
            $this->_getSession()->unsBeforeAuthUrl()
                ->setLastCustomerId($this->_getSession()->getId());
        }
        if (!$this->_getSession()->getBeforeAuthUrl() ||
            $this->_getSession()
                ->getBeforeAuthUrl() == $this->_storeManager
                ->getStore()->getBaseUrl()
        ) {
            // Set default URL to redirect customer to
            $this->_getSession()
                ->setBeforeAuthUrl($this->_customerHelperData->getAccountUrl());
            // Redirect customer to the last page visited after logging in
            if ($this->_getSession()->isLoggedIn()) {
                if ($this->_getSession()->getAfterAuthUrl()) {
                    $this->_getSession()
                        ->setBeforeAuthUrl($this->_getSession()
                            ->getAfterAuthUrl(true));
                }
            } else {
                $this->_getSession()
                    ->setBeforeAuthUrl($this->_customerHelperData
                        ->getLoginUrl());
            }
        } elseif ($this->_getSession()
                ->getBeforeAuthUrl() == $this->_customerHelperData
                ->getLogoutUrl()
        ) {
            $this->_getSession()
                ->setBeforeAuthUrl($this->_customerHelperData
                    ->getDashboardUrl());
        } else {
            if (!$this->_getSession()->getAfterAuthUrl()) {
                $this->_getSession()
                    ->setAfterAuthUrl($this->_getSession()->getBeforeAuthUrl());
            }
            if ($this->_getSession()->isLoggedIn()) {
                $this->_getSession()
                    ->setBeforeAuthUrl($this->_getSession()
                        ->getAfterAuthUrl(true));
            }
        }
        return $this->_getSession()->getBeforeAuthUrl(true);
    }

    /**
     * @return \Magento\Customer\Model\Session
     */
    protected function _getSession()
    {
        return $this->_customerSession;
    }
}