<?php
namespace MageArray\SocialLogin\Controller\Live;

use Magento\Customer\Model\Url;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use MageArray\SocialLogin\Helper\Live;
use Magento\Backend\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Exception;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Connect
 * @package MageArray\SocialLogin\Controller\Live
 */
class Connect extends Action
{
    /**
     * @var Live
     */
    protected $_liHelper;
    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $_customerSession;
    /**
     * @var Session
     */
    protected $_session;
    /**
     * @var Url
     */
    protected $_customerHelperData;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;
    /**
     * @var
     */
    protected $_flag;

    /**
     * Connect constructor.
     * @param Context $context
     * @param Session $session
     * @param \Magento\Customer\Model\Session $customerSession
     * @param Url $customerHelperData
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreManagerInterface $storeManager
     * @param Live $_liHelper
     */
    public function __construct(
        Context $context,
        Session $session,
        \Magento\Customer\Model\Session $customerSession,
        Url $customerHelperData,
        ScopeConfigInterface $scopeConfig,
        StoreManagerInterface $storeManager,
        Live $_liHelper
    ) {
        $this->_liHelper = $_liHelper;
        $this->_customerSession = $customerSession;
        $this->_session = $session;
        $this->_customerHelperData = $customerHelperData;
        $this->_scopeConfig = $scopeConfig;
        $this->_storeManager = $storeManager;
        parent::__construct($context);
    }

    /**
     *
     */
    public function execute()
    {
        try {
            $this->_connectCallback();
        } catch (Exception $e) {
            $this->messageManager->addNotice($e->getMessage());
        }

        $this->_sendResponse();
        $this->_windowCloseReload();

    }

    /**
     *
     */
    protected function _windowCloseReload()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">
                window.opener.location.reload(true);window.close();
            </script>'
        );
    }

    /**
     *
     */
    protected function _windowClose()
    {
        $this->getResponse()->setBody(
            '<script type="text/javascript">window.close();</script>'
        );
    }

    /**
     * @return $this|void
     * @throws Exception
     */
    protected function _connectCallback()
    {
        $errorCode = $this->getRequest()->getParam('error');
        $code = $this->getRequest()->getParam('code');
        $state = $this->getRequest()->getParam('state');
        if (!($errorCode || $code) && !$state) {
            // Direct route access - deny
            return;
        }
        if (!$state || $state != $this->_session->getLiveCsrf()) {
            return;
        }

        if ($errorCode) {
            // Live API read light - abort
            if ($errorCode === 'access_denied') {
                $this->_flag = "noaccess";
                $this->_windowClose();
            }
            return;
        }

        if ($code) {
            $attributeModel = $this->_objectManager
                ->get('Magento\Customer\Model\Customer')->getAttributes();
            $attributegId = false;
            $attributegtoken = false;
            foreach ($attributeModel as $cal => $valData) {
                if ($cal == 'ma_sociallogin_liid') {
                    $attributegId = true;
                }
                if ($cal == 'ma_sociallogin_litoken') {
                    $attributegtoken = true;
                }

            }
            if ($attributegId == false
                || $attributegtoken == false
            ) {
                $this->messageManager->addNotice(
                    __('Attribute `ma_sociallogin_liid` or
                 `ma_sociallogin_litoken` not exist !')
                );
                $this->_windowCloseReload();
            }
            // Live API green light - proceed

            $userInfo = $this->_liHelper->api("me");
            $token = $this->_liHelper->getAccessToken();

            $email = $userInfo->emails->account;

            $customersByLiveId = $this->_liHelper
                ->getCustomersByLiveId($userInfo->id);
            if ($this->_customerSession->isLoggedIn()) {
                // Logged in user
                if ($customersByLiveId->getSize()) {
                    // Live account already connected to other account - deny
                    $this->messageManager->addNotice(
                        __('Your Live account is already connected 
                        to one of our store accounts.')
                    );

                    return $this;
                }
                // Connect from account dashboard - attach
                $customerId = $this->_customerSession->getCustomerId();

                $this->_liHelper->connectByLiveId(
                    $userInfo->id,
                    $token,
                    $customerId
                );

                $this->messageManager->addSuccess(
                    __('Your Live account is now connected 
                        to your store account. You can now 
                        login using our Live ' .
                        'Login button or using store account credentials 
                        you will receive to your email address.')
                );

                return $this;
            }

            if ($customersByLiveId->getSize()) {
                // Existing connected user - login
                $customer = $customersByLiveId->getFirstItem();
                $this->_liHelper->loginByCustomer($customer);

                $this->messageManager->addSuccess(
                    __('You have successfully logged 
                    in using your Live account.')
                );

                return $this;
            }

            $customersByEmail = $this->_liHelper->getCustomersByEmail($email);
            if ($customersByEmail->getSize()) {
                // Email account already exists - attach, login
                $customerId = $customersByEmail->getFirstItem()->getId();

                $this->_liHelper->connectByLiveId(
                    $userInfo->id,
                    $token,
                    $customerId
                );

                $this->messageManager->addSuccess(
                    __('We have discovered you already 
                        have an account at our store. 
                        Your Live account is now connected ' .
                        'to your store account.')
                );

                return $this;
            }
            // New connection - create, attach, login
            $firstName = $userInfo->first_name;
            if (!$firstName) {
                throw new Exception(
                    __('Sorry, could not retrieve your 
                    Live first name. Please try again.')
                );
            }
            $lastName = $userInfo->last_name;
            if (!$lastName) {
                throw new Exception(
                    __('Sorry, could not retrieve
                     your Live last name. Please try again.')
                );
            }

            $this->_liHelper->connectByCreatingAccount(
                $userInfo->id,
                $token,
                $email,
                $userInfo->first_name,
                $userInfo->last_name
            );

            $this->messageManager->addSuccess(
                __('Your Live account is now connected to 
                    your new user account at our store. 
                    Now you can login using ' .
                    'our Live Login button or using 
                    store account credentials 
                    you will receive to your email address.')
            );
        }
    }

    /**
     * @return mixed
     */
    protected function _sendResponse()
    {
        $return = [
            'redirect' => $this->_loginPostRedirect()
        ];

        return json_encode($return);
    }

    /**
     * @return mixed
     */
    protected function _loginPostRedirect()
    {
        $lastCustomerId = $this->_getSession()->getLastCustomerId();
        if (isset(
                $lastCustomerId
            ) && $this->_getSession()
                ->isLoggedIn()
            && $lastCustomerId != $this->_getSession()
                ->getId()
        ) {
            $this->_getSession()
                ->unsBeforeAuthUrl()
                ->setLastCustomerId($this->_getSession()->getId());
        }
        if (!$this->_getSession()->getBeforeAuthUrl() ||
            $this->_getSession()
                ->getBeforeAuthUrl() == $this->_storeManager
                ->getStore()->getBaseUrl()
        ) {
            // Set default URL to redirect customer to
            $this->_getSession()
                ->setBeforeAuthUrl($this->_customerHelperData->getAccountUrl());
            // Redirect customer to the last page visited after logging in
            if ($this->_getSession()->isLoggedIn()) {
                if ($this->_getSession()->getAfterAuthUrl()) {
                    $this->_getSession()
                        ->setBeforeAuthUrl($this->_getSession()
                            ->getAfterAuthUrl(true));
                }
            } else {
                $this->_getSession()
                    ->setBeforeAuthUrl($this->_customerHelperData
                        ->getLoginUrl());
            }
        } elseif ($this->_getSession()
                ->getBeforeAuthUrl() == $this->_customerHelperData
                ->getLogoutUrl()
        ) {
            $this->_getSession()
                ->setBeforeAuthUrl($this->_customerHelperData
                    ->getDashboardUrl());
        } else {
            if (!$this->_getSession()->getAfterAuthUrl()) {
                $this->_getSession()
                    ->setAfterAuthUrl($this->_getSession()->getBeforeAuthUrl());
            }
            if ($this->_getSession()->isLoggedIn()) {
                $this->_getSession()
                    ->setBeforeAuthUrl($this->_getSession()
                        ->getAfterAuthUrl(true));
            }
        }
        return $this->_getSession()->getBeforeAuthUrl(true);
    }

    /**
     * @return \Magento\Customer\Model\Session
     */
    protected function _getSession()
    {
        return $this->_customerSession;
    }
}