<?php
namespace MageArray\SocialLogin\Helper;

use MageArray\SocialLogin\Model\Customerdata;
use Magento\Customer\Model\Customer;
use Magento\Customer\Model\CustomerFactory;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\Request\Http;
use Magento\Framework\HTTP\ZendClientFactory;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Live
 * @package MageArray\SocialLogin\Helper
 */
class Live extends AbstractHelper
{
    /**
     *
     */
    const XML_PATH_CLIENT_ID = 'magearray_social/live/api_key';
    /**
     *
     */
    const XML_PATH_CLIENT_SECRET = 'magearray_social/live/secret';
    /**
     *
     */
    const REDIRECT_URI_ROUTE = 'sociallogin/live/connect';

    /**
     *
     */
    const OAUTH2_TOKEN_URI = 'https://login.live.com/oauth20_token.srf';
    /**
     *
     */
    const OAUTH2_AUTH_URI = 'https://login.live.com/oauth20_authorize.srf';
    /**
     *
     */
    const OAUTH2_SERVICE_URI = 'https://apis.live.net/v5.0/';

    /**
     * @var string
     */
    protected $_state = '';
    /**
     * @var array
     */
    protected $_scope = ['wl.basic', 'wl.emails'];
    /**
     * @var mixed|null
     */
    protected $_clientId = null;
    /**
     * @var mixed|null
     */
    protected $_clientSecret = null;
    /**
     * @var null|string
     */
    protected $_redirectUri = null;
    /**
     * @var string
     */
    protected $_access = 'offline';
    /**
     * @var string
     */
    protected $_prompt = 'auto';
    /**
     * @var string
     */
    protected $_protocol = "http";
    /**
     * @var null
     */
    protected $_token = null;
    /**
     * @var Session
     */
    protected $_customerSession;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var ZendClientFactory
     */
    protected $_httpClientFactory;
    /**
     * @var CustomerFactory
     */
    protected $_customerFactory;
    /**
     * @var Data
     */
    protected $_helperData;
    /**
     * @var Customerdata
     */
    protected $_customerData;

    /**
     * Live constructor.
     * @param Context $context
     * @param ZendClientFactory $httpClientFactory
     * @param Session $customerSession
     * @param \MageArray\SocialLogin\Helper\Data $helperData
     * @param CustomerFactory $customerFactory
     * @param Customerdata $customerData
     */
    public function __construct(
        Context $context,
        ZendClientFactory $httpClientFactory,
        Session $customerSession,
        Data $helperData,
        CustomerFactory $customerFactory,
        Customerdata $customerData,
        Http $request
    ) {
        parent::__construct($context);
        $this->_scopeConfig = $this->scopeConfig;
        $this->_httpClientFactory = $httpClientFactory;
        $this->_customerSession = $customerSession;
        $this->_helperData = $helperData;
        $this->_customerFactory = $customerFactory;
        $this->_customerData = $customerData;
        $this->_request = $request;
        $this->_redirectUri = $this->_urlBuilder->sessionUrlVar(
            $this->_urlBuilder
                ->getUrl(self::REDIRECT_URI_ROUTE, ['_secure' => true])
        );
        $this->_clientSecret = $this->_getClientSecret();
        $this->_clientId = $this->_getClientId();

    }

    /**
     * @param $state
     */
    public function setState($state)
    {
        $this->_state = $state;
    }

    /**
     * @return string
     */
    public function getState()
    {
        return $this->_state;
    }

    /**
     * @return null|string
     */
    public function getRedirectUri()
    {
        return $this->_redirectUri;
    }

    /**
     * @return mixed|null
     */
    public function getClientId()
    {
        return $this->_clientId;
    }

    /**
     * @return mixed|null
     */
    public function getClientSecret()
    {
        return $this->_clientSecret;
    }

    /**
     * @return mixed
     */
    protected function _getClientSecret()
    {
        return $this->getStoreConfig(self::XML_PATH_CLIENT_SECRET);
    }

    /**
     * @return mixed
     */
    protected function _getClientId()
    {
        return $this->getStoreConfig(self::XML_PATH_CLIENT_ID);
    }

    /**
     * @param $path
     * @return mixed
     */
    public function getStoreConfig($path)
    {
        return $this->_scopeConfig
            ->getValue($path, ScopeInterface::SCOPE_STORE);
    }

    /**
     *
     */
    public function redirectUrl()
    {
        $this->_redirectUri = $this->_urlBuilder
            ->sessionUrlVar($this->_urlBuilder
                ->getUrl(self::REDIRECT_URI_ROUTE)
            );
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception
     */
    public function getXoauthLiveGuid()
    {
        if (empty($this->_token)) {
            $this->fetchAccessToken();
        }
        return $this->_token->user_id;
    }

    /**
     * @return string
     */
    public function createRequestUrl()
    {
        $url =
            self::OAUTH2_AUTH_URI . '?' .
            http_build_query(
                [
                    'client_id' => $this->getClientId(),
                    'scope' => implode(' ', $this->_scope),
                    'response_type' => 'code',
                    'redirect_uri' => $this->getRedirectUri(),
                    'state' => $this->getState()
                ]
            );
        return $url;

    }

    /**
     * @param $endpoint
     * @param string $method
     * @param array $params
     * @return mixed
     * @throws \Magento\Framework\Exception
     */
    public function api($endpoint, $method = 'GET', $params = [])
    {
        if (empty($this->_token)) {
            $this->fetchAccessToken();
        } else {
            if ($this->isAccessTokenExpired()) {
                $this->refreshAccessToken();
            }
        }

        $url = self::OAUTH2_SERVICE_URI . $endpoint;
        $method = strtoupper($method);

        $params = array_merge([
            'access_token' => $this->_token->access_token
        ], $params);
        $response = $this->_httpRequest($url, $method, $params);

        return $response;
    }

    /**
     * @throws \Magento\Framework\Exception
     */
    protected function fetchAccessToken()
    {
        if (empty($this->_request->getParam('code'))) {
            throw new \Magento\Framework\Exception(
                __('Unable to retrieve access code.')
            );
        }
        $response = $this->_httpRequest(
            self::OAUTH2_TOKEN_URI,
            'POST',
            [
                'code' => $this->_request->getParam('code'),
                'redirect_uri' => $this->getRedirectUri(),
                'client_id' => $this->getClientId(),
                'client_secret' => $this->getClientSecret(),
                'grant_type' => 'authorization_code'
            ]
        );

        $response->created = time();

        $this->_token = $response;

    }

    /**
     * @throws \Magento\Framework\Exception
     */
    protected function refreshAccessToken()
    {
        if (empty($this->_token->refresh_token)) {
            throw new \Magento\Framework\Exception(
                __('No refresh token, unable to refresh access token')
            );
        }

        $response = $this->_httpRequest(
            self::OAUTH2_TOKEN_URI,
            'POST',
            [
                'client_id' => $this->getClientId(),
                'client_secret' => $this->getClientSecret(),
                'refresh_token' => $this->_token->refresh_token,
                'grant_type' => 'refresh_token'
            ]
        );

        $this->_token->access_token = $response->access_token;
        $this->_token->expires_in = $response->expires_in;
        $this->_token->created = time();
    }

    /**
     * @return bool
     */
    protected function isAccessTokenExpired()
    {
        // If the token is set to expire in the next 30 seconds.
        $expired = ($this->_token
                    ->created + ($this->_token
                        ->expires_in - 30)) < time();

        return $expired;
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception
     */
    public function getAccessToken()
    {
        if (empty($this->_token)) {
            $this->fetchAccessToken();
        }

        return json_encode($this->_token);
    }

    /**
     * @param $url
     * @param string $method
     * @param array $params
     * @return mixed
     * @throws \Magento\Framework\Exception
     * @throws \Zend_Http_Client_Exception
     */
    protected function _httpRequest($url, $method = 'GET', $params = [])
    {
        $client = $this->_httpClientFactory->create();

        $client->setUri($url);
        switch ($method) {
            case 'GET':
                $client->setParameterGet($params);
                break;
            case 'POST':
                $client->setParameterPost($params);
                break;
            case 'DELETE':
                break;
            default:
                throw new \Magento\Framework\Exception(
                    __('Required HTTP method is not supported.')
                );
        }

        $response = $client->request($method);
        $decodedResponse = json_decode($response->getBody());
        if (empty($decodedResponse)) {
            $parsedResponse = [];
            parse_str($response->getBody(), $parsedResponse);

            $decodedResponse = json_decode(json_encode($parsedResponse));
        }

        // echo "<pre>";
        // print_r($decodedResponse);exit;
        if ($response->isError()) {
            $status = $response->getStatus();
            if (($status == 400 || $status == 401)) {
                if (isset($decodedResponse->error->message)) {
                    $message = $decodedResponse->error->message;
                } else {
                    $message = __('Unspecified OAuth error occurred.');
                }

                throw new \Magento\Framework\Exception($message);
            } else {
                $message = sprintf(
                    __('HTTP error %d occurred while issuing request.'),
                    $status
                );

                throw new \Magento\Framework\Exception($message);
            }
        }
        return $decodedResponse;
    }

    /**
     * @param $email
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getCustomersByEmail($email)
    {
        $customer = $this->_customerFactory->create();

        $collection = $customer->getResourceCollection()
            ->addAttributeToSelect('*')
            ->addAttributeToFilter('email', $email)
            ->setPage(1, 1);

        return $collection;
    }

    /**
     * @param $liveId
     * @param $token
     * @param $email
     * @param $firstName
     * @param $lastName
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function connectByCreatingAccount(
        $liveId,
        $token,
        $email,
        $firstName,
        $lastName
    ) {
        $customerDetails = [
            'firstname' => $firstName,
            'lastname' => $lastName,
            'email' => $email,
            'sendemail' => 0,
            'confirmation' => 0
        ];
        $customer = $this->_customerFactory->create();
        $customer->setData($customerDetails);
        $customerData = $customer->getDataModel();
        $customerData->setCustomAttribute('ma_sociallogin_liid', $liveId);
        $customerData->setCustomAttribute('ma_sociallogin_litoken', $token);
        $customer->updateData($customerData);
        $customer->setAddresses(null);
        $customer->save();

        $customer->sendNewAccountEmail('confirmed', '');

        $this->_customerSession->setCustomerAsLoggedIn($customer);
        $this->_customerSession->regenerateId();
    }

    /**
     * @param $liveId
     * @param $token
     * @param $customerId
     */
    public function connectByLiveId($liveId, $token, $customerId)
    {
        $customer = $this->_customerFactory->create();
        $customer->load($customerId);
        $customerData = $customer->getDataModel();
        $customerData->setCustomAttribute('ma_sociallogin_liid', $liveId);
        $customerData->setCustomAttribute('ma_sociallogin_litoken', $token);
        $customer->updateData($customerData);
        $customer->save();
        $this->_customerSession->setCustomerAsLoggedIn($customer);
        $this->_customerSession->regenerateId();
    }

    /**
     * @param Customer $customer
     */
    public function loginByCustomer(Customer $customer)
    {
        if ($customer->getConfirmation()) {
            $customer->setConfirmation(null);
            $customer->save();
        }

        $this->_customerSession->setCustomerAsLoggedIn($customer);
        $this->_customerSession->regenerateId();
    }

    /**
     * @param $liveId
     * @return mixed
     */
    public function getCustomersByLiveId($liveId)
    {
        $customer = $this->_customerFactory->create();

        $collection = $customer->getCollection()
            ->addAttributeToFilter('ma_sociallogin_liid', $liveId)
            ->setPageSize(1);

        return $collection;
    }

}