<?php
namespace MageArray\SocialLogin\Helper;

use MageArray\SocialLogin\Model\Customerdata;
use Magento\Customer\Model\Customer;
use Magento\Customer\Model\CustomerFactory;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\Request\Http;
use Magento\Framework\HTTP\ZendClientFactory;
use Magento\Framework\Oauth\Helper\Request as OauthRequestHelper;
use Magento\Framework\UrlInterface;
use Magento\Store\Model\ScopeInterface;

/**
 * Class Twitter
 * @property \Magento\Backend\Model\Session _session
 * @package MageArray\SocialLogin\Helper
 */
class Twitter extends AbstractHelper
{
    /**
     *
     */
    const REDIRECT_URI_ROUTE = 'sociallogin/twitter/connect';
    /**
     *
     */
    const REQUEST_TOKEN_URI_ROUTE = 'sociallogin/twitter/request';

    /**
     *
     */
    const OAUTH_URI = 'https://api.twitter.com/oauth';
    /**
     *
     */
    const OAUTH2_SERVICE_URI = 'https://api.twitter.com/1.1';

    /**
     *
     */
    const XML_PATH_ENABLED = 'magearray_social/twitter/enabled';
    /**
     *
     */
    const XML_PATH_CLIENT_ID = 'magearray_social/twitter/api_key';
    /**
     *
     */
    const XML_PATH_CLIENT_SECRET = 'magearray_social/twitter/secret';
    /**
     * @var mixed|null
     */
    protected $_clientId = null;
    /**
     * @var mixed|null
     */
    protected $_clientSecret = null;
    /**
     * @var null|string
     */
    protected $_redirectUri = null;
    /**
     * @var null
     */
    protected $_client = null;
    /**
     * @var null
     */
    protected $_token = null;
    /**
     * @var string
     */
    protected $_protocol = "http";
    /**
     * @var Session
     */
    protected $_customerSession;
    /**
     * @var ScopeConfigInterface
     */
    protected $_scopeConfig;
    /**
     * @var ZendClientFactory
     */
    protected $_httpClientFactory;
    /**
     * @var CustomerFactory
     */
    protected $_customerFactory;
    /**
     * @var Data
     */
    protected $_helperData;
    /**
     * @var Customerdata
     */
    protected $_customerData;
    /**
     * @var OauthRequestHelper
     */
    protected $_oauthHelper;

    /**
     * Twitter constructor.
     * @param Context $context
     * @param ZendClientFactory $httpClientFactory
     * @param Session $customerSession
     * @param \MageArray\SocialLogin\Helper\Data $helperData
     * @param CustomerFactory $customerFactory
     * @param Customerdata $customerData
     * @param \Magento\Backend\Model\Session $session
     * @param OauthRequestHelper $oauthHelper
     * @param Http $request
     */
    public function __construct(
        Context $context,
        ZendClientFactory $httpClientFactory,
        Session $customerSession,
        Data $helperData,
        CustomerFactory $customerFactory,
        Customerdata $customerData,
        \Magento\Backend\Model\Session $session,
        OauthRequestHelper $oauthHelper,
        Http $request
    ) {
        parent::__construct($context);
        $this->_scopeConfig = $this->scopeConfig;
        $this->_httpClientFactory = $httpClientFactory;
        $this->_customerSession = $customerSession;
        $this->_helperData = $helperData;
        $this->_customerFactory = $customerFactory;
        $this->_customerData = $customerData;
        $this->_session = $session;
        $this->_oauthHelper = $oauthHelper;
        $this->_request = $request;
        $this->_redirectUri = $this->_urlBuilder->sessionUrlVar(
            $this->_urlBuilder->getUrl(self::REDIRECT_URI_ROUTE,
                ['_secure' => true])
        );
        $this->_clientSecret = $this->_getClientSecret();
        $this->_clientId = $this->_getClientId();
    }

    /**
     *
     */
    public function getClientData()
    {

        $this->_client = new \Zend_Oauth_Consumer(
            [
                'callbackUrl' => $this->getRedirectUri(),
                'siteUrl' => self::OAUTH_URI,
                'authorizeUrl' => self::OAUTH_URI . '/authenticate',
                'consumerKey' => $this->getClientId(),
                'consumerSecret' => $this->getClientSecret()
            ]
        );

    }

    /**
     *
     */
    public function fetchRequestToken()
    {
        $this->getClientData();
        $requestToken = $this->_client->getRequestToken();
        if ($requestToken == false) {
            echo "Connect to https://api.twitter.com is false ! 
                    <br />Error code :" . $this->_client->getLastResponse();
            return;
        }

        $this->_session
            ->setTwitterRequestToken(serialize($requestToken));

        $this->_client->redirect();
    }

    /**
     * @return null
     */
    public function getClient()
    {
        return $this->_client;
    }

    /**
     * @return mixed|null
     */
    public function getClientId()
    {
        return $this->_clientId;
    }

    /**
     * @return mixed|null
     */
    public function getClientSecret()
    {
        return $this->_clientSecret;
    }

    /**
     * @return null|string
     */
    public function getRedirectUri()
    {
        return $this->_redirectUri;
    }

    /**
     * @return mixed
     */
    protected function _getClientSecret()
    {
        return $this->getStoreConfig(self::XML_PATH_CLIENT_SECRET);
    }

    /**
     * @return mixed
     */
    protected function _getClientId()
    {
        return $this->getStoreConfig(self::XML_PATH_CLIENT_ID);
    }

    /**
     * @param $path
     * @return mixed
     */
    public function getStoreConfig($path)
    {
        return $this->_scopeConfig
            ->getValue($path, ScopeInterface::SCOPE_STORE);
    }

    /**
     *
     */
    public function redirectUrl()
    {
        $this->_redirectUri = $this->_urlBuilder
            ->sessionUrlVar($this->_urlBuilder
                ->getUrl(self::REDIRECT_URI_ROUTE));
    }

    /**
     * @return mixed
     * @throws \Magento\Framework\Exception
     */
    public function getAccessToken()
    {
        if (empty($this->_token)) {
            $this->fetchAccessToken();
        }
        return serialize($this->_token);
    }

    /**
     * @throws \Magento\Framework\Exception
     */
    protected function fetchAccessToken()
    {
        $params = $this->_request->getParams();
        if (!($requestToken = $this->_session->getTwitterRequestToken())) {
            throw new \Magento\Framework\Exception(
                __('Unable to retrieve access code.')
            );
        }
        $this->getClientData();
        if (!($token = $this->_client
            ->getAccessToken($params, unserialize($requestToken)))
        ) {
            throw new \Magento\Framework\Exception(
                __('Unable to retrieve access token.')
            );
        }

        $this->_session->unsTwitterRequestToken();
        $this->_token = $token;
    }

    /**
     * @param $endpoint
     * @param string $method
     * @param array $params
     * @return mixed
     * @throws \Magento\Framework\Exception
     */
    public function api($endpoint, $method = 'GET', $params = [])
    {
        if (empty($this->_token)) {
            throw new \Magento\Framework\Exception(
                __('Unable to proceeed without an access token.')
            );
        }

        $url = self::OAUTH2_SERVICE_URI . $endpoint;

        $response = $this->_httpRequest($url, strtoupper($method), $params);

        return $response;
    }

    /**
     * @param $url
     * @param string $method
     * @param array $params
     * @return mixed
     * @throws \Magento\Framework\Exception
     */
    protected function _httpRequest($url, $method = 'GET', $params = [])
    {
        $client = $this->_token->getHttpClient(
            [
                'callbackUrl' => $this->getRedirectUri(),
                'siteUrl' => self::OAUTH_URI,
                'consumerKey' => $this->getClientId(),
                'consumerSecret' => $this->getClientSecret()
            ]
        );
        $client->setUri($url);
        switch ($method) {
            case 'GET':
                $client->setParameterGet($params);
                break;
            case 'POST':
                $client->setParameterPost($params);
                break;
            case 'DELETE':
                break;
            default:
                throw new \Magento\Framework\Exception(
                    __('Required HTTP method is not supported.')
                );
        }

        $response = $client->request($method);

        $decodedResponse = json_decode($response->getBody());

        if (empty($decodedResponse)) {
            $parsedResponse = [];
            parse_str($response->getBody(), $parsedResponse);

            $decodedResponse = json_decode(json_encode($parsedResponse));
        }

        if ($response->isError()) {
            $status = $response->getStatus();
            if (($status == 400 || $status == 401)) {
                if (isset($decodedResponse->error->message)) {
                    $message = $decodedResponse->error->message;
                } else {
                    $message = __('Unspecified OAuth error occurred.');
                }

                throw new \Magento\Framework\Exception($message);
            } else {
                $message = sprintf(
                    __('HTTP error %d occurred while issuing request.'),
                    $status
                );

                throw new \Magento\Framework\Exception($message);
            }
        }
        return $decodedResponse;
    }

    /**
     * @param $twitterId
     * @return mixed
     */
    public function getCustomersByTwitterId($twitterId)
    {
        $customer = $this->_customerFactory->create();

        $collection = $customer->getCollection()
            ->addAttributeToFilter('ma_sociallogin_tid', $twitterId)
            ->setPageSize(1);

        return $collection;
    }

    /**
     * @param $twitterId
     * @param $token
     * @param $customerId
     */
    public function connectByTwitterId($twitterId, $token, $customerId)
    {
        $customer = $this->_customerFactory->create();
        $customer->load($customerId);
        $customerData = $customer->getDataModel();
        $customerData->setCustomAttribute('ma_sociallogin_tid', $twitterId);
        $customerData->setCustomAttribute('ma_sociallogin_ttoken', $token);
        $customer->updateData($customerData);
        $customer->save();
        $this->_customerSession->setCustomerAsLoggedIn($customer);
        $this->_customerSession->regenerateId();
    }

    /**
     * @param Customer $customer
     */
    public function loginByCustomer(Customer $customer)
    {
        if ($customer->getConfirmation()) {
            $customer->setConfirmation(null);
            $customer->save();
        }

        $this->_customerSession->setCustomerAsLoggedIn($customer);
        $this->_customerSession->regenerateId();
    }

    /**
     * @param $email
     * @return mixed
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function getCustomersByEmail($email)
    {
        $customer = $this->_customerFactory->create();

        $collection = $customer->getResourceCollection()
            ->addAttributeToSelect('*')
            ->addAttributeToFilter('email', $email)
            ->setPage(1, 1);

        return $collection;
    }

    /**
     * @param $twitterId
     * @param $token
     * @param $email
     * @param $name
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function connectByCreatingAccount($twitterId, $token, $email, $name)
    {
        $name = explode(' ', $name, 2);
        if (count($name) > 1) {
            $firstName = $name[0];
            $lastName = $name[1];
        } else {
            $firstName = $name[0];
            $lastName = $name[0];
        }

        $customerDetails = [
            'firstname' => $firstName,
            'lastname' => $lastName,
            'email' => $email,
            'sendemail' => 0,
            'confirmation' => 0
        ];
        $customer = $this->_customerFactory->create();
        $customer->setData($customerDetails);
        $customerData = $customer->getDataModel();
        $customerData->setCustomAttribute('ma_sociallogin_tid', $twitterId);
        $customerData->setCustomAttribute('ma_sociallogin_ttoken', $token);
        $customer->updateData($customerData);
        $customer->setAddresses(null);
        $customer->save();

        $customer->sendNewAccountEmail('confirmed', '');

        $this->_customerSession->setCustomerAsLoggedIn($customer);
        $this->_customerSession->regenerateId();
    }

}