<?php
namespace MageArray\SocialLogin\Setup;

use Magento\Customer\Setup\CustomerSetupFactory;
use Magento\Customer\Model\Customer;
use Magento\Eav\Model\Entity\Attribute\Set as AttributeSet;
use Magento\Eav\Model\Entity\Attribute\SetFactory as AttributeSetFactory;
use Magento\Framework\Setup\InstallDataInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;


/**
 * @codeCoverageIgnore
 */
class InstallData implements InstallDataInterface
{
    protected $_customerSetupFactory;
    private $_attributeSetFactory;

    public function __construct(
        CustomerSetupFactory $customerSetupFactory,
        AttributeSetFactory $attributeSetFactory
    ) {
        $this->_customerSetupFactory = $customerSetupFactory;
        $this->_attributeSetFactory = $attributeSetFactory;
    }

    public function install(
        ModuleDataSetupInterface $setup,
        ModuleContextInterface $context
    ) {
        $customerSetup = $this->_customerSetupFactory
            ->create(['setup' => $setup]);
        $customerEntity = $customerSetup->getEavConfig()
            ->getEntityType('customer');
        $attributeSetId = $customerEntity->getDefaultAttributeSetId();

        $attributeSet = $this->_attributeSetFactory->create();
        $attributeGroupId = $attributeSet
            ->getDefaultGroupId($attributeSetId);

        //google setup
        $customerSetup->addAttribute(Customer::ENTITY, 'ma_sociallogin_gid', [
            'type' => 'text',
            'label' => 'Google ID',
            'input' => 'text',
            'required' => false,
            'sort_order' => 1000,
            'position' => 1000,
        ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_gid')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_gtoken', [
                'type' => 'text',
                'label' => 'Google Token',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1001,
                'position' => 1001,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_gtoken')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        //Facebook setup
        $customerSetup->addAttribute(Customer::ENTITY, 'ma_sociallogin_fid', [
            'type' => 'text',
            'label' => 'Facebook ID',
            'input' => 'text',
            'required' => false,
            'sort_order' => 1002,
            'position' => 1002,
        ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_fid')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_ftoken', [
                'type' => 'text',
                'label' => 'Facebook Token',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1003,
                'position' => 1003,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_ftoken')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        //Twitter setup
        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_tid', [
                'type' => 'text',
                'label' => 'Twitter ID',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1004,
                'position' => 1004,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_tid')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_ttoken', [
                'type' => 'text',
                'label' => 'Twitter Token',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1005,
                'position' => 1005,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_ttoken')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        //LinkedIn setup
        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_lid', [
                'type' => 'text',
                'label' => 'LinkedIn ID',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1006,
                'position' => 1006,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_lid')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_ltoken', [
                'type' => 'text',
                'label' => 'LinkedIn Token',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1007,
                'position' => 1007,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_ltoken')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        //Yahoo setup
        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_yid', [
                'type' => 'text',
                'label' => 'Yahoo ID',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1008,
                'position' => 1008,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_yid')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_ytoken', [
                'type' => 'text',
                'label' => 'Yahoo Token',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1009,
                'position' => 1009,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_ytoken')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();
        //Live setup
        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_liid', [
                'type' => 'text',
                'label' => 'Live ID',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1010,
                'position' => 1010,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_liid')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();

        $customerSetup->addAttribute(Customer::ENTITY,
            'ma_sociallogin_litoken', [
                'type' => 'text',
                'label' => 'Live Token',
                'input' => 'text',
                'required' => false,
                'sort_order' => 1011,
                'position' => 1011,
            ]);

        $attribute = $customerSetup->getEavConfig()
            ->getAttribute(Customer::ENTITY, 'ma_sociallogin_litoken')
            ->addData([
                'attribute_set_id' => $attributeSetId,
                'attribute_group_id' => $attributeGroupId,
                'used_in_forms' => [
                    'adminhtml_customer',
                    'checkout_register',
                    'customer_account_create',
                    'customer_account_edit',
                    'adminhtml_checkout'
                ],
                'is_used_for_customer_segment' => true,
                'is_system' => 0,
                'is_visible' => 1,
                'is_user_defined' => 1,
            ]);

        $attribute->save();
    }

}

