<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_AbandonedCart
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\AbandonedCart\Block\Email;

/**
 * Class Template
 * @package Mageplaza\AbandonedCart\Block\Email
 */
class Template extends \Magento\Framework\View\Element\Template
{
    /**
     * @var \Magento\Catalog\Api\ProductRepositoryInterfaceFactory
     */
    private $_productRepositoryFactory;

    /**
     * @var \Magento\Catalog\Helper\Image
     */
    private $imageHelper;

    /**
     * @var \Magento\Framework\Pricing\Helper\Data
     */
    private $pricingHelper;

    /**
     * @var \Mageplaza\AbandonedCart\Helper\Data
     */
    private $helperData;

    /**
     * @param \Magento\Catalog\Block\Product\Context $context
     * @param \Magento\Catalog\Api\ProductRepositoryInterfaceFactory $productRepositoryFactory
     * @param \Magento\Framework\Pricing\Helper\Data $pricingHelper
     * @param \Mageplaza\AbandonedCart\Helper\Data $helperData
     * @param array $data
     */
    public function __construct(
        \Magento\Catalog\Block\Product\Context $context,
        \Magento\Catalog\Api\ProductRepositoryInterfaceFactory $productRepositoryFactory,
        \Magento\Framework\Pricing\Helper\Data $pricingHelper,
        \Mageplaza\AbandonedCart\Helper\Data $helperData,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->_productRepositoryFactory = $productRepositoryFactory;
        $this->imageHelper = $context->getImageHelper();
        $this->pricingHelper = $pricingHelper;
        $this->helperData = $helperData;
    }

    /**
     * Get items in quote
     *
     * @return array
     */
    public function getProductCollection()
    {
        $quote = $this->getQuote();
        $productIds = [];
        if ($quote) {
            foreach ($quote->getAllVisibleItems() as $item) {
                $items[] = $this->_productRepositoryFactory->create()->getById($item->getProductId())->setQtyOrder($item->getQty());
            }
            return $items;
        }
    }

    /**
     * Get subtotal in quote
     *
     * @return string
     */
    public function getSubtotal()
    {
        $subtotal = $this->getQuote()->getSubtotal();
        return $this->pricingHelper->currency($subtotal, true, true);
    }

    /**
     * Get image url in quote
     *
     * @return string
     */
    public function getProductImage($_item)
    {
        return $this->imageHelper->init($_item, 'category_page_grid', ['height' => 100, 'width'=> 100])->getUrl();
    }

    /**
     * Get product url in quote
     *
     * @return string
     */
    public function getProductUrl($_item)
    {
        $productUrl = $_item->getUrlModel()->getUrl($_item);
        if ($this->helperData->getAnalyticsConfig()) {
            $productUrl .= $this->helperData->getAnalyticsConfig();
        }
        return $productUrl;
    }

    /**
     * Get checkout url
     *
     * @return string
     */
    public function getCheckoutUrl()
    {
        $cartLink = $this->getQuote()->getCartLink();
        if ($this->helperData->getAnalyticsConfig()) {
            $cartLink .= $this->helperData->getAnalyticsConfig();
        }
        return $cartLink;
    }

    /**
     * Get item price in quote
     *
     * @return string
     */
    public function getProductPrice($_item)
    {
        $productPrice = $_item->getPriceInfo()->getPrice('final_price')->getAmount()->getValue();
        return $this->pricingHelper->currency($productPrice, true, false);
    }
}
