<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_AbandonedCart
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\AbandonedCart\Controller\Checkout;

/**
 * Class Cart
 * @package Mageplaza\AbandonedCart\Controller\Checkout
 */
class Cart extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Mageplaza\AbandonedCart\Model\Token
     */
    private $tokenModel;

    /**
     * @var \Magento\Quote\Model\QuoteFactory
     */
    private $quoteFactory;

    /**
     * @var \Magento\Customer\Model\SessionFactory
     */
    private $customerSession;

    /**
     * @var \Magento\Checkout\Model\SessionFactory
     */
    private $checkoutSession;

    /**
     * @var \Mageplaza\AbandonedCart\Model\LogsFactory
     */
    private $logsFactory;

    /**
     * @var \Mageplaza\AbandonedCart\Helper\Data
     */
    private $helperData;

    /**
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Quote\Model\QuoteFactory $quoteFactory
     * @param \Mageplaza\AbandonedCart\Model\Token $tokenModel
     * @param \Mageplaza\AbandonedCart\Model\LogsFactory $logsFactory
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Mageplaza\AbandonedCart\Helper\Data $helperData
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Quote\Model\QuoteFactory $quoteFactory,
        \Mageplaza\AbandonedCart\Model\Token $tokenModel,
        \Mageplaza\AbandonedCart\Model\LogsFactory $logsFactory,
        \Magento\Customer\Model\SessionFactory $customerSession,
        \Mageplaza\AbandonedCart\Helper\Data $helperData,
        \Magento\Checkout\Model\SessionFactory $checkoutSession
    ) {
        parent::__construct($context);
        $this->tokenModel = $tokenModel;
        $this->quoteFactory = $quoteFactory;
        $this->customerSession = $customerSession;
        $this->logsFactory = $logsFactory;
        $this->helperData = $helperData;
        $this->checkoutSession = $checkoutSession;
    }

    /**
     * Recovery cart by cart link
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function execute()
    {
        if ($this->helperData->isEnabled()) {
            $token = $this->getRequest()->getParam('token');
            $quoteId = (int) $this->getRequest()->getParam('id');
            $resultRedirect = $this->resultRedirectFactory->create();
            $logModel = $this->logsFactory->create();
            if (!$this->tokenModel->validateCartLink($quoteId, $token)) {
                $this->messageManager->addError(__('You can\'t used link'));
            } else {
                $quote = $this->quoteFactory->create()->load($quoteId);
                if ($quote->getCustomerId()) {
                    $customerId = $quote->getCustomerId();
                    $customerModel = $this->customerSession->create();
                    if (!$customerModel->isLoggedIn()) {
                        try {
                            if ($customerModel->loginById($customerId)) {
                                $customerModel->regenerateId();
                                $logModel->updateRecovery($quoteId);
                                $this->messageManager->addSuccess(__('Recovery Success.'));
                            }
                        } catch (\Exception $exception) {
                            $this->messageManager->addError(__('You can\'t login.'));
                        }
                    } else {
                        if ($customerModel->getId() != $customerId) {
                            $this->messageManager->addNotice(__('Please login with %1', $quote->getCustomerEmail()));
                        }
                    }
                } else {
                    $this->checkoutSession->create()->setQuoteId($quote->getId());
                }
            }
        }
        return $this->_redirect('checkout/cart');
    }
}
