<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_AbandonedCart
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\AbandonedCart\Model;

use Magento\Framework\App\Area;
use Magento\Framework\Math\Random;

/**
 * Class AbandonedCart
 * @package Mageplaza\AbandonedCart\Model
 */
class AbandonedCart
{
    /**
     * @var \Magento\Quote\Model\QuoteFactory
     */
    private $quoteFactory;

    /**
     * @var \Mageplaza\AbandonedCart\Helper\Data
     */
    private $abandonedCartData;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * Date model
     *
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    private $date;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    private $transportBuilder;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    private $customerFactory;

    /**
     * @var \Magento\SalesRule\Api\Data\CouponGenerationSpecInterfaceFactory
     */
    private $generationSpecFactory;

    /**
     * @var \Magento\SalesRule\Model\Service\CouponManagementService
     */
    private $couponManagementService;

    /**
     * @var Random
     */
    private $mathRandom;

    /**
     * @var \Mageplaza\AbandonedCart\Model\Token
     */
    private $abandonedCartToken;

    /**
     * @var \Mageplaza\AbandonedCart\Model\LogsFactory
     */
    private $abandonedCartLogs;

    /**
     * Object manager
     *
     * @var \Magento\Framework\ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var \Magento\SalesRule\Model\CouponFactory
     */
    private $couponFactory;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\Timezone
     */
    private $templateFactory;

    /**
     * @var \Magento\SalesRule\Api\RuleRepositoryInterface
     */
    private $ruleRepositoryInterface;

    /**
     * @param \Magento\Quote\Model\QuoteFactory $quoteFactory
     * @param \Mageplaza\AbandonedCart\Helper\Data $abandonedCartData
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param Random $mathRandom
     * @param \Magento\Customer\Model\CustomerFactory $customerFactory
     * @param \Mageplaza\AbandonedCart\Model\Token $abandonedCartToken
     * @param \Mageplaza\AbandonedCart\Model\LogsFactory $abandonedCartLogs
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\SalesRule\Model\Service\CouponManagementService $couponManagementService
     * @param \Magento\SalesRule\Api\Data\CouponGenerationSpecInterfaceFactory $generationSpecFactory
     * @param \Magento\SalesRule\Model\CouponFactory $couponFactory
     * @param \Magento\Framework\Mail\Template\FactoryInterface $templateFactory
     * @param \Magento\SalesRule\Api\RuleRepositoryInterface $ruleRepositoryInterface
     */
    public function __construct(
        \Magento\Quote\Model\QuoteFactory $quoteFactory,
        \Mageplaza\AbandonedCart\Helper\Data $abandonedCartData,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        Random $mathRandom,
        \Magento\Customer\Model\CustomerFactory $customerFactory,
        \Mageplaza\AbandonedCart\Model\Token $abandonedCartToken,
        \Mageplaza\AbandonedCart\Model\LogsFactory $abandonedCartLogs,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\SalesRule\Model\Service\CouponManagementService $couponManagementService = null,
        \Magento\SalesRule\Api\Data\CouponGenerationSpecInterfaceFactory $generationSpecFactory = null,
        \Magento\SalesRule\Model\CouponFactory $couponFactory,
        \Magento\Framework\Mail\Template\FactoryInterface $templateFactory,
        \Magento\SalesRule\Api\RuleRepositoryInterface $ruleRepositoryInterface
    ) {
        $this->objectManager = $objectManager;
        $this->quoteFactory = $quoteFactory;
        $this->abandonedCartData = $abandonedCartData;
        $this->date = $date;
        $this->logger = $logger;
        $this->storeManager = $storeManager;
        $this->transportBuilder = $transportBuilder;
        $this->customerFactory = $customerFactory;
        $this->mathRandom = $mathRandom;
        $this->abandonedCartToken = $abandonedCartToken;
        $this->abandonedCartLogs = $abandonedCartLogs;
        $this->generationSpecFactory = $generationSpecFactory ?:
            $this->objectManager->get(\Magento\SalesRule\Api\Data\CouponGenerationSpecInterfaceFactory::class);
        $this->couponManagementService = $couponManagementService ?:
            $this->objectManager->get(\Magento\SalesRule\Model\Service\CouponManagementService::class);
        $this->couponFactory = $couponFactory;
        $this->templateFactory = $templateFactory;
        $this->ruleRepositoryInterface = $ruleRepositoryInterface;
    }

    /**
     * @return void
     */
    public function prepareForAbandonedCart()
    {
        $current = strtotime($this->date->date());
        $quoteCollection = $this->quoteFactory->create()->getCollection();
        $configs = $this->abandonedCartData->getEmailConfig();
        $lastSend = $current - (int)array_values($configs)[0]['send'] - 86400;
        $quoteCollection->addFieldToFilter(
                'items_count',
                ['neq' => '0']
            )->addFieldToFilter(
                'is_active',
                ['eq' => true]
            )->addFieldToFilter(
                'customer_email',
                ['neq' => null]
            )->addFieldToFilter(
                ['created_at', 'updated_at'],
                [
                    ['gteq' => $this->date->date('Y-m-d H:i:s', $lastSend)],
                    ['gteq' => $this->date->date('Y-m-d H:i:s', $lastSend)],
                ]
            )->setOrder(
                'updated_at'
            );
        if (!empty($configs)) {
            foreach ($quoteCollection as $quote) {
                $quoteUpdatedTime = strtotime($quote->getUpdatedAt());
                if ($quoteUpdatedTime < 0) {
                    $quoteUpdatedTime = strtotime($quote->getCreatedAt());
                }
                $quoteId = $quote->getId();
                foreach ($configs as $configId => $config) {
                    $validateEmail = $this->abandonedCartToken->validateEmail($quoteId, $configId);
                    $time = $quoteUpdatedTime + $config['send'];
                    if ($validateEmail && $time <= $current) {
                        $couponCode = '';
                        $toDate = '';
                        if ($config['coupon']) {
                            $coupon = $this->createCoupon();
                        }
                        if (!empty($coupon)) {
                            $couponCode = $coupon['coupon_code'];
                            $toDate = $coupon['to_date'];
                        }
                        $newCartToken = $this->mathRandom->getUniqueHash();
                        $this->sendMail($quote, $config, $newCartToken, $couponCode, $toDate);
                        $this->abandonedCartToken->saveToken($quoteId, $configId, $newCartToken);
                    }
                }
            }
        }
    }

    /**
     * Send abandoned cart email
     *
     * @param \Magento\Quote\Model\Quote $quote
     * @param array $config
     * @param string $newCartToken
     * @param string | null $couponCode
     * @param string | null $toDate
     * @return void
     */
    public function sendMail($quote, $config, $newCartToken, $couponCode = null, $toDate = null)
    {
        $success = true;
        if (!$quote->getCustomerIsGuest()) {
            $customer = $this->customerFactory->create()->load($quote->getCustomerId());
            $customerEmail = $customer->getEmail();
            $customerName = $customer->getName();
        } else {
            $customerEmail = $quote->getCustomerEmail();
            $customerName = $quote->getFirstname() . ' ' . $quote->getLastname();
        }
        try {
            $store = $this->storeManager->getStore($quote->getStoreId());
            $quote->setCartLink(
                $this->abandonedCartData->getUrlEmail(
                    'abandonedcart/checkout/cart',
                    [
                        '_nosid' => true,
                        'id' => $quote->getId(),
                        'token' => $newCartToken
                    ]
                )
            );
            $vars = [
                'store' => $store,
                'quote' => $quote,
                'customer_name' => ucfirst($customerName),
                'coupon_code' => $couponCode,
                'to_date' => $toDate,
                'sender' => $config['sender']
            ];
            $transport = $this->transportBuilder->setTemplateIdentifier($config['template'])
                ->setTemplateOptions(['area' => Area::AREA_FRONTEND, 'store' => $store->getId()])
                ->setTemplateVars($vars)
                ->setFrom($config['sender'])
                ->addTo($customerEmail, $customerName)
                ->getTransport();
            $emailBody = $this->templateFactory
                ->get(
                    $config['template']
                )->setVars($vars)->setOptions(
                    ['area' => Area::AREA_FRONTEND, 'store' => $store->getId()]
                )->processTemplate();
            $transport->sendMessage();
        } catch (\Exception $e) {
            $success = false;
            $this->logger->error($e->getMessage());
        }
        $this->abandonedCartLogs->create()->saveLogs($config, $quote, $customerEmail, $customerName, $config['sender'], $emailBody, $success, $couponCode);
    }

    /**
     * @param \Mageplaza\AbandonedCart\Model\Logs $log
     * @return void
     */
    public function sendAgain($log = null)
    {
        if ($log != null) {
            try {
                $store = $this->storeManager->getStore();
                $this->transportBuilder->setTemplateIdentifier('send_again')
                    ->setTemplateOptions(['area' => Area::AREA_FRONTEND, 'store' => $store->getId()])
                    ->setTemplateVars(
                        [
                            'body' => htmlspecialchars_decode($log->getEmailContent()),
                            'subject' => $log->getSubject()
                        ]
                    )
                    ->setFrom($log->getSender())
                    ->addTo($log->getCustomerEmail(), $log->getCustomerName())
                    ->getTransport()
                    ->sendMessage();
                $log->setUpdatedAt($this->date->date())
                    ->setStatus(true);
            } catch (\Exception $e) {
                $this->logger->error($e->getMessage());
            }
            $log->setSequentNumber($log->getSequentNumber() + 1)->save();
        }
    }

    /**
     * Generate Coupon Code by Configuration
     *
     * @return array
     */
    private function createCoupon()
    {
        $helper = $this->abandonedCartData;
        $coupon = [];
        $data = [];
        if ($helper->getCouponConfig('length') && $helper->getCouponConfig('rule')) {
            $length = (int) $helper->getCouponConfig('length');
            if (!$length) {
                $length = 5;
            }
            $data = [
                        'rule_id' => (int) $helper->getCouponConfig('rule'),
                        'quantity' => 1,
                        'length' => $length,
                        'format' => $helper->getCouponConfig('format'),
                        'prefix' => $helper->getCouponConfig('prefix'),
                        'suffix' => $helper->getCouponConfig('suffix'),
                        'dash' => (int) $helper->getCouponConfig('dash')
                    ];
            $couponSpec = $this->generationSpecFactory->create(['data' => $data]);
            $couponCodes = $this->couponManagementService->generate($couponSpec);
            $couponCode = $couponCodes[0];
            $valid = (int) $helper->getCouponConfig('valid');
            $coupon = $this->couponFactory->create()->loadByCode($couponCode);
            if ($valid) {
                $expirationDate = strtotime($this->date->date()) + $valid * 3600;
                if (!$coupon->getExpirationDate() || ($coupon->getExpirationDate() && strtotime($coupon->getExpirationDate()) > $expirationDate)) {
                    $coupon->setExpirationDate($this->date->date('Y-m-d H:i:s', $expirationDate))->save();
                }
            }
            if ($couponCode) {
                if ($coupon->getExpirationDate()) {
                    $toDate = $coupon->getExpirationDate();
                }
                $coupon = [
                            'coupon_code' => $couponCode,
                            'to_date' => $toDate
                            ];
            }
        }
        return $coupon;
    }
}
