<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_AbandonedCart
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\AbandonedCart\Model;

/**
 * Class Logs
 * @package Mageplaza\AbandonedCart\Model
 */
class Logs extends \Magento\Framework\Model\AbstractModel
{
    /**
     * @var \Magento\Email\Model\TemplateFactory
     */
    private $templateFactory;

    /**
     * Construct
     *
     * @param \Magento\Backend\Block\Template\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Email\Model\TemplateFactory $templateFactory
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Email\Model\TemplateFactory $templateFactory
    ) {
        $this->templateFactory = $templateFactory;
        parent::__construct(
            $context,
            $registry
        );
    }

    /**
     * @return void
     */
    public function _construct()
    {
        parent::_construct();
        $this->_init('Mageplaza\AbandonedCart\Model\ResourceModel\Logs');
    }

    /**
     * @param array $config
     * @param \Magento\Quote\Model\Quote $quote
     * @param string $customerEmail
     * @param string $customerName
     * @param string $sender
     * @param string $body
     * @param bool $status
     * @param string $couponCode
     * @return void
     */
    public function saveLogs($config, $quote, $customerEmail, $customerName, $sender, $body, $status = false, $couponCode = null)
    {
        $subject = '';
        preg_match('/<title>(.*?)<\/title>/s',htmlspecialchars_decode($body), $matches);
        if (!empty($matches)) {
            $subject = $matches[1];
        }
        $this->setSubject($subject)
            ->setCustomerEmail($customerEmail)
            ->setCouponCode($couponCode)
            ->setQuoteId($quote->getId())
            ->setSender($sender)
            ->setCustomerName($customerName)
            ->setSequentNumber(1)
            ->setEmailContent(htmlspecialchars($body))
            ->setStatus($status)
            ->save();
    }

    /**
     *
     * @param int $quoteId
     * @return mixed
     */
    public function updateRecovery($quoteId)
    {
        return $this->_getResource()->updateRecovery($quoteId);
    }

    /**
     *
     * @return mixed
     */
    public function clear()
    {
        return $this->_getResource()->clear();
    }
}
