<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_AutoRelated
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\AutoRelated\Controller\Ajax;

use Magento\Backend\App\Action\Context;
use Magento\Catalog\Model\SessionFactory;
use Magento\Framework\App\Action\Action;
use Magento\Framework\Json\Helper\Data;
use Magento\Framework\View\Result\PageFactory;
use Mageplaza\AutoRelated\Helper\Data as HelperData;
use Mageplaza\AutoRelated\Model\ResourceModel\RuleFactory;
use Psr\Log\LoggerInterface;

/**
 * Class Load
 * @package Mageplaza\AutoRelated\Controller\Ajax
 */
class Load extends Action
{
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    private $resultPageFactory;

    /**
     * @var \Magento\Framework\Json\Helper\Data
     */
    private $jsonHelper;

    /**
     * @var \Magento\Catalog\Model\SessionFactory
     */
    private $catalogSession;

    /**
     * @var \Mageplaza\AutoRelated\Helper\Data
     */
    private $helperData;

    /**
     * @var \Mageplaza\AutoRelated\Model\RuleFactory
     */
    private $autoRelatedModelRule;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;

    /**
     * Load constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Json\Helper\Data $jsonHelper
     * @param \Mageplaza\AutoRelated\Helper\Data $helperData
     * @param \Magento\Catalog\Model\SessionFactory $catalogSession
     * @param \Mageplaza\AutoRelated\Model\RuleFactory $autoRelatedModelRule
     */
    public function __construct(
        Context $context,
        PageFactory $resultPageFactory,
        LoggerInterface $logger,
        Data $jsonHelper,
        HelperData $helperData,
        SessionFactory $catalogSession,
        \Mageplaza\AutoRelated\Model\RuleFactory $autoRelatedModelRule
    )
    {
        parent::__construct($context);

        $this->jsonHelper           = $jsonHelper;
        $this->resultPageFactory    = $resultPageFactory;
        $this->catalogSession       = $catalogSession;
        $this->helperData           = $helperData;
        $this->autoRelatedModelRule = $autoRelatedModelRule;
        $this->logger               = $logger;
    }

    /**
     * Execute view action
     *
     * @return \Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        if (!$this->helperData->isEnabled()) {
            return false;
        }

        $result     = [];
        $ruleIds    = [];
        $id         = '';
        $resultPage = $this->resultPageFactory->create();
        $params     = $this->getRequest()->getParams();
        if ($params['module'] == 'catalog' && $params['controller'] == 'product' && $params['action'] == 'view' && $params['product_id']) {
            $pageType = 'product';
            $id       = $params['product_id'];
        } elseif ($params['module'] == 'catalog' && $params['controller'] == 'category' && $params['action'] == 'view' && $params['category_id']) {
            $pageType = 'category';
            $id       = $params['category_id'];
        } elseif ($params['module'] == 'checkout' && $params['controller'] == 'cart' && $params['action'] == 'index') {
            $pageType = 'cart';
        } else {
            $pageType = 'cms';
            if (isset($params['ruleIds'])) {
                $ruleIds = array_filter(explode(',', $params['ruleIds']));
                foreach ($ruleIds as $ruleId) {
                    $ruleParam = $this->autoRelatedModelRule->create()->load($ruleId);
                    if ($ruleParam->hasChild()) {
                        $ruleChild = $ruleParam->getChild();
                        $ruleIds[] = $ruleChild['rule_id'];
                    }
                    if ($ruleParam->getParentId()) {
                        $ruleIds[] = $ruleParam->getParentId();
                    }
                }
            }
        }

        try {
            /** @var \Mageplaza\AutoRelated\Model\ResourceModel\Rule $autoRelatedRule */
            $autoRelatedRule = $this->autoRelatedModelRule->create()->getResource();
            $data            = $autoRelatedRule->getProductList($pageType, $id, $ruleIds);
            if (!empty($data)) {
                $i = 0;
                foreach ($data as $location => $infos) {
                    $html                     = '';
                    $result['data'][$i]['id'] = $location;
                    foreach ($infos as $info) {
                        $productIds = $info['product_ids'];
                        $rule       = $info['rule'];
                        $ruleId     = $rule['rule_id'];
                        if (!empty($productIds)) {
                            $html .= $resultPage->getLayout()
                                ->createBlock('Mageplaza\AutoRelated\Block\Product\ProductList\ProductList')
                                ->setTemplate('Mageplaza_AutoRelated::product/list/items.phtml')
                                ->setProductIds($productIds)
                                ->setRequestDefault($params)
                                ->setRule($rule)
                                ->toHtml();
                            $autoRelatedRule->updateImpression($pageType, $id, $ruleId);
                        }
                    }
                    $result['data'][$i]['content'] = $html;
                    $i++;
                }
            }
            if (!empty($result)) {
                $result['status'] = true;
                $this->catalogSession->create()->unsAutoRelated();
            }
        } catch (\Exception $e) {
            $result['status'] = false;
            $this->logger->critical($e);
        }
        $this->getResponse()->representJson(
            $this->jsonHelper->jsonEncode($result)
        );
    }
}
