<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_FrequentlyBought
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\FrequentlyBought\Controller\Cart;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Checkout\Model\Cart as CustomerCart;

/**
 * Class Add
 * @package Mageplaza\FrequentlyBought\Controller\Cart
 */
class Add  extends \Magento\Checkout\Controller\Cart\Add
{
    /**
     * @var \Mageplaza\FrequentlyBought\Helper\Data
     */
    private $fbtDataHelper;

    /**
     * @var \Magento\Framework\Escaper
     */
    private $escaper;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Magento\Checkout\Helper\Cart
     */
    private $cartHelper;

    /**
     * Constructor
     *
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\Data\Form\FormKey\Validator $formKeyValidator
     * @param CustomerCart $cart
     * @param ProductRepositoryInterface $productRepository
     * @param \Mageplaza\FrequentlyBought\Helper\Data $fbtDataHelper
     * @param \Psr\Log\LoggerInterface $logger
     * @param \Magento\Framework\Escaper $escaper
     * @param \Magento\Checkout\Helper\Cart $cartHelper
     */
    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Data\Form\FormKey\Validator $formKeyValidator,
        CustomerCart $cart,
        ProductRepositoryInterface $productRepository,
        \Mageplaza\FrequentlyBought\Helper\Data $fbtDataHelper,
        \Psr\Log\LoggerInterface $logger,
        \Magento\Framework\Escaper $escaper,
        \Magento\Checkout\Helper\Cart $cartHelper
    ) {
        parent::__construct(
            $context,
            $scopeConfig,
            $checkoutSession,
            $storeManager,
            $formKeyValidator,
            $cart,
            $productRepository
        );
        $this->fbtDataHelper = $fbtDataHelper;
        $this->storeManager = $storeManager;
        $this->escaper = $escaper;
        $this->logger = $logger;
        $this->cartHelper = $cartHelper;
        $this->productRepository = $productRepository;
    }

    /**
     * Add product to shopping cart action
     *
     * @return \Magento\Framework\Controller\Result\Redirect
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function execute()
    {
        if (!$this->fbtDataHelper->getConfig()) {
            return $this->resultRedirectFactory->create()->setPath('*/*/');
        }
        $storeId = $this->storeManager->getStore()->getId();
        $params = $this->getRequest()->getParams();
        try {
            if (!empty($params['mageplaza_fbt'])) {
                $productsName = [];
                foreach ($params['mageplaza_fbt'] as $productId => $value) {
                    $paramsFbt = [];
                    $product = $this->productRepository->getById($productId, false, $storeId);
                    $productType = $product->getTypeId();
                    $paramsFbt['product'] = $productId;
                    switch ($productType) {
                        case 'configurable' :
                            if (isset($params['super_attribute']) && isset($params['super_attribute'][$productId])) {
                                $paramsFbt['super_attribute'] = $params['super_attribute'][$productId];
                            }
                            break;
                        case 'grouped':
                            if (isset($params['super_group']) && isset($params['super_group'][$productId])) {
                                $paramsFbt['super_group'] = $params['super_group'][$productId];
                            }
                            break;
                        case 'bundle':
                            if (isset($params['bundle_option']) && isset($params['bundle_option'][$productId])) {
                                $paramsFbt['bundle_option'] = $params['bundle_option'][$productId];
                                $paramsFbt['bundle_option_qty'] = $params['bundle_option_qty'][$productId];
                            }
                            break;
                        case 'downloadable':
                            if (isset($params['links']) && isset($params['links'][$productId])) {
                                $paramsFbt['links'] = $params['links'][$productId];
                            }
                            break;
                        default:
                            break;
                    }
                    if (isset($params['options_'.$productId])) {
                        $paramsFbt['options'] = $params['options_'.$productId];
                    }
                    $productsName[] = '"' . $product->getName() . '"';
                    $this->cart->addProduct($product, $paramsFbt);
                }
                $this->cart->save();
                if (!$this->_checkoutSession->getNoCartRedirect(true)) {
                    if (!$this->cart->getQuote()->getHasError()) {
                        $message = __(
                            'You added %1 to your shopping cart.', join(', ', $productsName)
                        );
                        $this->messageManager->addSuccessMessage($message);
                    }
                    return $this->goBack(null, $product);
                }
            } else {
                $this->messageManager->addError(
                    $this->escaper->escapeHtml(__('Please select product(s).'))
                );
            }
        } catch (\Magento\Framework\Exception\LocalizedException $e) {
            if ($this->_checkoutSession->getUseNotice(true)) {
                $this->messageManager->addNotice(
                    $this->escaper->escapeHtml($e->getMessage())
                );
            } else {
                $messages = array_unique(explode("\n", $e->getMessage()));
                foreach ($messages as $message) {
                    $this->messageManager->addError(
                        $this->escaper->escapeHtml($message)
                    );
                }
            }

            $url = $this->_checkoutSession->getRedirectUrl(true);

            if (!$url) {
                $cartUrl = $this->cartHelper->getCartUrl();
                $url = $this->_redirect->getRedirectUrl($cartUrl);
            }

            return $this->goBack($url);

        } catch (\Exception $e) {
            $this->messageManager->addException($e, __('We can\'t add this item to your shopping cart right now.'));
            $this->logger->critical($e);
            return $this->goBack();
        }
    }
}
