<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_FrequentlyBought
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\FrequentlyBought\Helper;

/**
 * Class Data
 * @package Mageplaza\FrequentlyBought\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
	/**
	 * @var \Magento\Framework\App\Config\ScopeConfigInterface
	 */
	protected $scopeConfig;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    /**
     * @var \Magento\Framework\Filesystem
     */
    private $filesystem ;

    /**
     * @var \Magento\Framework\Image\AdapterFactory
     */
    private $imageFactory;

	/**
	 * Constructor
	 *
	 * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\Filesystem $filesystem
     * @param \Magento\Framework\Image\AdapterFactory $imageFactory
	 */
	public function __construct(
        \Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Filesystem $filesystem,
        \Magento\Framework\Image\AdapterFactory $imageFactory
	) {
		parent::__construct($context);
		$this->scopeConfig = $context->getScopeConfig();
        $this->storeManager = $storeManager;
        $this->filesystem = $filesystem;
        $this->imageFactory = $imageFactory;
	}

	/**
	 * Get Config
	 *
     * @param string $field
	 * @param null $storeId
	 * @return mixed
	 */
	public function getConfig($field = 'enabled', $storeId = null)
    {
        if ($this->scopeConfig->getValue('frequentlybought/general/enabled')) {
            return $this->scopeConfig->getValue('frequentlybought/general/' . $field,
                \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $storeId);
        }
        return false;
    }

    /**
     * Get Separator Image Config
     *
     * @return string
     */
    public function getIcon()
    {
        $icon = $this->getConfig('separator_image');
        if (!$icon) {
            return false;
        }
        return $this->resize($icon, 30);
    }

    /**
     * Resize Image Config
     *
     * @param string $image
     * @param int $width
     * @param null $height
     * @return string
     */
    private function resize($image, $width = null, $height = null)
    {
        $absolutePath = $this->filesystem->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)->getAbsolutePath('mageplaza/frequentlybought/').$image;

        $imageResized = $this->filesystem->getDirectoryRead(\Magento\Framework\App\Filesystem\DirectoryList::MEDIA)->getAbsolutePath('mageplaza/frequentlybought/resized/'.$width.'/').$image;
        //create image factory...
        $imageResize = $this->imageFactory->create();
        $imageResize->open($absolutePath);
        $imageResize->constrainOnly(true);
        $imageResize->keepTransparency(true);
        $imageResize->keepFrame(false);
        $imageResize->keepAspectRatio(true);
        $imageResize->resize($width,$height);
        //destination folder
        $destination = $imageResized ;
        //save image
        $imageResize->save($destination);

        $resizedURL = $this->storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA).'mageplaza/frequentlybought/resized/'.$width.'/'.$image;
        return $resizedURL;
    }
}
