<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://www.mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category   Mageplaza
 * @package    Mageplaza_PdfInvoice
 * @copyright   Copyright (c) 2017-2018 Mageplaza (http://www.mageplaza.com/)
 * @license     https://www.mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\PdfInvoice\Controller\Adminhtml\Template;

use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Filesystem;
use Mageplaza\PdfInvoice\Helper\PrintProcess as HelperData;
use Mageplaza\PdfInvoice\Model\Source\Type;

/**
 * Class Load
 * @package Mageplaza\PdfInvoice\Controller\Adminhtml\Template
 */
class Load extends Action
{
    /**
     * @type \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var \Magento\Framework\Filesystem
     */
    protected $filesystem;

    /**
     * @var \Mageplaza\PdfInvoice\Helper\Data
     */
    protected $helperData;

    /**
     * Load constructor.
     * @param Context $context
     * @param Filesystem $filesystem
     * @param JsonFactory $resultJsonFactory
     * @param HelperData $helperData
     */
    public function __construct(
        Context $context,
        Filesystem $filesystem,
        JsonFactory $resultJsonFactory,
        HelperData $helperData
    )
    {
        $this->filesystem        = $filesystem;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helperData        = $helperData;

        parent::__construct($context);
    }

    /**
     * @return $this|\Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface
     */
    public function execute()
    {
        $templateHtml = $templateCss = '';
        $templateType = $this->getRequest()->getParam('templateType', Type::INVOICE);
        $templateId   = $this->getRequest()->getParam('templateId', 'template1');

        try {
            $templateHtml = $this->helperData->getDefaultTemplateHtml($templateType, $templateId);
            $templateCss  = $this->helperData->getDefaultTemplateCss($templateType, $templateId);
            $status       = true;
            $message      = __('Load message success!');
        } catch (\Exception $e) {
            $status  = false;
            $message = __("Cannot load template.");
        }

        /** @var \Magento\Framework\Controller\Result\Json $result */
        $result = $this->resultJsonFactory->create();

        return $result->setData(['status' => $status, 'message' => $message, 'templateHtml' => $templateHtml, 'templateCss' => $templateCss]);
    }
}

