<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_ReviewBooster
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\ReviewBooster\Helper;

/**
 * Class Data
 * @package Mageplaza\ReviewBooster\Helper
 */
class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    const ANALYTICS_ENDABLED = 'reviewbooster/analytics/enabled';

    const ANALYTICS_SOURCE = 'reviewbooster/analytics/source';

    const ANALYTICS_MEDIUM = 'reviewbooster/analytics/medium';

    const ANALYTICS_NAME = 'reviewbooster/analytics/name';

    const ANALYTICS_TERM = 'reviewbooster/analytics/term';

    const ANALYTICS_CONTENT = 'reviewbooster/analytics/content';

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var \Magento\Framework\Url
     */
    private $urlHelper;

	/**
	 * Constructor
	 *
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Framework\Url $urlHelper
	 */
	public function __construct(
		\Magento\Framework\App\Helper\Context $context,
        \Magento\Framework\Url $urlHelper
	) {
		parent::__construct($context);
        $this->scopeConfig = $context->getScopeConfig();
        $this->urlHelper = $urlHelper;
	}

    /**
     * Get Email Config
     *
     * @return array
     */
    public function getEmailConfig()
    {
        $emailConfig = $this->scopeConfig->getValue('reviewbooster/general/email');
        if ($emailConfig) {
            $configs = unserialize($emailConfig);
            foreach ($configs as $configId => $config) {
                if ($config['send']) {
                    $configSeconds = 0;
                    $configTimes = explode(' ', $config['send']);
                    foreach ($configTimes as $configTime) {
                        if (strpos($configTime, 'd') !== false) {
                            $configSeconds += (int) str_replace('d', '', $configTime) * 24 * 60 * 60;
                        }
                        if (strpos($configTime, 'h') !== false) {
                            $configSeconds += (int) str_replace('h', '', $configTime) * 60 * 60;
                        }
                        if (strpos($configTime, 'm') !== false) {
                            $configSeconds += (int) str_replace('m', '', $configTime)  * 60;
                        }
                    }
                    $configs[$configId]['send'] = $configSeconds;
                    $send[$configId] = $configSeconds;
                }
            }
            array_multisort($send, SORT_DESC, $configs);
            return $configs;
        }
        return [];
    }

    /**
     *
     * @return string
     */
    public function getAnalyticsConfig()
    {
        $suffix = '';
        $source = $this->scopeConfig->getValue(self::ANALYTICS_SOURCE);
        $medium = $this->scopeConfig->getValue(self::ANALYTICS_MEDIUM);
        $name = $this->scopeConfig->getValue(self::ANALYTICS_NAME);
        $term = $this->scopeConfig->getValue(self::ANALYTICS_TERM);
        $content = $this->scopeConfig->getValue(self::ANALYTICS_CONTENT);
        if ($this->scopeConfig->getValue(self::ANALYTICS_ENDABLED) && $source) {
            $suffix .= '?utm_source=' . $source;
            if ($medium) {
                $suffix .= '&utm_medium=' . $medium;
            }
            if ($name) {
                $suffix .= '&utm_campaign=' . $name;
            }
            if ($term) {
                $suffix .= '&utm_term=' . $term;
            }
            if ($content) {
                $suffix .= '&utm_content=' . $content;
            }
        }
        return $suffix;
    }

    /**
     *
     * @return bool
     */
    public function isEnabled()
    {
        return (bool) $this->scopeConfig->getValue('reviewbooster/general/enabled');
    }

    public function getOrderConfig($field)
    {
        $orderConfig = $this->scopeConfig->getValue('reviewbooster/order/' . $field);
        if ($orderConfig) {
            $skus = explode(', ', $orderConfig);
            return $skus;
        }
        return false;
    }

    /**
     * Get Url For Email
     *
     * @param string $routePath
     * @param array $routeParams
     * @return string
     */
    public function getUrlEmail($routePath, $routeParams = [])
    {
        return $this->urlHelper->setUseSession(false)->getUrl($routePath, $routeParams);
    }
}
