<?php
/**
 * Mageplaza
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the mageplaza.com license that is
 * available through the world-wide-web at this URL:
 * https://mageplaza.com/LICENSE.txt
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade this extension to newer
 * version in the future.
 *
 * @category    Mageplaza
 * @package     Mageplaza_ReviewBooster
 * @copyright   Copyright (c) 2017 Mageplaza (https://www.mageplaza.com/)
 * @license     http://mageplaza.com/LICENSE.txt
 */

namespace Mageplaza\ReviewBooster\Model\ResourceModel;

/**
 * Class Logs
 * @package Mageplaza\ReviewBooster\Model\ResourceModel
 */
class Logs extends \Magento\Framework\Model\ResourceModel\Db\AbstractDb
{
    /**
     * Date model
     *
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    private $date;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\Timezone
     */
    private $timeZone;

    /**
     * @var Magento\Framework\App\ResourceConnection
     */
    private $resourceConnection;

    /**
     * constructor
     *
     * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
     * @param \Magento\Framework\Model\ResourceModel\Db\Context $context
     * @param \Magento\Framework\Stdlib\DateTime\Timezone $timeZone
     */
    public function __construct(
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Magento\Framework\Model\ResourceModel\Db\Context $context,
        \Magento\Framework\Stdlib\DateTime\Timezone $timeZone
    ) {
        $this->date = $date;
        $this->timeZone = $timeZone;
        $this->resourceConnection = $context->getResources();
        parent::__construct($context);
    }

    /**
     * Initialize resource model
     *
     * @return void
     */
    public function _construct()
    {
        $this->_init('mageplaza_reviewbooster_logs', 'id');
    }

    /**
     * before save callback
     *
     * @param \Magento\Framework\Model\AbstractModel|\Mageplaza\ReviewBooster\Model\Logs $object
     * @return $this
     */
    protected function _beforeSave(\Magento\Framework\Model\AbstractModel $object)
    {
        if ($object->isObjectNew()) {
            $object->setCreatedAt($this->date->date());
        }
        $object->setUpdatedAt($this->date->date());
        return parent::_beforeSave($object);
    }

    /**
     *
     * @param string $date
     * @return string
     */
    private function convertDate($date)
    {
        return $this->date->date('Y-m-d H:i:s', strtotime($date));
    }

    /**
     *
     * @param string $fromDate
     * @param string $toDate
     * @param string|null $dimension
     * @return array
     */
    public function loadReportData($fromDate, $toDate, $dimension = null)
    {
        $result = [];
        $timeZoneFrom = $this->timeZone->date($fromDate);
        $timeZoneTo = $this->timeZone->date($toDate);
        $timeDiff = $timeZoneFrom->diff($timeZoneTo);
        if ($dimension == 'month') {
            $numbers = $timeDiff->m;
            $level = ' month';
            if ($numbers == 0 && $this->date->date('m',$fromDate) != $this->date->date('m',$toDate)) {
                $numbers = 1;
            }
        } else {
            $level = ' days';
            $numbers = $timeDiff->days;
        }
        for ($number = 0; $number <= $numbers; $number++) {
            if ($dimension == 'month') {
                $fromDate = $this->date->date('m/01/Y',$fromDate);
            }
            $date = $this->date->date('m/d/Y', $fromDate . '+'.$number.$level);
            $nextDate = $this->date->date('m/d/Y', $date . '+1'. $level);
            $dateFormat = $date;
            if ($dimension == 'month') {
                $date = $this->date->date('m/01/Y',$date);
                $dateFormat = $this->date->date('m/Y', $date);
            }
            $result[] = [
                            $dateFormat,
                            $this->getLogData($date, $nextDate, 'sent'),
                            $this->getLogData($date, $nextDate, 'error')
                        ];
        }
        return $result;
    }

    /**
     *
     * @return void
     */
    public function clear()
    {
        $bind = ['display' => false];
        $this->getConnection()->update($this->getMainTable(), $bind);
    }

    /**
     *
     * @param string $date
     * @param string $nextDate
     * @param string $column
     * @return int
     */
    private function getLogData($date, $nextDate, $column)
    {
        $adapter = $this->resourceConnection->getConnection();
        $select = $adapter->select()
            ->from($this->getMainTable())
            ->where('updated_at >= ?', $this->convertDate($date))
            ->where('updated_at < ?', $this->convertDate($nextDate));
        if ($column == 'error') {
            $select->where('status = ?', false);
        } else {
            $select->where('status = ?', 1);
        }
        $collection = $adapter->fetchCol($select);
        return count($collection);
    }
}
