<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Block\Adminhtml\System\Config\Fieldset;

use Magento\Backend\Block\Template;
use Magento\Framework\Data\Form\Element\Renderer\RendererInterface;
use Magento\Framework\Data\Form\Element\AbstractElement;
use Magento\Backend\Block\Template\Context;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Module\ModuleList\Loader;
use Mageside\BackupToDropbox\Helper\Backup as Helper;

class Hint extends Template implements RendererInterface
{
    /**
     * @var string
     */
    protected $_template = 'Mageside_BackupToDropbox::system/config/fieldset/hint.phtml';

    /**
     * @var ProductMetadataInterface
     */
    protected $_metaData;

    /**
     * @var Loader
     */
    protected $_loader;

    /**
     * @var Helper
     */
    protected $_helper;

    /**
     * Hint constructor.
     * @param Context $context
     * @param ProductMetadataInterface $productMetaData
     * @param Loader $loader
     * @param Helper $helper
     * @param array $data
     */
    public function __construct(
        Context $context,
        ProductMetadataInterface $productMetaData,
        Loader $loader,
        Helper $helper,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->_metaData = $productMetaData;
        $this->_loader = $loader;
        $this->_helper = $helper;
    }

    /**
     * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return mixed
     */
    public function render(AbstractElement $element)
    {
        return $this->toHtml();
    }

    /**
     * @return mixed
     */
    public function getModuleName()
    {
        return $this->_helper->getConfigModule('XML_MODULE_NAME');
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        $modules = $this->_loader->load();
        $v = "";
        if (isset($modules['Mageside_BackupToDropbox'])) {
            $v = "v" . $modules['Mageside_BackupToDropbox']['setup_version'];
        }
        
        return $v;
    }

    /**
     * @return mixed
     */
    public function getModulePage()
    {
        return $this->_helper->getConfigModule('XML_MODULE_PAGE_LINK');
    }
}
