<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Block\Adminhtml\System\Config\Form\Field;

use Magento\Framework\Convert\DataObject;

class Buttons extends \Magento\Config\Block\System\Config\Form\Field
{
    const BUTTON_TEMPLATE = 'system/config/button/button.phtml';

    protected $_buttonsCollection = [];

    /**
     * Set template to itself
     *
     * @return $this
     */
    protected function _prepareLayout()
    {
        parent::_prepareLayout();
        if (!$this->getTemplate()) {
            $this->setTemplate(static::BUTTON_TEMPLATE);
        }
        return $this;
    }
    /**
     * Render button
     *
     * @param  \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    public function render(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        // Remove scope label
        $element->unsScope()->unsCanUseWebsiteValue()->unsCanUseDefaultValue();
        return parent::render($element);
    }
    /**
     * Get the button and scripts contents
     *
     * @param \Magento\Framework\Data\Form\Element\AbstractElement $element
     * @return string
     */
    protected function _getElementHtml(\Magento\Framework\Data\Form\Element\AbstractElement $element)
    {
        $this->_addButton(
            'button-to-check',
            $this->_urlBuilder->getUrl('backuptodropbox/action/checkConnection'),
            'Check Connection',
            'mageside-button-left'
        );
        $this->_addButton(
            'button-to-backup',
            $this->_urlBuilder->getUrl('backuptodropbox/action/run'),
            'Run Backup',
            'mageside-button-right'
        );
        return $this->_toHtml();
    }

    /**
     * @return array
     */
    public function getButtons()
    {
        return $this->_buttonsCollection;
    }

    /**
     * @param $id
     * @param $ajaxUrl
     * @param $label
     * @param null $class
     */
    private function _addButton($id, $ajaxUrl, $label, $class = null)
    {
        $button = new \Magento\Framework\DataObject();
        $button->addData(
            [
                'id'            => $id,
                'button_label'  => __($label),
                'class'         => $class,
                'ajax_url'      => $ajaxUrl
            ]
        );
        $this->_buttonsCollection[] = $button;
    }
}
