<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Controller\Adminhtml\Action;

use Magento\Framework\Controller\Result\JsonFactory;
use Mageside\BackupToDropbox\Helper\Backup as Helper;
use Mageside\BackupToDropbox\Model\Dropbox as BackupToDropbox;

class CheckConnection extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Mageside_BackupToDropbox::config_mageside_backuptodropbox';

    /**
     * @var Helper
     */
    protected $_helper;

    /**
     * @var BackupToDropbox
     */
    private $_dropboxModel;

    /**
     * @var JsonFactory
     */
    protected $_resultJsonFactory;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * Run constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Helper $helper
     * @param BackupToDropbox $dropboxModel
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        JsonFactory $resultJsonFactory,
        Helper $helper,
        BackupToDropbox $dropboxModel,
        \Psr\Log\LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->_helper = $helper;
        $this->_resultJsonFactory = $resultJsonFactory;
        $this->_dropboxModel = $dropboxModel;
        $this->_logger = $logger;
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $resultJson */
        $resultJson = $this->_resultJsonFactory->create();
        try {
            $accessToken = $this->_helper->getConfigModule('XML_PATH_BACKUP_ACCESS_TOKEN');

            if (!isset($accessToken)) {
                $result = [['type' => 'error',
                    'message' => __('Please enter and save your DropBox Access Token.')
                ]];
            } else {
                $this->_dropboxModel->checkConnection();
                $result = $this->_dropboxModel->getMessages();
            }
        } catch (\Exception $e) {
            $result = [['type' => 'error', 'message' => $e->getMessage()]];
            $this->_logger->info($e);
        }
        $resultJson->setData($result);
        return $resultJson;
    }
}
