<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Controller\Adminhtml\Action;

use Magento\Framework\Controller\Result\JsonFactory;
use Mageside\BackupToDropbox\Helper\Backup as Helper;
use Magento\Cron\Model\Schedule as CronSchedule;
use Magento\Framework\Stdlib\DateTime\DateTime as DateTime;
use Mageside\BackupToDropbox\Model\Dropbox as BackupToDropbox;

class Run extends \Magento\Backend\App\Action
{
    /**
     * Authorization level of a basic admin session
     *
     * @see _isAllowed()
     */
    const ADMIN_RESOURCE = 'Mageside_BackupToDropbox::config_mageside_backuptodropbox';

    /**
     * @var Helper
     */
    protected $_helper;

    /**
     * @var DateTime
     */
    private $_date;

    /**
     * @var BackupToDropbox
     */
    private $_dropboxModel;

    /**
     * @var CronSchedule
     */
    private $_cronSchedule;

    /**
     * @var JsonFactory
     */
    protected $_resultJsonFactory;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $_logger;

    /**
     * Run constructor.
     * @param \Magento\Backend\App\Action\Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Helper $helper
     * @param DateTime $dateTime
     * @param CronSchedule $cronSchedule
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(
        \Magento\Backend\App\Action\Context $context,
        JsonFactory $resultJsonFactory,
        Helper $helper,
        DateTime $dateTime,
        CronSchedule $cronSchedule,
        BackupToDropbox $dropbox,
        \Psr\Log\LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->_resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_date = $dateTime;
        $this->_cronSchedule = $cronSchedule;
        $this->_dropboxModel = $dropbox;
        $this->_logger = $logger;
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {
        /** @var \Magento\Framework\Controller\Result\Json $resultJson */
        $resultJson = $this->_resultJsonFactory->create();
        try {
            $accessToken = $this->_helper->getConfigModule('XML_PATH_BACKUP_ACCESS_TOKEN');
            $running = $this->_dropboxModel->isRunning();
            if (!isset($accessToken)) {
                $result = [['type' => 'error',
                    'message' => __('Please enter and save your DropBox Access Token and then Run.')
                ]];
            } elseif ($running) {
                $result = [['type' => 'error',
                    'message' => __('Backup to DropBox is already running!')
                ]];
            } else {
                $this->_cronSchedule->addData($this->prepareData());
                $this->_cronSchedule->save();
                $result = [[
                    'type' => 'success',
                    'message' => __('Backup to Dropbox will be started by cron within 5 minutes')
                ]];
            }
            $resultJson->setData($result);
        } catch (\Exception $e) {
            $result = ['type' => 'error', 'message' => $e->getMessage()];
            $resultJson->setData($result);
            $this->_logger->info($e);
        }
        return $resultJson;
    }

    /**
     * @return array
     */
    private function prepareData()
    {
        $date = $this->_date->date();
        return [
            'job_code'  => 'system_backup_to_dropbox',
            'status'    => 'pending',
            'messages'  => 'Force launch of backup to Dropbox',
            'scheduled_at' => $date
        ];
    }
}
