<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\Filesystem;
use Magento\Framework\App\Filesystem\DirectoryList;
use Mageside\BackupToDropbox\Helper\UiFormHelper;
use \Magento\Backend\Model\UrlInterface;
use Magento\Framework\App\MaintenanceMode;

class Backup extends AbstractHelper
{
    /**
     * @var array
     */
    protected $_optionsArray = [
        'XML_MODULE_NAME' => 'general/module_name',
        'XML_MODULE_PAGE_LINK' => 'general/module_page_link',
        'XML_PATH_ENABLED' => 'general/enabled',
        'XML_PATH_BACKUP_TYPE' => 'general/type',
        'XML_PATH_BACKUP_MAINTENANCE_MODE' => 'general/maintenance',
        'XML_PATH_BACKUP_CLIENT_ID' => 'general/dropbox_app_key',
        'XML_PATH_BACKUP_CLIENT_SECRET' => 'general/dropbox_app_secret',
        'XML_PATH_BACKUP_ACCESS_TOKEN' => 'general/dropbox_token',
        'XML_PATH_BACKUP_SEND_EMAIL' => 'notifications/send_mail',
        'XML_PATH_BACKUP_EMAIL' => 'notifications/email',
        'XML_PATH_BACKUP_DELETE_OLD' => 'clean_backup/delete_old',
        'XML_PATH_BACKUP_AFTER_DAYS' => 'clean_backup/after_days',
        'XML_DB_BACKUP_FORMAT' => 'general/db_backup_format',
        'XML_DB_TABLE_IGNORE_LIST' => 'backup_ignore_list/table_ignore_list',
        'XML_FOLDER_IGNORE_LIST' => 'backup_ignore_list/folder_ignore_list'
    ];

    /**
     * @var Filesystem
     */
    protected $_filesystem;
    /**
     * @var \Mageside\BackupToDropbox\Helper\UiFormHelper
     */
    protected $_uiFormHelper;

    /**
     * @var \Magento\Backend\Model\UrlInterface
     */
    public $_backendUrl;

    /**
     * @var \Magento\Framework\Unserialize\Unserialize
     */
    protected $_unserialize;

    /**
     * @var Filesystem\Directory\ReadInterface
     */
    protected $_directoryRoot;

    /**
     * Backup constructor.
     * @param Filesystem $filesystem
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Mageside\BackupToDropbox\Helper\UiFormHelper $uiFormHelper
     * @param \Magento\Framework\Unserialize\Unserialize $unserialize
     */
    public function __construct(
        Filesystem $filesystem,
        \Magento\Framework\App\Helper\Context $context,
        \Mageside\BackupToDropbox\Helper\UiFormHelper $uiFormHelper,
        \Magento\Framework\Unserialize\Unserialize $unserialize,
        \Magento\Backend\Model\UrlInterface $helper
    ) {
        parent::__construct($context);
        $this->_filesystem = $filesystem;
        $this->_uiFormHelper = $uiFormHelper;
        $this->_backendUrl = $helper;
        $this->_directoryRoot = $filesystem->getDirectoryRead(DirectoryList::ROOT);
        $this->_unserialize = $unserialize;
    }

    /**
     * @return bool
     */
    public function isBackupToDropboxEnabled()
    {
        if ($this->scopeConfig->getValue('mageside_backuptodropbox/general/enabled')
            && $this->isModuleOutputEnabled("Mageside_BackupToDropbox")
        ) {
            return true;
        }
        return false;
    }

    /**
     * @param $key
     * @return mixed
     */
    public function getConfigModule($key)
    {
        if (!empty($this->_optionsArray)) {
            $optionsArray = $this->_optionsArray;
            if (array_key_exists($key, $optionsArray)) {
                $path = $optionsArray[$key];

                return $this->scopeConfig->getValue('mageside_backuptodropbox/' . $path);
            }
        }
    }

    /**
     * @param $backupManager
     * @return mixed
     */
    public function getBackupIgnoreFolderPaths($backupManager)
    {
        $ignoreFolderList = [];
        if ($this->getConfigModule('XML_FOLDER_IGNORE_LIST')) {
            $folderIgnoreOptions = $this->getConfigModule('XML_FOLDER_IGNORE_LIST');
            $dataFromDb = $this->_unserialize->unserialize($folderIgnoreOptions);
            if (is_array($dataFromDb)
                && !array_key_exists('folder_ignore_list', $dataFromDb)
            ) {
                return $backupManager;
            } else {
                $folders = $this->_unserialize->unserialize($folderIgnoreOptions)['folder_ignore_list'];
                $rootDirectory = $this->_directoryRoot->getAbsolutePath();
                foreach ($folders as $value) {
                    $value = ltrim($value, "/");
                    $ignoreFolderList[] = $rootDirectory . $value;
                }
                $backupManager->addIgnorePaths($ignoreFolderList);
            }
        }
        return $backupManager;
    }

    /**
     * @return array
     */
    public function getIgnoreTableList()
    {
        $ignoreTableList = [];
        if ($this->getConfigModule('XML_DB_TABLE_IGNORE_LIST')) {
            $dataFromDb = $this->_unserialize->unserialize($this->getConfigModule('XML_DB_TABLE_IGNORE_LIST'));
            if (is_array($dataFromDb)
                && !array_key_exists('table_ignore_list', $dataFromDb)
            ) {
                return $ignoreTableList;
            } else {
                $tables = $dataFromDb['table_ignore_list'];
                foreach ($tables as $value) {
                    $value = ltrim($value, "/");
                    $ignoreTableList[] = $value;
                }
            }
        }

        return $ignoreTableList;
    }

    /**
     * @return mixed
     */
    public function getSender()
    {
        if (empty($this->getConfigModule('XML_PATH_BACKUP_EMAIL'))) {
            return $this->scopeConfig->getValue('trans_email/ident_support/email');
        }
        return $this->getConfigModule('XML_PATH_BACKUP_EMAIL');
    }

    /**
     * @param string $route
     * @param array $params
     * @return string
     */
    public function getUrl($route = '', $params = [])
    {
        return $this->_backendUrl->getUrl($route, $params);
    }
}
