<?php
/**
 * Pmclain_Twilio extension
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the GPL v3 License
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://www.gnu.org/licenses/gpl.txt
 *
 * @category       Pmclain
 * @package        Twilio
 * @copyright      Copyright (c) 2017
 * @license        https://www.gnu.org/licenses/gpl.txt GPL v3 License
 */

namespace Natz\AccountVerify\Helper;

use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;


class Data extends AbstractHelper
{
    const GENERAL_CONFIG_PATH = 'account_verification/general/';
    const TWILIO_CONFIG_PATH = 'account_verification/twilio/';
    const B2B_CONFIG_PATH = 'account_verification/b2bsms/';
    const EMAIL_CONFIG_PATH = 'account_verification/email_notification/';
    const NEXMO_CONFIG_PATH = 'account_verification/nexmo/';

    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $scopeConfig;

    protected $mathRandom;

    /**
     * Data constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Math\Random $mathRandom,
        ScopeConfigInterface $scopeConfig
    ) {
        parent::__construct($context);
        $this->scopeConfig = $scopeConfig;
        $this->mathRandom = $mathRandom;
    }

    /**
     * @return bool
     */
    public function isSMSEnabled()
    {
        if ($this->scopeConfig->getValue(
            self::GENERAL_CONFIG_PATH . 'is_active',
            ScopeInterface::SCOPE_STORE
        )) {
            return true;
        }
        return false;
    }

    /**
     * @return string
     */
    public function getLicense()
    {
        return $this->scopeConfig->getValue(
            self::GENERAL_CONFIG_PATH . 'license',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getSMSTemplate()
    {
        return $this->scopeConfig->getValue(
            self::GENERAL_CONFIG_PATH . 'sms_template',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getSMSGateway()
    {
        return $this->scopeConfig->getValue(
            self::GENERAL_CONFIG_PATH . 'sms_gateway',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getAccountSid()
    {
        return $this->scopeConfig->getValue(
            self::TWILIO_CONFIG_PATH . 'account_sid',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getAccountAuthToken()
    {
        return $this->scopeConfig->getValue(
            self::TWILIO_CONFIG_PATH . 'account_token',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getTwilioPhone()
    {
        return $this->scopeConfig->getValue(
            self::TWILIO_CONFIG_PATH . 'twilio_number',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getTwilioAdminPhone()
    {
        return $this->scopeConfig->getValue(
            self::TWILIO_CONFIG_PATH . 'admin_phone_number',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getB2BSMSHost()
    {
        return $this->scopeConfig->getValue(
            self::B2B_CONFIG_PATH . 'sms_host',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getB2BUserId()
    {
        return $this->scopeConfig->getValue(
            self::B2B_CONFIG_PATH . 'sms_userid',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getB2BPassword()
    {
        return $this->scopeConfig->getValue(
            self::B2B_CONFIG_PATH . 'sms_password',
            ScopeInterface::SCOPE_STORE
        );
    }

    /**
     * @return string
     */
    public function getRandomNumber()
    {
        return $this->mathRandom->getRandomNumber();
    }


    /**
     * @return string
     */
    public function getRandomString($length = 5)
    {
        // $chars = \Magento\Framework\Math\Random::CHARS_LOWERS
        //     . \Magento\Framework\Math\Random::CHARS_UPPERS
        //     . \Magento\Framework\Math\Random::CHARS_DIGITS;
        $chars = \Magento\Framework\Math\Random::CHARS_DIGITS;


        return $ver_code = $this->mathRandom->getRandomString($length, $chars);
    }

        /**
    * @return string
     */
    public function getActiveIsAdminEmail()
    {
        return $this->scopeConfig->getValue(
            self::EMAIL_CONFIG_PATH . 'is_admin_email',
            ScopeInterface::SCOPE_STORE
        );
    }
    
     /**
    * @return string
     */
    public function getAdminEmail()
    {
        return $this->scopeConfig->getValue(
            self::EMAIL_CONFIG_PATH . 'admin_email',
            ScopeInterface::SCOPE_STORE
        );
    }
    
     /**
    * @return string
     */
    public function getAdminEmailTemplate()
    {
        return $this->scopeConfig->getValue(
            self::EMAIL_CONFIG_PATH . 'admin_email_template',
            ScopeInterface::SCOPE_STORE
        );
    }
     /**
    * @return string
     */
    public function getIsActiveCustomerEmail()
    {
        return $this->scopeConfig->getValue(
            self::EMAIL_CONFIG_PATH . 'is_customer_email',
            ScopeInterface::SCOPE_STORE
        );
    }    

     /**
    * @return string
     */
    public function getCustomerEmailTemplate()
    {
        return $this->scopeConfig->getValue(
            self::EMAIL_CONFIG_PATH . 'customer_email_template',
            ScopeInterface::SCOPE_STORE
        );
    }    

    public function getStoreEmail()
    {
        return $this->scopeConfig->getValue(
            'trans_email/ident_general/email',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }
    public function getStorename()
    {
        return $this->scopeConfig->getValue(
            'trans_email/ident_general/name',
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

    public function getNexmoApiKey()
    {
        return $this->scopeConfig->getValue(
            self::NEXMO_CONFIG_PATH . 'api_key',
            ScopeInterface::SCOPE_STORE
        );
    }

    public function getNexmoApiSecret()
    {
        return $this->scopeConfig->getValue(
            self::NEXMO_CONFIG_PATH . 'api_secret',
            ScopeInterface::SCOPE_STORE
        );
    }

    public function getNexmoPhoneNumber()
    {
        return $this->scopeConfig->getValue(
            self::NEXMO_CONFIG_PATH . 'number',
            ScopeInterface::SCOPE_STORE
        );
    }
}