<?php
namespace Natz\AccountVerify\Model;


class EmailProcess
{

    protected $_helper;

    protected $_messageParser;

    protected $transportBuilder;

    protected $storeManager;

    protected $inlineTranslation;

    public function __construct(
        \Natz\AccountVerify\Helper\Data $helper,
        \Natz\AccountVerify\Helper\MessageTemplateParser $messageParser,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation

    ) {
        $this->_helper = $helper;
        $this->_messageParser = $messageParser;
        $this->_transportBuilder = $transportBuilder;
        $this->_storeManager = $storeManager;
        $this->inlineTranslation = $inlineTranslation;
    }


    private function getStore()
    {
        return $this->_storeManager->getStore();
    }


    private function generateTemplate($emailTemplate,$receiverInfo,$customerInfo)
    {


        $senderInfo = [
            'name' => $this->_helper->getStorename(),
            'email' => $this->_helper->getStoreEmail()
        ];

        

        $postObject = new \Magento\Framework\DataObject();
        $postObject->setData($customerInfo);
        $this->_helper->getCustomerEmailTemplate();

        $template =  $this->_transportBuilder->setTemplateIdentifier($emailTemplate)
                ->setTemplateOptions(
                    [
                        'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
                        'store' => $this->_storeManager->getStore()->getId(),
                    ]
                )
                ->setTemplateVars(['data' => $postObject])
                ->setFrom($senderInfo)
                ->addTo($receiverInfo['email'],$receiverInfo['name']);
                return $this;        
    }

    public function sendEmailToAdmin($customerInfo)
    {
        $receiverInfo = [
            'name' => $this->_helper->getStorename(),
            'email' => $this->_helper->getAdminEmail()
        ];

        $adminEmailTemplate = $this->_helper->getAdminEmailTemplate();
        $this->inlineTranslation->suspend();    
        $this->generateTemplate($adminEmailTemplate,$receiverInfo,$customerInfo);    
        $transport = $this->_transportBuilder->getTransport();
        $transport->sendMessage();        
        $this->inlineTranslation->resume();
    }


    public function sendEmailToCustomer($customerInfo)
    {
        $receiverInfo = [
            'name' => $customerInfo['name'],
            'email' => $customerInfo['email']
        ];

        $CustomerEmailTemplate = $this->_helper->getCustomerEmailTemplate();
        $this->inlineTranslation->suspend();    
        $this->generateTemplate($CustomerEmailTemplate,$receiverInfo,$customerInfo);    
        $transport = $this->_transportBuilder->getTransport();
        $transport->sendMessage();        
        $this->inlineTranslation->resume();
    }

}