<?php

namespace Natz\AccountVerify\Model;



use Magento\Mtf\Util\Command\Cli;

use Twilio\Rest\Client;

use Nexmo\Client as NexmoClient;

require_once '/home/chase/public_html/vendor/twilio/sdk/Twilio/autoload.php';

class SmsSend

{



    protected $_helper;



    protected $_messageParser;



    protected $_countryCode;



    public function __construct(

        \Natz\AccountVerify\Model\Smslogs $sms,

        \Natz\AccountVerify\Helper\Data $helper,

        \Natz\AccountVerify\Helper\CountryCode $country_code,

        \Natz\AccountVerify\Helper\MessageTemplateParser $messageParser

    ) {

        $this->_sms = $sms;

        $this->_helper = $helper;

        $this->_messageParser = $messageParser;

        $this->_countryCode = $country_code;

    }



    /**

     * @param $code

     * @param $mobileno

     * @param $countryCode

     * @return array

     */

    public function sendVerificationCode($code,$mobileno,$countryCode) {



        switch ($this->_helper->getSMSGateway()) {

            case 'b2bsms': $response = $this->sendB2BSms($code,$mobileno,$countryCode); break;

            case 'twilio': $response = $this->sendTwilioSms($code,$mobileno,$countryCode);break;

            case 'nexmo': $response = $this->sendNexmoSms($code,$mobileno,$countryCode);break;

            default: $response = $this->sendB2BSms($code,$mobileno,$countryCode);break;

        }

        return $response;

    }



    private function sendB2BSms($code,$mb,$countryCode)

    {   



        $host = trim($this->_helper->getB2BSMSHost());

        $username = trim($this->_helper->getB2BUserId());

        $password = trim($this->_helper->getB2BPassword());

        $message = trim($this->_helper->getSMSTemplate());

        

        $areacode = $this->_countryCode->getCountryCallingCode($countryCode);

        $mobileNumber = $this->_messageParser->updateCountryCode($areacode,$mb,10);



        $vars = [];

        $vars['code'] = $code;

        

        $template = $this->_messageParser->filterTemplate($this->_messageParser->parseTemplate($message, $vars));



        $mobileno =  $this->_messageParser->filterMobile($mobileNumber,'b2b');



        $url = $host."?userid=".$username."&pwd=".$password."&msg=".$template."&mobileno=". $mobileno;

        $response =$this->execCurl($url);



        $result= array($response,$mobileno);

        return $result;





    }



    private function execCurl($url)

    {

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, $url);

        curl_setopt($ch, CURLOPT_HEADER, 0);

        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

        $response = curl_exec($ch);



        curl_close($ch);

        return $response;

    }



    protected function sendTwilioSms($code,$mb,$countryCode)

    {

        $message = trim($this->_helper->getSMSTemplate());



        $areacode = $this->_countryCode->getCountryCallingCode($countryCode);

        $mobileNumber = $this->_messageParser->updateCountryCode($areacode,$mb,10);

        $vars = [];

        $vars['code'] = $code;



        $template = $this->_messageParser->parseTemplate($message, $vars);



        $template = htmlspecialchars($template);

        $mobileno =  $this->_messageParser->filterMobile($mobileNumber,$mode='twilio');



        $client = new Client($this->_helper->getAccountSid(), $this->_helper->getAccountAuthToken());



        $result = $client->messages->create(

                $mobileno,

                [

                    'from' => $this->_helper->getTwilioPhone(),

                    'body' => $template

                ]

            );



        return $result;



    }



    protected function sendNexmoSms($code,$mb,$countryCode)

    {

        $message = trim($this->_helper->getSMSTemplate());



        $areacode = $this->_countryCode->getCountryCallingCode($countryCode);

        $mobileNumber = $this->_messageParser->updateCountryCode($areacode,$mb,10);

        $vars = [];

        $vars['code'] = $code;



        $template = $this->_messageParser->parseTemplate($message, $vars);



        $template = htmlspecialchars($template);



        $mobileno =  $this->_messageParser->filterMobile($mobileNumber);



        /**

         * @var NexmoClient $client

         */

        $client = new NexmoClient(new \Nexmo\Client\Credentials\Basic($this->_helper->getNexmoApiKey(), $this->_helper->getNexmoApiSecret()));



        $result = $client->message()->send(

            [

                'to'    => $mobileno,

                'from' => $this->_helper->getNexmoPhoneNumber(),

                'text' => $template

            ]

        );



        return $result;



    }

}