<?php

namespace Potato\PerformanceMonitoring\Block\Adminhtml\Form\Tab;

use Magento\Backend\Block\Template;
use Potato\PerformanceMonitoring\Model\Chart;
use Potato\PerformanceMonitoring\Model\ResourceModel\Memory\CollectionFactory as MemoryCollectionFactory;
use Magento\Backend\Block\Widget\Tab\TabInterface;

/**
 * Class Memory
 */
class Memory extends \Magento\Backend\Block\Widget implements TabInterface
{
    protected $_template = 'Potato_PerformanceMonitoring::tab_memory.phtml';
    
    /** @var Chart  */
    protected $chart;

    /** @var MemoryCollectionFactory  */
    protected $memoryCollectionFactory;

    /**
     * Memory constructor.
     * @param Template\Context $context
     * @param Chart $chart
     * @param MemoryCollectionFactory $memoryCollectionFactory
     * @param array $data
     */
    public function __construct(
        Template\Context $context,
        Chart $chart,
        MemoryCollectionFactory $memoryCollectionFactory,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->chart = $chart;
        $this->memoryCollectionFactory = $memoryCollectionFactory;
    }

    /**
     * Memory Chart
     *
     * @return mixed
     */
    public function getDataJson()
    {
        $groupedDataFlag = false;
        if ($this->chart->canGroup()) {
            $groupedDataFlag = true;
        }

        /** @var \Potato\PerformanceMonitoring\Model\ResourceModel\Memory\Collection $collection */
        $collection = $this->memoryCollectionFactory->create();
        $collection
            ->setDateFilter(
                $this->chart->getDateFrom(),
                $this->chart->getDateTo()
            )
            ->orderByDate()
        ;
        $data = [
            [
                [
                    'label' => __('DateTime'),
                    'type' => 'datetime'
                ],
                __('Memory(Mb)'),
                __('Swap(Mb)')
            ],
        ];
        if ($groupedDataFlag) {
            $collection->groupByHour();
            $data = [
                [
                    [
                        'label' => __('DateTime'),
                        'type' => 'datetime'
                    ],
                    [
                        'label' => __('(Max) Memory(Mb)'),
                        'type' => 'number'
                    ],
                    [
                        'label' => __('(Avg) Memory(Mb)'),
                        'type' => 'number'
                    ],
                    [
                        'label' => __('(Max) Swap(Mb)'),
                        'type' => 'number'
                    ],
                    [
                        'label' => __('(Avg) Swap(Mb)'),
                        'type' => 'number'
                    ]
                ]
            ];
        }
        $zendJson = new \Zend_Json();
        if (!count($collection->getItems())) {
            return $zendJson::encode([], false, ['enableJsonExprFinder' => true]);
        }
        foreach ($collection as $row) {
            if ($groupedDataFlag) {
                $data[] = [
                    $this->chart->formatDateToJs(
                    $row->getCreatedAt()),
                    (float)$row->getMaxRam(),
                    (float)$row->getAvgRam(),
                    (float)$row->getMaxSwap(),
                    (float)$row->getAvgSwap()
                ];
                continue;
            }
            $data[] = [
                $this->chart->formatDateToJs($row->getCreatedAt()), (float)$row->getValue(), (float)$row->getSwap()
            ];
        }
        return $zendJson::encode($data, false, ['enableJsonExprFinder' => true]);
    }

    /**
     * Top Website Transactions Chart Data
     *
     * @return mixed
     */
    public function getWebTransactionDataJson()
    {
        /** @var \Potato\PerformanceMonitoring\Model\ResourceModel\TransactionWeb\Collection $collection */
        $collection = $this->chart->getWebTransactionCollection();
        $collection->setMaxMemoryFilter()
            ->setOrderByMemory()
        ;
        $data = $collection->toChartArray('memory');
        return json_encode($data);
    }

    /**
     * System Transactions Chart Data
     *
     * @return mixed
     */
    public function getTransactionDataJson()
    {
        return $this->chart->getSystemTransactionsChartData('memory');
    }

    /**
     * Tab settings
     *
     * @return \Magento\Framework\Phrase
     */
    public function getTabLabel()
    {
        return __('Memory Usage');
    }

    /**
     * @return \Magento\Framework\Phrase
     */
    public function getTabTitle()
    {
        return __('Memory Usage');
    }

    /**
     * @return bool
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * @return bool
     */
    public function isHidden()
    {
        return false;
    }
}