<?php

namespace Potato\PerformanceMonitoring\Cron;

use Potato\PerformanceMonitoring\Model\Config;
use Potato\PerformanceMonitoring\Api\CpuRepositoryInterface;
use Potato\PerformanceMonitoring\Api\MemoryRepositoryInterface;
use Potato\PerformanceMonitoring\Api\TransactionSystemRepositoryInterface;
use Potato\PerformanceMonitoring\Api\TransactionWebRepositoryInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\Stdlib\DateTime;

/**
 * Class RemoveExpired
 */
class RemoveExpired
{
    /** @var Config  */
    protected $config;

    /** @var TransactionWebRepositoryInterface  */
    protected $transactionWebRepository;

    /** @var TransactionSystemRepositoryInterface  */
    protected $transactionSystemRepository;

    /** @var CpuRepositoryInterface  */
    protected $cpuRepository;

    /** @var MemoryRepositoryInterface  */
    protected $memoryRepository;
    
    /** @var LoggerInterface  */
    protected $logger;

    /**
     * RemoveExpired constructor.
     * @param Config $config
     * @param CpuRepositoryInterface $cpuRepository
     * @param MemoryRepositoryInterface $memoryRepository
     * @param TransactionSystemRepositoryInterface $transactionSystemRepository
     * @param TransactionWebRepositoryInterface $transactionWebRepository
     * @param LoggerInterface $logger
     */
    public function __construct(
        Config $config,
        CpuRepositoryInterface $cpuRepository,
        MemoryRepositoryInterface $memoryRepository,
        TransactionSystemRepositoryInterface $transactionSystemRepository,
        TransactionWebRepositoryInterface $transactionWebRepository,
        LoggerInterface $logger
    ) {
        $this->config = $config;
        $this->cpuRepository = $cpuRepository;
        $this->memoryRepository = $memoryRepository;
        $this->transactionSystemRepository = $transactionSystemRepository;
        $this->transactionWebRepository = $transactionWebRepository;
        $this->logger = $logger;
    }

    /**
     * @return $this
     */
    public function execute()
    {
        if (!$this->config->canUseModule() && $this->config->getHistoryLifetime() > 0) {
            return $this;
        }
        $expiredDate = new \DateTime();
        $expiredDate->modify('-' . $this->config->getHistoryLifetime() . ' days');
        $expiredDateStr = $expiredDate->format(DateTime::DATETIME_PHP_FORMAT);
        try {
            $this->transactionWebRepository->removeDataByDate($expiredDateStr);
            $this->transactionSystemRepository->removeDataByDate($expiredDateStr);
            $this->cpuRepository->removeDataByDate($expiredDateStr);
            $this->memoryRepository->removeDataByDate($expiredDateStr);
        } catch (\Exception $e) {
            $this->logger->error($e->getMessage());
        }
        return $this;
    }
}
