<?php

namespace Potato\PerformanceMonitoring\Model;

use Potato\PerformanceMonitoring\Api\Data;
use Magento\Framework;
use Magento\Framework\Stdlib\DateTime\DateTime;

/**
 * Class Cpu
 */
class Cpu extends Framework\Model\AbstractModel
{
    /** @var Data\CpuInterfaceFactory  */
    private $cpuDataFactory;

    /** @var Framework\Api\DataObjectHelper  */
    private $dataObjectHelper;

    /** @var DateTime  */
    protected $date;

    /**
     * Cpu constructor.
     * @param Framework\Model\Context $context
     * @param Framework\Registry $registry
     * @param ResourceModel\Cpu $resource
     * @param ResourceModel\Cpu\Collection $resourceCollection
     * @param Data\CpuInterfaceFactory $cpuDataFactory
     * @param Framework\Api\DataObjectHelper $dataObjectHelper
     * @param DateTime $dateTime
     * @param array $data
     */
    public function __construct(
        Framework\Model\Context $context,
        Framework\Registry $registry,
        ResourceModel\Cpu $resource,
        ResourceModel\Cpu\Collection $resourceCollection,
        Data\CpuInterfaceFactory $cpuDataFactory,
        Framework\Api\DataObjectHelper $dataObjectHelper,
        DateTime $dateTime,
        array $data = []
    ) {
        $this->cpuDataFactory = $cpuDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->date = $dateTime;
        parent::__construct(
            $context,
            $registry,
            $resource,
            $resourceCollection,
            $data
        );
    }

    /**
     * Initialize resource mode
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(ResourceModel\Cpu::class);
    }

    /**
     * Retrieve Cpu model with data
     *
     * @return \Potato\PerformanceMonitoring\Api\Data\CpuInterface
     */
    public function getDataModel()
    {
        $data = $this->getData();
        $dataObject = $this->cpuDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $dataObject,
            $data,
            Data\CpuInterface::class
        );
        return $dataObject;
    }

    /**
     * @return $this
     */
    public function beforeSave()
    {
        //set current date before save
        $this->setCreatedAt($this->date->gmtDate());
        return parent::beforeSave();
    }
}
