<?php
namespace Potato\PerformanceMonitoring\Model\Mail;

use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\DataObject;
use Magento\Store\Model\StoreManagerInterface;
use Potato\PerformanceMonitoring\Model\Config;
use Magento\Framework\App\Area;
use Magento\Framework\Mail\Template\TransportBuilder;
use Potato\PerformanceMonitoring\Model\Source\Recipient;
use Magento\Store\Model\Store;

/**
 * Class Sender
 */
class Sender
{
    /** @var TransportBuilder  */
    protected $transportBuilder;
    
    /** @var StoreManagerInterface  */
    protected $storeManager;

    /** @var Config  */
    protected $config;
    
    /** @var Recipient  */
    protected $recipient;
    
    /**
     * Sender constructor.
     * @param TransportBuilder $transportBuilder
     * @param StoreManagerInterface $storeManager
     * @param Config $config
     * @param Recipient $recipient
     */
    public function __construct(
        TransportBuilder $transportBuilder,
        StoreManagerInterface $storeManager,
        Config $config,
        Recipient $recipient
    ) {
        $this->transportBuilder = $transportBuilder;
        $this->storeManager = $storeManager;
        $this->config = $config;
        $this->recipientSource = $recipient;
    }


    /**
     * @param int $load
     * @return Sender
     */
    public function sendAlert($load)
    {
        $recipient = $this->recipientSource->getUserByOption(
            $this->config->getEmailRecipient()
        );
        $variables  = [
            'admin_name'   => $recipient->getName(),
            'load_average' => $load,
            'top'          => $this->config->getTopStatVariable()
        ];
        return $this->sendEmail(
            $this->config->getEmailTemplate(),
            $recipient->getEmail(),
            $recipient->getName(),
            $this->config->getEmailSender(),
            $this->config->getBCC(),
            $variables
        );
    }

    /**
     * @param mixed $templateId
     * @param string $recipientEmail
     * @param string $recipientName
     * @param string $sender
     * @param string $bcc
     * @param array $variables
     * @return $this
     */
    public function sendEmail(
        $templateId,
        $recipientEmail,
        $recipientName,
        $sender,
        $bcc = null,
        $variables = []
    ) {
        $this->transportBuilder
            ->setTemplateIdentifier($templateId)
            ->setTemplateOptions([
                'area' => Area::AREA_ADMINHTML,
                'store' => Store::DEFAULT_STORE_ID
            ])
            ->setTemplateVars($variables)
            ->setFrom($sender)
            ->addBcc($bcc)
            ->addTo($recipientEmail, $recipientName)
        ;
        $transport = $this->transportBuilder->getTransport();
        $transport->sendMessage();
        return $this;
    }
}
