<?php

namespace Potato\PerformanceMonitoring\Model;

use Potato\PerformanceMonitoring\Api\Data;
use Magento\Framework;
use Magento\Framework\Stdlib\DateTime\DateTime;

/**
 * Class Memory
 */
class Memory extends Framework\Model\AbstractModel
{
    /** @var Data\MemoryInterfaceFactory  */
    private $memoryDataFactory;

    /** @var Framework\Api\DataObjectHelper  */
    private $dataObjectHelper;

    /** @var DateTime  */
    protected $date;

    /**
     * Memory constructor.
     * @param Framework\Model\Context $context
     * @param Framework\Registry $registry
     * @param ResourceModel\Memory $resource
     * @param ResourceModel\Memory\Collection $resourceCollection
     * @param Data\MemoryInterfaceFactory $memoryDataFactory
     * @param Framework\Api\DataObjectHelper $dataObjectHelper
     * @param DateTime $dateTime
     * @param array $data
     */
    public function __construct(
        Framework\Model\Context $context,
        Framework\Registry $registry,
        ResourceModel\Memory $resource,
        ResourceModel\Memory\Collection $resourceCollection,
        Data\MemoryInterfaceFactory $memoryDataFactory,
        Framework\Api\DataObjectHelper $dataObjectHelper,
        DateTime $dateTime,
        array $data = []
    ) {
        $this->memoryDataFactory = $memoryDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->date = $dateTime;
        parent::__construct(
            $context,
            $registry,
            $resource,
            $resourceCollection,
            $data
        );
    }

    /**
     * Initialize resource mode
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(ResourceModel\Memory::class);
    }

    /**
     * Retrieve Memory model with data
     *
     * @return \Potato\PerformanceMonitoring\Api\Data\MemoryInterface
     */
    public function getDataModel()
    {
        $data = $this->getData();
        $dataObject = $this->memoryDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $dataObject,
            $data,
            Data\MemoryInterface::class
        );
        return $dataObject;
    }

    /**
     * @return $this
     */
    public function beforeSave()
    {
        //set current date before save
        $this->setCreatedAt($this->date->gmtDate());
        return parent::beforeSave();
    }
}
