<?php

namespace Potato\PerformanceMonitoring\Model\ResourceModel\Memory;

use Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection;
use Potato\PerformanceMonitoring\Model;

/**
 * Class Collection
 */
class Collection extends AbstractCollection
{
    /**
     * Init collection and determine table names
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(
            Model\Memory::class,
            Model\ResourceModel\Memory::class
        );
    }

    /**
     * @param string $from
     * @param string|null $to
     * @return $this
     */
    public function setDateFilter($from, $to = null)
    {
        $this->addFieldToFilter('created_at', ['gt' => $from]);
        if ($to) {
            $this->addFieldToFilter('created_at', ['lt' => $to]);
        }
        return $this;
    }

    /**
     * @return $this
     */
    public function orderByDate()
    {
        $this->getSelect()->order('created_at ' . \Zend_Db_Select::SQL_DESC);
        return $this;
    }

    /**
     * @return $this
     */
    public function groupByHour()
    {
        $this->getSelect()->columns(
            [
                'max_ram' => new \Zend_DB_Expr('MAX(main_table.value)'),
                'avg_ram' =>  new \Zend_DB_Expr('ROUND(AVG(main_table.value),2)'),
                'max_swap' => new \Zend_DB_Expr('MAX(main_table.swap)'),
                'avg_swap' =>  new \Zend_DB_Expr('ROUND(AVG(main_table.swap),2)')
            ]
        );
        $this->getSelect()->group(["HOUR(main_table.created_at)", "DAY(main_table.created_at)", "YEAR(main_table.created_at)", "MONTH(main_table.created_at)"]);
        return $this;
    }
}
