<?php

namespace Potato\PerformanceMonitoring\Model;

use Potato\PerformanceMonitoring\Api\Data;
use Magento\Framework;
use Magento\Framework\Stdlib\DateTime\DateTime;

/**
 * Class TransactionSystem
 */
class TransactionSystem extends Framework\Model\AbstractModel
{
    /** @var Data\TransactionSystemInterfaceFactory  */
    private $transactionSystemDataFactory;

    /** @var Framework\Api\DataObjectHelper  */
    private $dataObjectHelper;

    /** @var DateTime  */
    protected $date;

    /**
     * TransactionSystem constructor.
     * @param Framework\Model\Context $context
     * @param Framework\Registry $registry
     * @param ResourceModel\TransactionSystem $resource
     * @param ResourceModel\TransactionSystem\Collection $resourceCollection
     * @param Data\TransactionSystemInterfaceFactory $transactionSystemDataFactory
     * @param Framework\Api\DataObjectHelper $dataObjectHelper
     * @param DateTime $dateTime
     * @param array $data
     */
    public function __construct(
        Framework\Model\Context $context,
        Framework\Registry $registry,
        ResourceModel\TransactionSystem $resource,
        ResourceModel\TransactionSystem\Collection $resourceCollection,
        Data\TransactionSystemInterfaceFactory $transactionSystemDataFactory,
        Framework\Api\DataObjectHelper $dataObjectHelper,
        DateTime $dateTime,
        array $data = []
    ) {
        $this->transactionSystemDataFactory = $transactionSystemDataFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->date = $dateTime;
        parent::__construct(
            $context,
            $registry,
            $resource,
            $resourceCollection,
            $data
        );
    }

    /**
     * Initialize resource mode
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init(ResourceModel\TransactionSystem::class);
    }

    /**
     * Retrieve TransactionSystem model with data
     *
     * @return \Potato\PerformanceMonitoring\Api\Data\TransactionSystemInterface
     */
    public function getDataModel()
    {
        $data = $this->getData();
        $dataObject = $this->transactionSystemDataFactory->create();
        $this->dataObjectHelper->populateWithArray(
            $dataObject,
            $data,
            Data\TransactionSystemInterface::class
        );
        return $dataObject;
    }

    /**
     * @return $this
     */
    public function beforeSave()
    {
        //set current date before save
        $this->setCreatedAt($this->date->gmtDate());
        return parent::beforeSave();
    }
}
