<?php

namespace Potato\PerformanceMonitoring\Model;

use Magento\Framework\Exception\NoSuchEntityException;
use Potato\PerformanceMonitoring\Api\Data\TransactionSystemInterfaceFactory;

/**
 * Class TransactionSystem
 */
class TransactionSystemRegistry
{
    /** @var TransactionSystemFactory  */
    private $transactionSystemFactory;

    /** @var array  */
    private $transactionSystemRegistryById = [];

    /** @var ResourceModel\TransactionSystem  */
    private $transactionSystemResource;

    /** @var TransactionSystemInterfaceFactory  */
    private $dataFactory;

    /**
     * TransactionSystemRegistry constructor.
     * @param TransactionSystemFactory $transactionSystemFactory
     * @param ResourceModel\TransactionSystem $transactionSystemResource
     * @param TransactionSystemInterfaceFactory $dataFactory
     */
    public function __construct(
        TransactionSystemFactory $transactionSystemFactory,
        ResourceModel\TransactionSystem $transactionSystemResource,
        TransactionSystemInterfaceFactory $dataFactory
    ) {
        $this->transactionSystemResource = $transactionSystemResource;
        $this->transactionSystemFactory = $transactionSystemFactory;
        $this->dataFactory = $dataFactory;
    }

    /**
     * @param int $transactionSystemId
     * @return TransactionSystem
     * @throws NoSuchEntityException
     */
    public function retrieve($transactionSystemId)
    {
        if (!isset($this->transactionSystemRegistryById[$transactionSystemId])) {
            /** @var TransactionSystem $transactionSystem */
            $transactionSystem = $this->transactionSystemFactory->create();
            $this->transactionSystemResource->load($transactionSystem, $transactionSystemId);
            if (!$transactionSystem->getId()) {
                throw NoSuchEntityException::singleField('transactionSystemId', $transactionSystemId);
            } else {
                $this->transactionSystemRegistryById[$transactionSystemId] = $transactionSystem;
            }
        }
        return $this->transactionSystemRegistryById[$transactionSystemId];
    }

    /**
     * @param int $transactionSystemId
     * @return void
     */
    public function remove($transactionSystemId)
    {
        if (isset($this->transactionSystemRegistryById[$transactionSystemId])) {
            unset($this->transactionSystemRegistryById[$transactionSystemId]);
        }
    }

    /**
     * @param TransactionSystem $transactionSystem
     * @return $this
     */
    public function push(TransactionSystem $transactionSystem)
    {
        $this->transactionSystemRegistryById[$transactionSystem->getId()] = $transactionSystem;
        return $this;
    }

    /**
     * @return \Potato\PerformanceMonitoring\Api\Data\TransactionSystemInterface
     */
    public function create()
    {
        return $this->dataFactory->create();
    }
}
