<?php

namespace Potato\PerformanceMonitoring\Model;

use Magento\Framework\Exception\NoSuchEntityException;
use Potato\PerformanceMonitoring\Api\Data\TransactionWebInterfaceFactory;


/**
 * Class TransactionWeb
 */
class TransactionWebRegistry
{
    /** @var TransactionWebFactory  */
    private $transactionWebFactory;

    /** @var array  */
    private $transactionWebRegistryById = [];

    /** @var ResourceModel\TransactionWeb  */
    private $transactionWebResource;

    /** @var TransactionWebInterfaceFactory  */
    private $dataFactory;

    /**
     * TransactionWebRegistry constructor.
     * @param TransactionWebFactory $transactionWebFactory
     * @param ResourceModel\TransactionWeb $transactionWebResource
     * @param TransactionWebInterfaceFactory $dataFactory
     */
    public function __construct(
        TransactionWebFactory $transactionWebFactory,
        ResourceModel\TransactionWeb $transactionWebResource,
        TransactionWebInterfaceFactory $dataFactory
    ) {
        $this->transactionWebResource = $transactionWebResource;
        $this->transactionWebFactory = $transactionWebFactory;
        $this->dataFactory = $dataFactory;
    }

    /**
     * @param int $transactionWebId
     * @return TransactionWeb
     * @throws NoSuchEntityException
     */
    public function retrieve($transactionWebId)
    {
        if (!isset($this->transactionWebRegistryById[$transactionWebId])) {
            /** @var TransactionWeb $transactionWeb */
            $transactionWeb = $this->transactionWebFactory->create();
            $this->transactionWebResource->load($transactionWeb, $transactionWebId);
            if (!$transactionWeb->getId()) {
                throw NoSuchEntityException::singleField('transactionWebId', $transactionWebId);
            } else {
                $this->transactionWebRegistryById[$transactionWebId] = $transactionWeb;
            }
        }
        return $this->transactionWebRegistryById[$transactionWebId];
    }

    /**
     * @param int $transactionWebId
     * @return void
     */
    public function remove($transactionWebId)
    {
        if (isset($this->transactionWebRegistryById[$transactionWebId])) {
            unset($this->transactionWebRegistryById[$transactionWebId]);
        }
    }

    /**
     * @param TransactionWeb $transactionWeb
     * @return $this
     */
    public function push(TransactionWeb $transactionWeb)
    {
        $this->transactionWebRegistryById[$transactionWeb->getId()] = $transactionWeb;
        return $this;
    }

    /**
     * @return \Potato\PerformanceMonitoring\Api\Data\WebSystemInterface
     */
    public function create()
    {
        return $this->dataFactory->create();
    }
}
