<?php
/**
 * A Magento 2 module named Techdivi/Prodsync
 * Copyright (C) 2017
 *
 * This file included in Techdivi/Prodsync is licensed under OSL 3.0
 *
 * http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 * Please see LICENSE.txt for the full text of the OSL 3.0 license
 */

namespace Techdivi\Prodsync\Console\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

class Allupdate extends Command
{

    const NAME_ARGUMENT = "name";

    const NAME_OPTION = "o";

    private $itemData = null;

    private $soap = null;

    private $soapURL = 'http://58.27.236.238:901/services.asmx?wsdl';

    private $soapOptions = array('connection_timeout' => 600);

    private $soapAllItemsErrors = 0;

    private $soapGetBarcodeDataErrors = 0;

    private $barcodeData = array();

    private $currentProduct = 0;

    private $countProducts = 0;

    /**
     *
     * {@inheritdoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {

//        $name = $input->getArgument(self::NAME_ARGUMENT);
        $option = $input->getOption(self::NAME_OPTION);
//        $this->logConsole($option);

        //$output->writeln("Hello " . $name);
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        /** @var \Magento\Catalog\Model\ResourceModel\Product\Collection $productCollection */
        $productCollection = $objectManager->create('Magento\Catalog\Model\ResourceModel\Product\Collection');
        /**
         * Apply filters here
         */
        $eventManager = $objectManager->create('\Magento\Framework\Event\ManagerInterface');
        $productCollection->addAttributeToSelect('*');

        if ($option == '1') $productCollection->addAttributeToFilter('status',\Magento\Catalog\Model\Product\Attribute\Source\Status::STATUS_ENABLED);

        $productCollection->load();

        $this->countProducts = count($productCollection);
//        $this->logConsole($this->countProducts);
//die($this->countProducts);
        $this->currentProduct = 0;

        $this->getAllData();

        foreach ($productCollection as $product) {

            try {
                $sku = $product->getSku();
//                if ($sku <> '28-0004-000123') continue;

                $eventManager->dispatch('isp_catalog_product_save_after', ['product' => $product]);

                $productData = $this->getProductData($sku);
                if (!is_array($productData)) {
                    $this->logConsole('No product data from Service for SKU="' . $sku . '"', true, '0;31m');
//                    $this->currentProduct++;
//                    continue;
                }

                $barcodesString = $product->getBarcode();

                $barcodes = explode(",", $barcodesString);
                $barcodes = array_map('trim', $barcodes);
                $barcodes = array_fill_keys( array_values($barcodes), false);
//    if (count($barcodes) < 2) continue;

                $this->logConsole('SKU="' . $sku . '". Barcodes before: ' . $barcodesString);

                if (!is_array($productData)) {
                    if (isset($product['CHILDID'])) $barcodes[$product['CHILDID']] = false;
                }

                foreach ($barcodes as $k => $v)
                {
                    $tempItems = $this->getAllDataBarcd($k);
                    if (is_array($tempItems))
                    {
                        foreach ($tempItems as $item)
                        {
                            if (is_array($item))
                            {
                                $barcodes[$k] = $item;
                                if ( isset($item['ALTERNATEBARCODE']) AND !isset( $barcodes[$item['ALTERNATEBARCODE']] ) )
                                    $barcodes[$item['ALTERNATEBARCODE']] = $this->getAllDataBarcd($item['ALTERNATEBARCODE']);
                            }
                        }
                    }
                }

                $this->logConsole('SKU="' . $sku . '". Barcodes after: ' . implode( ',', array_keys($barcodes) ) );

                $maxRetailPrice = 0;
                $maxCostPrice = 0;
                $maxRetailPriceID = '';

                if (is_array($productData)) {
                    $maxRetailPrice = $productData['PRICE'];
                    $maxCostPrice = $productData['COST_PRICE'];
                    $maxRetailPriceID = $sku;
                }

                foreach ($barcodes as $k => $v)
                {
                    if (is_array($v))
                    {
                        if (isset($v['RETAILPRICE'])) {
                            $maxRetailPrice = max($maxRetailPrice, $v['RETAILPRICE']);
                            $maxRetailPriceID = $k;
                        }
                        if (isset($v['COSTPRICE'])) $maxCostPrice = max($maxCostPrice, $v['COSTPRICE']);
                    }
                }

                if (!empty($maxRetailPrice))
                {
                    $this->logConsole('SKU="' . $sku . '". Max retail price at "' . $maxRetailPriceID . '": ' . $maxRetailPrice);
                    $productModel = $objectManager->get('\Magento\Catalog\Model\Product');
                    $refreshProduct = $productModel->loadByAttribute('sku', $sku);
                    $refreshBrcodesString = $refreshProduct->getBarcode();

                    $refreshBarcodes = explode(",", $refreshBrcodesString);
                    $refreshBarcodes = array_map('trim', $refreshBarcodes);
                    $refreshBarcodes = array_fill_keys( array_values($refreshBarcodes), false);
                    foreach ($barcodes as $kk => $vv) $refreshBarcodes[$kk] = false;

//                    $this->logConsole('combined barcodes: ' . implode( ',', array_keys($barcodes) ));
//                    die('stop');
                    $refreshProduct->addAttributeUpdate('price', $maxRetailPrice, 0);
                    if (!empty($maxCostPrice)) $product->addAttributeUpdate('cost', $maxCostPrice, 0);
                    $refreshProduct->addAttributeUpdate('barcode', implode( ',', array_keys($barcodes) ), 0);
                }
                else
                {
                    $this->logConsole('Max retail price not found for SKU="' . $sku . '"', true, '0;31m');
                }


            } catch (Exception $e) {
                $this->logConsole($e->getMessage(), true, '0;31m');
            }
            $this->currentProduct++;
        }
    }

    private function logConsole($message, $newline = true, $color = '')
    {
        // RED - 0;31m
        if ($color) echo "\033[".$color;
        echo '[' . date('H:i:s') . '] (' . $this->currentProduct . '/' . $this->countProducts . ') ' . $message;
        if ($color) echo "\033[0m";
        if ($newline) echo "\n";

        if ($this->soapAllItemsErrors > 2)
        {
            echo "\033[0;31m Seems, that Service is unavaliable. Stop execution.\033[0m" . "\n";
            die();
        }

        if ($this->soapGetBarcodeDataErrors > 20)
        {
            echo "\033[0;31m Seems, that Service is unavaliable. Stop execution.\033[0m" . "\n";
            die();
        }

    }

    /**
     *
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this->setName("techdivi_prodsync:allupdate");
        $this->setDescription("Update Everything");
        $this->setDefinition([
//            new InputArgument(self::NAME_ARGUMENT, InputArgument::OPTIONAL, "Name"),
            new InputOption(self::NAME_OPTION, "o", InputOption::VALUE_NONE, "Update only activated products")
        ]);
        parent::configure();
    }

    private function getAllDataBarcd($barcd)
    {

        try {
//            echo "\nFetching Data from Bar Code Service\n";
            //$client = new \SoapClient("http://43.245.131.42:901/services.asmx?wsdl");
            if (is_null($this->soap)) $this->soap = new \SoapClient("http://58.27.236.238:901/services.asmx?wsdl", array('connection_timeout' => 5000));

            $data = $this->soap->GetItemsByBarcode(array(
                'barcd' => $barcd,
            ));

            $data = $this->_json_decode($data->GetItemsByBarcodeResult);
//            print_r($a);
            if (is_array($data)) {
                return $data;
            }

            return false;

        } catch (\SoapFault $fault) {
            $this->soapGetBarcodeDataErrors++;
            $this->logConsole( 'Service error: ' . $fault->getMessage(), true, '0;31m' );
            return null;
        }

    }

    private function getAllData()
    {
        try {
            $this->logConsole("Fetching All Items Data from Service");
            ini_set('default_socket_timeout', 5000);

//     $client = new \SoapClient("http://58.27.236.238:901/services.asmx?wsdl");
            if (is_null($this->soap)) $this->soap = new \SoapClient("http://58.27.236.238:901/services.asmx?wsdl", array('connection_timeout' => 5000));

            $data = $this->soap->GetAllItems(array(
                'rnFrom' => 1,
                'rnTo' => 20000
            ));

            // echo $data->GetAllItemsResult;
            $itemData = $this->_json_decode($data->GetAllItemsResult);

            foreach ($itemData as $k => $v) $this->itemData[$v['ID']] = $v;

            $this->logConsole("Service Call Done");
        } catch (\SoapFault $fault) {
            $this->soapAllItemsErrors++;
            $this->logConsole('Soap error: ' . $fault->getMessage(), true, '0;31m');
//            echo "error";
//            echo "http://58.27.236.238:901/services.asmx";
            return null;
        }

    }

    private function getProductData($sku)
    {
        if (empty($this->itemData)) {
            $this->getAllData();
        }

        $item = isset($this->itemData[$sku]) ? $this->itemData[$sku] : false;

        return $item;
    }

    private function _json_decode($string)
    {
        if (get_magic_quotes_gpc()) {
            $string = stripslashes($string);
        }

        return json_decode($string, true);
    }

    public function __construct(\Magento\Framework\App\State $state)
    {
        $state->setAreaCode('adminhtml'); // or 'adminhtml', depending on your needs
        parent::__construct();
    }
}

