<?php

namespace Themecafe\CODOrderVerification\Controller\Frontend;

class OtpSend extends \Magento\Framework\App\Action\Action {

    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\View\Result\PageFactory $resultPageFactory,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory,
        \Magento\Checkout\Model\Session $_checkoutSession,
        \Magento\Framework\App\ResourceConnection $resourceConnection,
        \Themecafe\CODOrderVerification\Model\OtpVerificationFactory $otpVerificationFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $DateTime,
        \Themecafe\CODOrderVerification\Helper\Data $helper,
        \Magento\Store\Model\StoreManagerInterface $storeInterface
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->resultPageFactory = $resultPageFactory;
        $this->om = $context->getObjectManager();
        $this->_checkoutSession = $_checkoutSession;
        $this->_resources = $resourceConnection;
        $this->otpVerificationFactory = $otpVerificationFactory;
        $this->DateTime = $DateTime;
        $this->helper = $helper;
        $this->_storeInterface = $storeInterface;
        $this->messageManager = $context->getMessageManager();
        $this->request = $context->getRequest();
        parent::__construct($context);
    }
    public function execute() {
        $otpcode = $this->generateOtpCode();
        $billingTelephone = $this->request->getParam('billingNumber');
        $quoteid = $this->_checkoutSession->getQuoteId();
        $contactnumber = $billingTelephone;
        
        $Otpcollection = $this->otpVerificationFactory->create()->getCollection();
        $quoteArray = $Otpcollection->getColumnValues('quote_id');
        
        $timestamp = $this->DateTime->timestamp();
        
        try {
            $storeId = $this->_storeInterface->getStore()->getId();
            $message = str_replace("##OTP##", $otpcode, $this->helper->otpMessage($storeId));
            if(trim($this->helper->otpMessage($storeId))==""){
                $message = $otpcode;
            }
            
            $this->helper->sendSms($contactnumber, $message, 'ALL', $storeId);
            $response['messages'] = 1;
            if($this->helper->getQueryStringElement()) {
                $this->updateOTP($quoteid, $quoteArray, $otpcode, $contactnumber, $timestamp);
                $response['success'] = 1;
            }
        } catch (\RuntimeException $e) {
            $this->messageManager->addError(__('Problem while sending your requestion. %1', $e->getMessage()));
        }
        
        $this->getResponse()->representJson(
            $this->om->get('Magento\Framework\Json\Helper\Data')->jsonEncode($response)
        );
        return ;
    }
    public function updateOTP($quoteid, $quoteArray, $otpcode, $contactnumber, $timestamp) {
        $otpModel = $this->otpVerificationFactory->create();
        if(in_array($quoteid, $quoteArray)) {
            $otpModel->load($quoteid, 'quote_id');
            $otpModel->setData('verified',0);
        }else {
            $otpModel->setData('quote_id',$quoteid);
        }
        $otpModel->setData('send_time',$timestamp);
        $otpModel->setData('mobile_number',$contactnumber);
        $otpModel->setData('otp_code',$otpcode);
        $otpModel->save();
    }
    public function generateOtpCode() {
        $string = '0123456789';
        $string_shuffled = str_shuffle($string);
        return substr($string_shuffled, 1, 4);
	/*return rand(1000,9999);*/
/*	return 1234;*/
    }
}
