<?php
namespace Vnecoms\FeaturedProduct\Block\Adminhtml\Block\Edit\Tab;

use Magento\Backend\Block\Widget\Form;
use Magento\Backend\Block\Widget\Form\Generic;
use Magento\Backend\Block\Widget\Tab\TabInterface;

class Main extends Generic implements TabInterface
{
    
    /**
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry;

    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $date;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Source\Template
     */
    protected $blockTemplate;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Source\SortBy
     */
    protected $sortBy;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Source\Attribute
     */
    protected $attribute;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Source\Period
     */
    protected $period;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Source\Direction
     */
    protected $sortDirection;
    
    /**
     * @var \Magento\Backend\Block\Widget\Form\Renderer\Fieldset
     */
    protected $rendererFieldset;
    
    /**
     * @var \Magento\Rule\Block\Conditions
     */
    protected $conditions;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Source\Status
     */
    protected $blockStatus;
    
   /**
    * 
    * @param \Magento\Backend\Block\Template\Context $context
    * @param \Magento\Framework\Registry $registry
    * @param \Magento\Framework\Data\FormFactory $formFactory
    * @param \Magento\Framework\Stdlib\DateTime\DateTime $date
    * @param \Vnecoms\FeaturedProduct\Model\Source\Template $tabTemplate
    * @param \Vnecoms\FeaturedProduct\Model\Source\SortBy $sortBy
    * @param \Vnecoms\FeaturedProduct\Model\Source\Period $period
    * @param \Vnecoms\FeaturedProduct\Model\Source\Attribute $attribute
    * @param \Vnecoms\FeaturedProduct\Model\Source\Direction $direction
    * @param \Vnecoms\FeaturedProduct\Model\Source\Status $tabsStatus
    * @param \Vnecoms\FeaturedProduct\Block\Adminhtml\Rule\Conditions $conditions
    * @param \Magento\Backend\Block\Widget\Form\Renderer\Fieldset $rendererFieldset
    * @param array $data
    */
    public function __construct(
        \Magento\Backend\Block\Template\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Data\FormFactory $formFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        \Vnecoms\FeaturedProduct\Model\Source\Template $blockTemplate,
        \Vnecoms\FeaturedProduct\Model\Source\SortBy $sortBy,
        \Vnecoms\FeaturedProduct\Model\Source\Period $period,
        \Vnecoms\FeaturedProduct\Model\Source\Attribute $attribute,
        \Vnecoms\FeaturedProduct\Model\Source\Direction $direction,
        \Vnecoms\FeaturedProduct\Model\Source\Status $blockStatus,
        \Vnecoms\FeaturedProduct\Block\Adminhtml\Rule\Conditions $conditions,
        \Magento\Backend\Block\Widget\Form\Renderer\Fieldset $rendererFieldset,
        array $data = []
    ) {
        $this->coreRegistry = $registry;
        $this->date = $date;
        $this->blockTemplate = $blockTemplate;
        $this->sortBy = $sortBy;
        $this->period = $period;
        $this->attribute = $attribute;
        $this->sortDirection = $direction;
        $this->blockStatus = $blockStatus;
        $this->rendererFieldset = $rendererFieldset;
        $this->conditions = $conditions;
        
        parent::__construct($context, $registry, $formFactory, $data);
    }

    
    /**
     * Prepare content for tab
     *
     * @return \Magento\Framework\Phrase
     * @codeCoverageIgnore
     */
    public function getTabLabel()
    {
        return __('Block Information');
    }

    /**
     * Prepare title for tab
     *
     * @return \Magento\Framework\Phrase
     * @codeCoverageIgnore
     */
    public function getTabTitle()
    {
        return __('Block Information');
    }

    /**
     * Returns status flag about this tab can be showed or not
     *
     * @return bool
     * @codeCoverageIgnore
     */
    public function canShowTab()
    {
        return true;
    }

    /**
     * Returns status flag about this tab hidden or not
     *
     * @return bool
     * @codeCoverageIgnore
     */
    public function isHidden()
    {
        return false;
    }

    /**
     * @return Form
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    protected function _prepareForm()
    {
        $model = $this->coreRegistry->registry('current_block');

        /** @var \Magento\Framework\Data\Form $form */
        $form = $this->_formFactory->create();
        
        $fieldset = $form->addFieldset('block_info_fieldset', ['legend' => __('Block Information'), 'class' => 'featuredproduct-info']);
                
        if ($model->getId()) {
            $fieldset->addField('block_id', 'hidden', ['name' => 'block_id']);
        }
                
        
        $field = $fieldset->addField(
            'title',
            'text',
            [
                'name' => 'title',
                'label' => __('Title'),
                'title' => __('Title'),
                'required' => true
            ]
        );
        $field = $fieldset->addField(
            'css_class',
            'text',
            [
                'name' => 'css_class',
                'label' => __('Css Class'),
                'title' => __('Css Class'),
                'required' => false
            ]
        );
        $field = $fieldset->addField(
            'cache_lifetime',
            'text',
            [
                'name' => 'cache_lifetime',
                'label' => __('Cache Lifetime'),
                'title' => __('Cache Lifetime'),
                'class' => 'validate-number',
                'note' => __("In seconds. Set 0 or leave blank to disable cache.")
            ]
        );
        
        $fieldset->addField(
            'status',
            'select',
            [
                'name' => 'status',
                'label' => __('Status'),
                'title' => __('Status'),
                'options' => $this->blockStatus->getOptionArray(),
            ]
        );

        /* Product Filter*/
        $rule = $this->coreRegistry->registry('current_catalog_rule');
        
        $renderer = $this->rendererFieldset->setTemplate(
            'Magento_CatalogRule::promo/fieldset.phtml'
        )->setNewChildUrl(
            $this->getUrl('catalog_rule/promo_catalog/newConditionHtml/form/rule_conditions_fieldset')
        );
        
        $fieldset = $form->addFieldset(
            'rule_conditions_fieldset',
            ['legend' => __('Product Filter')]
        )->setRenderer(
            $renderer
        );
        $fieldset->addField(
            'current_category_filter',
            'select',
            [
                'name' => 'current_category_filter',
                'label' => __('Filter by current category'),
                'title' => __('Filter by current category'),
                'options' => [
                    '0' => __("No"),
                    "1" => __("Yes"),
                ],
            ]
        );
        $fieldset->addField(
            'conditions',
            'text',
            ['name' => 'conditions', 'label' => __('Conditions'), 'title' => __('Conditions'), 'required' => true]
        )->setRule(
            $rule
        )->setRenderer(
            $this->conditions
        );
        
        
        /*Product List*/
        $fieldset = $form->addFieldset('tab_product_list_fieldset', ['legend' => __('Product List')]);
        $field = $fieldset->addField(
            'template',
            'select',
            [
                'name' => 'template',
                'label' => __('Representation'),
                'title' => __('Representation'),
                'onchange' => 'homeTabsTemplateChange()',
                'required' => true,
                'values' => $this->blockTemplate->getOptionArray()
            ]
        );
        $field = $fieldset->addField(
            'product_limit',
            'text',
            [
                'name' => 'product_limit',
                'label' => __('Product Limit'),
                'title' => __('Product Limit'),
                'class' => 'validate-number',
                'required' => true
            ]
        );
        $field = $fieldset->addField(
            'sort_by',
            'select',
            [
                'name' => 'sort_by',
                'label' => __('Sort By'),
                'title' => __('Sort By'),
                'onchange' => 'homeTabsSortByChange()',
                'required' => true,
                'values' => $this->sortBy->getOptionArray()
            ]
        );
        $field = $fieldset->addField(
            'sort_period',
            'select',
            [
                'name' => 'sort_period',
                'label' => __('Period'),
                'title' => __('Period'),
                'required' => true,
                'values' => $this->period->getOptionArray()
            ]
        );
        $field = $fieldset->addField(
            'sort_attribute',
            'select',
            [
                'name' => 'sort_attribute',
                'label' => __('Attribute'),
                'title' => __('Attribute'),
                'required' => true,
                'values' => $this->attribute->getOptionArray()
            ]
        );
        $field = $fieldset->addField(
            'sort_direction',
            'select',
            [
                'name' => 'sort_direction',
                'label' => __('Sort Direction'),
                'title' => __('Sort Direction'),
                'values' => $this->sortDirection->getOptionArray()
            ]
        );
        
        
        /*Slider Option*/
        $fieldset = $form->addFieldset('tab_slider_options_fieldset', ['legend' => __('Slider Options')]);
        
        $field = $fieldset->addField(
            'slider_rows',
            'select',
            [
                'name' => 'additional_data[slider][rows]',
                'label' => __('Rows'),
                'title' => __('Rows'),
                'required' => true,
                'values' => [
                    '1' => 1,
                    '2' => 2,
                    '3' => 3,
                    '4' => 4,
                    '5' => 5,
                    '6' => 6,
                    '7' => 7,
                    '8' => 8,
                    '9' => 9,
                ]
            ]
        );
        
        $field = $fieldset->addField(
            'slider_cols_1200',
            'select',
            [
                'name' => 'additional_data[slider][cols_1200]',
                'label' => __('Columns (1200)'),
                'title' => __('Columns (1200)'),
                'required' => true,
                'values' => [
                    '1' => 1,
                    '2' => 2,
                    '3' => 3,
                    '4' => 4,
                    '5' => 5,
                    '6' => 6,
                ],
                'note' => __('For devices have screen size equal or greater than 1200px.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_cols_992',
            'select',
            [
                'name' => 'additional_data[slider][cols_992]',
                'label' => __('Columns (992)'),
                'title' => __('Columns (992)'),
                'required' => true,
                'values' => [
                    '1' => 1,
                    '2' => 2,
                    '3' => 3,
                    '4' => 4,
                    '5' => 5,
                    '6' => 6,
                ],
                'note' => __('For devices have screen size from 992px to 1199px.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_cols_768',
            'select',
            [
                'name' => 'additional_data[slider][cols_768]',
                'label' => __('Columns (768)'),
                'title' => __('Columns (768)'),
                'required' => true,
                'values' => [
                    '1' => 1,
                    '2' => 2,
                    '3' => 3,
                    '4' => 4,
                    '5' => 5,
                    '6' => 6,
                ],
                'note' => __('For devices have screen size from 768px to 991px.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_cols_480',
            'select',
            [
                'name' => 'additional_data[slider][cols_480]',
                'label' => __('Columns (480)'),
                'title' => __('Columns (480)'),
                'required' => true,
                'values' => [
                    '1' => 1,
                    '2' => 2,
                    '3' => 3,
                    '4' => 4,
                    '5' => 5,
                    '6' => 6,
                ],
                'note' => __('For devices have screen size from 480px to 767px.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_cols_0',
            'select',
            [
                'name' => 'additional_data[slider][cols_0]',
                'label' => __('Columns (small)'),
                'title' => __('Columns (small)'),
                'required' => true,
                'values' => [
                    '1' => 1,
                    '2' => 2,
                    '3' => 3,
                    '4' => 4,
                    '5' => 5,
                    '6' => 6,
                ],
                'note' => __('For devices have screen size less than 480px.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_margin',
            'text',
            [
                'name' => 'additional_data[slider][margin]',
                'label' => __('Margin Item'),
                'title' => __('Margin Item'),
                'required' => false,
                'class' => 'validate-number',
                'note' => __('margin-right(px) on item.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_loop',
            'select',
            [
                'name' => 'additional_data[slider][loop]',
                'label' => __('Loop'),
                'title' => __('Loop'),
                'values' => [
                    '0' => __("No"),
                    '1' => __("Yes"),
                ],
                'note' => __('Infinity loop. Duplicate last and first items to get loop illusion.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_slideBy',
            'text',
            [
                'name' => 'additional_data[slider][slideBy]',
                'label' => __('Slide By'),
                'title' => __('Slide By'),
                'class' => 'validate-number',
                'required' => false,
                'note' => __('Navigation slide by x.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_autoplay',
            'select',
            [
                'name' => 'additional_data[slider][autoplay]',
                'label' => __('Autoplay'),
                'title' => __('Autoplay'),
                'onchange' => 'homeTabsAutoplayChange()',
                'values' => [
                    '0' => __("No"),
                    '1' => __("Yes"),
                ],
            ]
        );
        $field = $fieldset->addField(
            'slider_autoplayTimeout',
            'text',
            [
                'name' => 'additional_data[slider][autoplayTimeout]',
                'label' => __('Autoplay Timeout'),
                'title' => __('Autoplay Timeout'),
                'class' => 'validate-number',
                'required' => false,
                'note' => __('Autoplay interval timeout (mini seconds).')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_autoplayHoverPause',
            'select',
            [
                'name' => 'additional_data[slider][autoplayHoverPause]',
                'label' => __('Autoplay Hover Pause'),
                'title' => __('Autoplay Hover Pause'),
                'values' => [
                    '0' => __("No"),
                    '1' => __("Yes"),
                ],
                'note' => __('Pause on mouse hover.')
            ]
        );
        
        $field = $fieldset->addField(
            'slider_autoplaySpeed',
            'text',
            [
                'name' => 'additional_data[slider][autoplaySpeed]',
                'label' => __('Autoplay Speed'),
                'title' => __('Autoplay Speed'),
                'class' => 'validate-number',
                'required' => false,
                'note' => __('Autoplay speed (mini seconds).')
            ]
        );
        
        $form->setFieldNameSuffix('block');
        if (!$model->getBlockId()) {
            $additionalData = [
                'slider' => [
                    'rows' => 1,
                    'cols_1200' => 5,
                    'cols_992' => 5,
                    'cols_768' => 3,
                    'cols_480' => 2,
                    'cols_0' => 1,
                    'margin' => 20,
                    'loop' => false,
                    'slideBy' => 1,
                    'autoplay' => false,
                    'autoplayTimeout' => 5000,
                    'autoplayHoverPause' => false,
                    'autoplaySpeed' => 3000,
                ]
            ];
        } else {
            $additionalData = json_decode($model->getAdditionalData(), true);
            $additionalData = $additionalData?$additionalData:[];
        }

        $form->setValues($model->getData());
        foreach ($additionalData as $key => $data) {
            foreach ($data as $field => $value) {
                $field = $form->getElement($key.'_'.$field);
                if ($field) {
                    $field->setValue($value);
                }
            }
        }
        
        $this->setForm($form);

        return parent::_prepareForm();
    }
}
