<?php
namespace Vnecoms\FeaturedProduct\Controller\Adminhtml\Index;

use Vnecoms\FeaturedProduct\Controller\Adminhtml\AbstractAction;

class Edit extends AbstractAction
{
    /**
     * Is access to section allowed
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Vnecoms_FeaturedProduct::block') && parent::_isAllowed();
    }
    
    /**
     * @return void
     */
    public function execute()
    {
        $id = $this->getRequest()->getParam('id');
        $model = $this->_objectManager->create('Vnecoms\FeaturedProduct\Model\Block');

        if ($id) {
            $model->load($id);
            if (!$model->getBlockId()) {
                $this->messageManager->addError(__('This block no longer exists.'));
                $this->_redirect('featuredproduct/index/*');
                return;
            }
        }

        // set entered data if was error when we do save
        $data = $this->_objectManager->get('Magento\Backend\Model\Session')->getBlockData(true);
        if (!empty($data)) {
            $model->addData($data);
        }
        
        $rule = $this->_objectManager->create('Magento\CatalogRule\Model\Rule');
        if ($model->getId()) {
            $rule->setConditionsSerialized($model->getConditionsSerialized());
        }
        $rule->getConditions()->setFormName('featuredproduct_form');
        $rule->getConditions()->setJsFormObject(
            $rule->getConditionsFieldSetId($rule->getConditions()->getFormName())
        );
        
        $this->_coreRegistry->register('current_catalog_rule', $rule);
        $this->_coreRegistry->register('current_block', $model);
        $this->_coreRegistry->register('block', $model);

        $this->_initAction();
        $this->_view->getPage()->getConfig()->getTitle()->prepend(__('Featured Products Block'));
        $this->_view->getPage()->getConfig()->getTitle()->prepend(
            $model->getBlockId() ? $model->getTitle() : __('New Block')
        );


        $breadcrumb = $id ? __('Edit Block') : __('New Block');
        $this->_addBreadcrumb($breadcrumb, $breadcrumb);
        $this->_view->renderLayout();
    }
}
