<?php
namespace Vnecoms\FeaturedProduct\Controller\Adminhtml\Index;

use Vnecoms\FeaturedProduct\Controller\Adminhtml\AbstractAction;

use Magento\Framework\Controller\ResultFactory;
use Magento\Backend\App\Action\Context;
use Magento\Ui\Component\MassAction\Filter;
use Vnecoms\FeaturedProduct\Model\ResourceModel\Block\CollectionFactory;
use Magento\Framework\Registry;
use Magento\Framework\Stdlib\DateTime\Filter\Date;

/**
 * Class MassDelete
 */
class MassStatus extends AbstractAction
{
    /**
     * @var Filter
     */
    protected $filter;

    /**
     * @var CollectionFactory
     */
    protected $collectionFactory;

    /**
     * Is access to section allowed
     *
     * @return bool
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Vnecoms_FeaturedProduct::block') && parent::_isAllowed();
    }
    
    /**
     * 
     * @param Context $context
     * @param Registry $coreRegistry
     * @param Date $dateFilter
     * @param Filter $filter
     * @param CollectionFactory $collectionFactory
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        Date $dateFilter,
        Filter $filter,
        CollectionFactory $collectionFactory
    ) {
        $this->filter = $filter;
        $this->collectionFactory = $collectionFactory;
        parent::__construct($context, $coreRegistry, $dateFilter);
    }

    /**
     * Execute action
     *
     * @return \Magento\Backend\Model\View\Result\Redirect
     * @throws \Magento\Framework\Exception\LocalizedException|\Exception
     */
    public function execute()
    {
        $collection = $this->filter->getCollection($this->collectionFactory->create());
        $collectionSize = $collection->getSize();
        $status = $this->getRequest()->getParam('status');
        $cache = $this->_objectManager->get('Magento\Framework\App\CacheInterface');
        foreach ($collection as $block) {
            $block->setStatus($status);
            $block->save();
            
            $cache->clean(['vnecoms_featuredproduct_block_'.$block->getId()]);
            $cache->clean(['vnecoms_featuredproduct_block_productlist_'.$block->getId()]);
        }

        $this->messageManager->addSuccess(__('A total of %1 record(s) have been updated.', $collectionSize));

        /** @var \Magento\Backend\Model\View\Result\Redirect $resultRedirect */
        $resultRedirect = $this->resultFactory->create(ResultFactory::TYPE_REDIRECT);
        return $resultRedirect->setPath('*/*/');
    }
}
