<?php
namespace Vnecoms\FeaturedProduct\Controller\Adminhtml\Index;

use Vnecoms\FeaturedProduct\Controller\Adminhtml\AbstractAction;
use Magento\Framework\Exception\LocalizedException;

class Save extends AbstractAction
{
    /**
     * @return void
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     */
    public function execute()
    {
        if ($this->getRequest()->getPostValue()) {
            /** @var Vnecoms\HomeTabs\Model\Tab $model */
            $model = $this->_objectManager->create('Vnecoms\FeaturedProduct\Model\Block');
            try {
                $data = $this->getRequest()->getPostValue();
                $data = isset($data['block'])?$data['block']:[];
                $id = isset($data['block_id'])?$data['block_id']:'';
                if ($id) {
                    $model->load($id);
                }
                
                $ruleData = $this->getRequest()->getParam('rule');
                if (isset($ruleData['conditions'])) {
                    /* Process rule conditions*/
                    $tmpData = ['conditions'=>$ruleData['conditions']];
                    
                    /** @var \Magento\CatalogRule\Model\Rule $model */
                    $tmpRule = $this->_objectManager->create('Vnecoms\FeaturedProduct\Model\Rule');
                    
                    $tmpRule->loadPost($tmpData);
                    $data['conditions_serialized'] = $tmpRule->getSerializedConditions();
                }
                if (isset($data['additional_data']) && is_array($data['additional_data'])) {
                    $data['additional_data'] = json_encode($data['additional_data']);
                }
                
                $model->loadPost($data);

                $this->_objectManager->get('Magento\Backend\Model\Session')->setBlockData($data);

                $model->save();
                
                $cache = $this->_objectManager->get('Magento\Framework\App\CacheInterface');
                $cache->clean(['vnecoms_featuredproduct_block_'.$model->getId()]);
                $cache->clean(['vnecoms_featuredproduct_block_productlist_'.$model->getId()]);
                
                $this->messageManager->addSuccess(__('You saved the block.'));
                $this->_objectManager->get('Magento\Backend\Model\Session')->setBlockData(false);

                if ($this->getRequest()->getParam('back')) {
                    $this->_redirect('featuredproduct/*/edit', ['id' => $model->getId()]);
                    return;
                }
                $this->_redirect('featuredproduct/*/');
                return;
            } catch (LocalizedException $e) {
                $this->messageManager->addError($e->getMessage());
            } catch (\Exception $e) {
                $this->messageManager->addError(
                    __('Something went wrong while saving the rule data. Please review the error log.')
                );
                $this->_objectManager->get('Psr\Log\LoggerInterface')->critical($e);
                $this->_objectManager->get('Magento\Backend\Model\Session')->setPageData($data);
            }
        }
        $this->_redirect('featuredproduct/index/');
    }
}
