<?php
namespace Vnecoms\FeaturedProduct\Controller\Block;

use Magento\Framework\App\Action\Context;
use Vnecoms\FeaturedProduct\Model\Source\Template;

class Load extends \Magento\Framework\App\Action\Action
{
    /**
     * @var \Magento\Framework\Controller\Result\JsonFactory
     */
    protected $_resultJsonFactory;
    
    /**
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry;
    
    /**
     *
     * @param Context $context
     * @param \Magento\Framework\Registry $coreRegistry
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     */
    public function __construct(
        Context $context,
        \Magento\Framework\Registry $coreRegistry,
        \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
    ) {
        $this->_resultJsonFactory = $resultJsonFactory;
        $this->coreRegistry = $coreRegistry;
        parent::__construct($context);
    }
    
    
    public function execute()
    {
        $refererUrl = $this->_request->getServer('HTTP_REFERER');
        $baseUrl =  trim($this->_url->getBaseUrl(), '/');
        $requestUri = '';
        if ($refererUrl) {
            $requestUri = trim(str_replace($baseUrl, '', $refererUrl), '/');
        }
        $this->getRequest()->setRequestUri($requestUri);
        
        $blockId = $this->getRequest()->getParam('block_id');
        $block = $this->_objectManager->create('Vnecoms\FeaturedProduct\Model\Block');
        $block->load($blockId);
        
        $response = new \Magento\Framework\DataObject();
        if (!$block->getBlockId()) {
            $response->setData([
                'product_list' => __('The block content is not available.'),
            ]);
        } else {
            $this->coreRegistry->register('current_block', $block);
            $this->coreRegistry->register('block', $block);
            
            $categoryId = (int)$this->getRequest()->getParam('category', false);
            if ($categoryId) {
                $category = $this->_objectManager->get('Magento\Catalog\Model\Category')->load($categoryId);
                if ($category->getId()) {
                    $this->coreRegistry->register('current_category', $category);
                }
            }
            
            $this->_view->loadLayout();
            $productListBlock = $this->_view->getLayout()->getBlock('featuredproduct.products_list');
            if ($block->getTemplate() == Template::TEMPLATE_SLIDER) {
                $productListBlock->setTemplate('Vnecoms_FeaturedProduct::product/slider.phtml');
            }
            $additionalData = json_decode($block->getAdditionalData(), true);
            $response->setData([
                'product_list' => $productListBlock->toHtml(),
                'additional_data' => $additionalData?$additionalData:[],
            ]);
        }
        
        return $this->_resultJsonFactory->create()->setJsonData($response->toJson());
    }
}
