<?php
namespace Vnecoms\FeaturedProduct\Model;

class Rule extends \Magento\CatalogRule\Model\Rule
{
    /**
     * Get array of product ids which are matched by rule
     *
     * @return array
     */
    public function getMatchingProductIds()
    {
        if ($this->_productIds === null) {
            $this->_productIds = [];
            $this->setCollectedAttributes([]);
    
            /** @var $productCollection \Magento\Catalog\Model\ResourceModel\Product\Collection */
            $productCollection = $this->_productCollectionFactory->create();
            if ($this->_productsFilter) {
                $productCollection->addIdFilter($this->_productsFilter);
            }
            $this->getConditions()->collectValidatedAttributes($productCollection);

            $this->_resourceIterator->walk(
                $productCollection->getSelect(),
                [[$this, 'callbackValidateProduct']],
                [
                    'attributes' => $this->getCollectedAttributes(),
                    'product' => $this->_productFactory->create()
                ]
            );
        }
    
        return $this->_productIds;
    }
    
    public function callbackValidateProduct($args)
    {
        $product = clone $args['product'];
        $product->setData($args['row']);
    
        $websites = $this->_getWebsitesMap();
        $results = [];
    
        foreach ($websites as $websiteId => $defaultStoreId) {
            $product->setStoreId($defaultStoreId);
            if (!isset($this->_productIds[$websiteId])) {
                $this->_productIds[$websiteId] = [];
            }
            if ($this->getConditions()->validate($product)) {
                $this->_productIds[$websiteId][] = $product->getId();
            }
        }
    }
    
    /**
     * Get serialized
     * @return string
     */
    public function getSerializedConditions(){
        if(property_exists($this, 'serializer')){
            return $this->serializer->serialize($this->getConditions()->asArray());
        }
        return serialize($this->getConditions()->asArray());
    }
}
