<?php
namespace Vnecoms\Sms\Controller\Account;

use Magento\Framework\App\Action\Context;

class SmsPost extends \Magento\Customer\Controller\AbstractAccount
{
    /**
     * @var \Vnecoms\Sms\Helper\Data
     */
    protected $helper;
    
    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;
    
    /**
     * @var \Vnecoms\Sms\Model\MobileFactory
     */
    protected $mobileFactory;
    
    /**
     * @param Context $context
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Vnecoms\Sms\Model\MobileFactory $mobileFactory
     */
    public function __construct(
        Context $context,
        \Vnecoms\Sms\Helper\Data $helper,
        \Magento\Customer\Model\Session $customerSession,
        \Vnecoms\Sms\Model\MobileFactory $mobileFactory
    ) {
        $this->helper = $helper;
        $this->customerSession = $customerSession;
        $this->mobileFactory = $mobileFactory;
        parent::__construct($context);
    }
    /**
     * Default customer account page
     *
     * @return \Magento\Framework\View\Result\Page
     */
    public function execute()
    {
        $mobileNum = $this->getRequest()->getParam('mobile_number');
        $otp = $this->getRequest()->getParam('mobile_otp');
        try{
            if(!$mobileNum){
                throw new \Exception(__("Please enter your mobile number."));
            }
            
            $customer = $this->customerSession->getCustomer();
            
            $mobile = $this->mobileFactory->create();
            $mobile->load($customer->getId(), 'customer_id');
            
            if($this->helper->isEnableVerifyingCustomerMobile()){
                $mobile->setCustomerId($customer->getId())->save();
            }else{
                $mobile->addData([
                    'customer_id' => $customer->getId(),
                    'mobile' => $mobileNum,
                ])->save();
            }
            
            $this->messageManager->addSuccess(__("The mobile number is updated."));
        }catch(\Exception $e){
            $this->messageManager->addError($e->getMessage());
        }
        
        $this->_redirect('customer/account/sms');
    }
}
