<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Checkout;

    class ApplyCoupon extends AbstractCheckout      {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["success"]      = false;
            $returnArray["message"]      = "";
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $itemId       = $wholeData["itemId"]       ?? 0;
                        $quoteId      = $wholeData["quoteId"]      ?? 0;
                        $storeId      = $wholeData["storeId"]      ?? 1;
                        $customerId   = $wholeData["customerId"]   ?? 0;
                        $couponCode   = $wholeData["couponCode"]   ?? "";
                        $removeCoupon = $wholeData["removeCoupon"] ?? 0;
                        $environment  = $this->_emulate->startEnvironmentEmulation($storeId);
                        $quote        = new \Magento\Framework\DataObject();
                        if ($customerId != 0) {
                            $quoteCollection = $this->_quoteFactory->create()->getCollection()
                                ->addFieldToFilter("customer_id", $customerId)
                                ->addFieldToFilter("is_active", 1)
                                ->addOrder("updated_at", "DESC");
                            $quote = $quoteCollection->getFirstItem();
                        }
                        if ($quoteId != 0)
                            $quote = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
                        if((bool)$removeCoupon)
                            $couponCode = "";
                        $codeLength = strlen($couponCode);
                        $isCodeLengthValid = $codeLength && $codeLength <= \Magento\Checkout\Helper\Cart::COUPON_CODE_MAX_LENGTH;
                        $itemsCount = $quote->getItemsCount();
                        if ($itemsCount) {
                            $quote->getShippingAddress()->setCollectShippingRates(true);
                            $quote->setCouponCode($isCodeLengthValid ? $couponCode : "")->collectTotals();
                            $this->_quoteRepository->save($quote);
                        }
                        if ($codeLength) {
                            $escaper = $this->_escaper;
                            if (!$itemsCount) {
                                if ($isCodeLengthValid) {
                                    $coupon = $this->_couponFactory->create();
                                    $coupon->load($couponCode, "code");
                                    if ($coupon->getId()) {
                                        $quote->setCouponCode($couponCode)->save();
                                        $returnArray["success"] = true;
                                        $returnArray["message"] = __("You used coupon code '%1'.", $escaper->escapeHtml($couponCode));
                                    } else
                                        $returnArray["message"] = __("The coupon code '%1' is not valid.", $escaper->escapeHtml($couponCode));
                                } else
                                    $returnArray["message"] = __("The coupon code '%1' is not valid.", $escaper->escapeHtml($couponCode));
                            } else {
                                if ($isCodeLengthValid && $couponCode == $quote->getCouponCode()) {
                                    $returnArray["success"] = true;
                                    $returnArray["message"] = __("You used coupon code '%1'.", $escaper->escapeHtml($couponCode));
                                } else {
                                    $returnArray["message"] = __("The coupon code '%1' is not valid.", $escaper->escapeHtml($couponCode));
                                    $quote->collectTotals();
                                    $this->_quoteRepository->save($quote);
                                }
                            }
                        } else {
                            $returnArray["success"] = true;
                            $returnArray["message"] = __("You canceled the coupon code.");
                        }
                        $quote->getShippingAddress()->setCollectShippingRates(true);
                        $quote->collectTotals()->save();
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }