<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Checkout;

    use Amasty\Deliverydate\Model\DeliverydateConfigProvider;

    class ShippingPaymentMethodInfo extends AbstractCheckout    {

        public function execute()   {
            $returnArray                    = [];
            $returnArray["message"]         = "";
            $returnArray["success"]         = false;
            $returnArray["cartCount"]       = 0;
            $returnArray["paymentMethods"]  = [];
            $returnArray["shippingMethods"] = [];
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width         = $wholeData["width"]          ?? 1000;
                        $taxvat         = $wholeData["taxvat"]         ?? "";
                        $quoteId        = $wholeData["quoteId"]        ?? 0;
                        $storeId        = $wholeData["storeId"]        ?? 1;
                        $customerId     = $wholeData["customerId"]     ?? 0;
                        $checkoutMethod = $wholeData["checkoutMethod"] ?? "";
                        $environment    = $this->_emulate->startEnvironmentEmulation($storeId);
                        $customer = $this->_customerFactory->create()->load($customerId);
                        $defaultBillingAddressId  = $customer->getDefaultBilling();
                        $defaultShippingAddressId = $customer->getDefaultShipping();
                        if ($defaultBillingAddressId == $defaultShippingAddressId)
                            $useForShipping = 1;
                        $billingData    = $wholeData["billingData"]    ?? '{"addressId":"'.$defaultBillingAddressId.'","newAddress":{},"useForShipping":"'.$useForShipping.'"}';
                        $shippingData   = $wholeData["shippingData"]   ?? '{"addressId":"'.$defaultShippingAddressId.'","newAddress":{},"sameAsBilling":"'.$useForShipping.'"}';
                        $billingData    = $this->_jsonHelper->jsonDecode($billingData);
                        $shippingData   = $this->_jsonHelper->jsonDecode($shippingData);
                        
                        $store          = $this->_store;
                        $baseCurrency   = $store->getBaseCurrencyCode();
                        $currency       = $wholeData["currency"] ?? $baseCurrency;
                        $store->setCurrentCurrencyCode($currency);
                        $quote          = new \Magento\Framework\DataObject();
                        if ($customerId != 0) {
                            $quoteCollection = $this->_quoteFactory->create()->getCollection()
                                ->addFieldToFilter("customer_id", $customerId)
                                ->addFieldToFilter("is_active", 1)
                                ->addOrder("updated_at", "DESC");
                            $quote = $quoteCollection->getFirstItem();
                        }
                        if ($quoteId != 0)
                            $quote = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
                        if($quote->getItemsQty()*1 == 0){
                            $returnArray["message"] = __("Sorry Something went wrong !!");
                            return $this->getJsonResponse($returnArray);
                        }
                        else{
                            $returnArray["cartCount"] = $quote->getItemsQty()*1;
                        }
                        $useForShipping = 0;
                        if(!empty($billingData))    {
                            $saveInAddressBook = 0;
                            if(isset($billingData["newAddress"]["saveInAddressBook"]))
                                $saveInAddressBook = $billingData["newAddress"]["saveInAddressBook"];
                            if($checkoutMethod == "register")
                                $saveInAddressBook = 1;
                            if($billingData["useForShipping"] != "")
                                $useForShipping = $billingData["useForShipping"];
                            $addressId = 0;
                            if($billingData["addressId"] != "")
                                $addressId = $billingData["addressId"];
                            $quote->setCheckoutMethod($checkoutMethod)->save();
                            $newAddress = [];
                            if($billingData["newAddress"] != "")
                                if(!empty($billingData["newAddress"]))
                                    $newAddress = $billingData["newAddress"];
                            $address     = $quote->getBillingAddress();
                            $addressForm = $this->_customerForm;
                            $addressForm->setFormCode("customer_address_edit")->setEntityType("customer_address");
                            if($addressId > 0) {
                                $customerAddress = $this->_customerAddress->load($addressId)->getDataModel();
                                if($customerAddress->getId()) {
                                    if($customerAddress->getCustomerId() != $quote->getCustomerId()){
                                        $returnArray["message"] = __("Customer Address is not valid.");
                                        return $this->getJsonResponse($returnArray);
                                    }
                                    $address->importCustomerAddressData($customerAddress)->setSaveInAddressBook(0);
                                    $addressForm->setEntity($address);
                                    $addressErrors = $addressForm->validateData($address->getData());
                                    if($addressErrors !== true){
                                        $returnArray["message"] = implode(", ", $addressErrors);
                                        return $this->getJsonResponse($returnArray);
                                    }
                                }
                            }
                            else {
                                $addressForm->setEntity($address);
                                $addressData = [
                                    "firstname"  => $newAddress["firstName"],
                                    "lastname"   => $newAddress["lastName"],
                                    "middlename" => $newAddress["middleName"] ?? "",
                                    "prefix"     => $newAddress["prefix"]     ?? "",
                                    "suffix"     => $newAddress["suffix"]     ?? "",
                                    "company"    => $newAddress["company"],
                                    "street"     => $newAddress["street"],
                                    "city"       => $newAddress["city"],
                                    "country_id" => $newAddress["country_id"],
                                    "region"     => $newAddress["region"],
                                    "region_id"  => $newAddress["region_id"],
                                    "postcode"   => $newAddress["postcode"],
                                    "telephone"  => $newAddress["telephone"],
                                    "fax"        => $newAddress["fax"],
                                    "taxvat"     => $newAddress["taxvat"]     ?? "",
                                    "dob"        => $newAddress["dob"]        ?? "",
                                    "gender"     => $newAddress["gender"]     ?? ""
                                ];
                                $addressErrors  = $addressForm->validateData($addressData);
                                if($addressErrors !== true){
                                    $returnArray["message"] = implode(", ", $addressErrors);
                                    return $this->getJsonResponse($returnArray);
                                }
                                $addressForm->compactData($addressData);
                                $address->setCustomerAddressId(null);
                                $address->setSaveInAddressBook($saveInAddressBook);
                                $quote->setCustomerFirstname($newAddress["firstName"])->setCustomerLastname($newAddress["lastName"]);
                            }
                            if(in_array($checkoutMethod, ["register", "guest"])){
                                $websiteId = $this->_storeManager->getStore()->getWebsiteId();
                                if (!empty($newAddress["email"]) && !\Zend_Validate::is($newAddress["email"], "EmailAddress")) {
                                    $returnArray["message"] = __("Invalid email format");
                                    return $this->getJsonResponse($returnArray);
                                }
                                if($this->_customerFactory->create()->setWebsiteId($websiteId)->loadByEmail(trim($newAddress["email"]))->getId() > 0){
                                    $returnArray["message"] = __("Email already exist");
                                    return $this->getJsonResponse($returnArray);
                                }
                                $quote->setCustomerEmail(trim($newAddress["email"]));
                                if(in_array($this->_helper->getConfigData("customer/address/taxvat_show"), ["req", "opt"]))
                                    $quote->setCustomerTaxvat($newAddress["taxvat"]);
                                $address->setEmail(trim($newAddress["email"]));
                            }
                            if(!$address->getEmail() && $quote->getCustomerEmail()){
                                $address->setEmail($quote->getCustomerEmail());
                            }
                            if(($validateRes = $address->validate()) !== true){
                                $returnArray["message"] = implode(",", $validateRes);
                                return $this->getJsonResponse($returnArray);
                            }
                            if(true !== ($result = $this->_validateCustomerData($wholeData))) {
                                $returnArray["message"] = implode(",", $result);
                                return $this->getJsonResponse($returnArray);
                            }
                            if(!$quote->getCustomerId() && "register" == $quote->getCheckoutMethod()) {
                                if($this->_customerEmailExists($address->getEmail(), $this->_storeManager->getStore()->getWebsiteId())){
                                    $returnArray["message"] = __("This email already exist.");
                                    return $this->getJsonResponse($returnArray);
                                }
                            }
                            if(!$quote->isVirtual()) {
                                $usingCase = isset($useForShipping) ? (int)$useForShipping : 0;
                                switch($usingCase) {
                                    case 0:
                                        $shipping = $quote->getShippingAddress();
                                        $shipping->setSameAsBilling(0);
                                        $setStepDataShipping = 0;
                                        break;
                                    case 1:
                                        $billing = clone $address;
                                        $billing->unsAddressId()->unsAddressType();
                                        $shipping = $quote->getShippingAddress();
                                        $shippingMethod = $shipping->getShippingMethod();
                                        $shipping->addData($billing->getData())
                                            ->setSameAsBilling(1)
                                            ->setSaveInAddressBook(0)
                                            ->setShippingMethod($shippingMethod)
                                            ->setCollectShippingRates(true);
                                        $setStepDataShipping = 1;
                                        break;
                                }
                            }
                            $quote->collectTotals()->save();
                            if(!$quote->isVirtual() && $setStepDataShipping)
                                $quote->getShippingAddress()->setCollectShippingRates(true);
                        }
                        else{
                            $returnArray["message"] = __("Invalid Billing data.");
                            return $this->getJsonResponse($returnArray);
                        }
                        $returnArray['lastName'] = $address->getLastname();
                        $returnArray['firstName'] = $address->getFirstname();
                        $returnArray['countryCode'] = $address->getCountry();
                        $returnArray['email'] = $customer->getEmail() ? $customer->getEmail() : $quote->getCustomerEmail();
                        $returnArray['mobileNumber'] = $address->getTelephone();
// step 4 process starts here ///////////////////////////////////////////////////////////////////////////////////////////////////
                        if(!$quote->isVirtual()){
                            if($useForShipping == 0){
                                if($shippingData != ""){
                                    $sameAsBilling = 0;
                                    if($shippingData["sameAsBilling"] != "")
                                        $sameAsBilling = $shippingData["sameAsBilling"];
                                    $newAddress = [];
                                    if($shippingData["newAddress"] != "")
                                        if(!empty($shippingData["newAddress"]))
                                            $newAddress = $shippingData["newAddress"];
                                    $addressId = 0;
                                    if($shippingData["addressId"] != "")
                                        $addressId = $shippingData["addressId"];
                                    $saveInAddressBook = 0;
                                    if(isset($shippingData["newAddress"]["saveInAddressBook"]) && $shippingData["newAddress"]["saveInAddressBook"] != "")
                                        $saveInAddressBook = $shippingData["newAddress"]["saveInAddressBook"];
                                    $address = $quote->getShippingAddress();
                                    $addressForm = $this->_customerForm;
                                    $addressForm->setFormCode("customer_address_edit")->setEntityType("customer_address");
                                    if($addressId > 0) {
                                        $customerAddress = $this->_customerAddress->load($addressId)->getDataModel();
                                        if($customerAddress->getId()) {
                                            if($customerAddress->getCustomerId() != $quote->getCustomerId()){
                                                $returnArray["message"] = __("Customer Address is not valid.");
                                                return $this->getJsonResponse($returnArray);
                                            }
                                            $address->importCustomerAddressData($customerAddress)->setSaveInAddressBook(0);
                                            $addressForm->setEntity($address);
                                            $addressErrors  = $addressForm->validateData($address->getData());
                                            if($addressErrors !== true){
                                                $returnArray["message"] = implode(", ", $addressErrors);
                                                return $this->getJsonResponse($returnArray);
                                            }
                                        }
                                    }
                                    else {
                                        $addressForm->setEntity($address);
                                        $addressData = [
                                            "firstname"  => $newAddress["firstName"],
                                            "lastname"   => $newAddress["lastName"],
                                            "middlename" => $newAddress["middleName"] ?? "",
                                            "prefix"     => $newAddress["prefix"]     ?? "",
                                            "suffix"     => $newAddress["suffix"]     ?? "",
                                            "company"    => $newAddress["company"],
                                            "street"     => $newAddress["street"],
                                            "city"       => $newAddress["city"],
                                            "country_id" => $newAddress["country_id"],
                                            "region"     => $newAddress["region"],
                                            "region_id"  => $newAddress["region_id"],
                                            "postcode"   => $newAddress["postcode"],
                                            "telephone"  => $newAddress["telephone"],
                                            "fax"        => $newAddress["fax"],
                                            "taxvat"     => $newAddress["taxvat"]     ?? "",
                                            "dob"        => $newAddress["dob"]        ?? "",
                                            "gender"     => $newAddress["gender"]     ?? ""
                                        ];
                                        $addressErrors = $addressForm->validateData($addressData);
                                        if($addressErrors !== true){
                                            $returnArray["message"] = implode(", ", $addressErrors);
                                            return $this->getJsonResponse($returnArray);
                                        }
                                        $addressForm->compactData($addressData);
                                        $address->setCustomerAddressId(null);
// Additional form data, not fetched by extractData (as it fetches only attributes) /////////////////////////////////////////////
                                        $address->setSaveInAddressBook($saveInAddressBook);
                                        $address->setSameAsBilling($sameAsBilling);
                                    }
                                    // $address->implodeStreetAddress();
                                    $address->setCollectShippingRates(true);
                                    if(($validateRes = $address->validate()) !== true){
                                        $returnArray["message"] = implode(", ", $validateRes);
                                        return $this->getJsonResponse($returnArray);
                                    }
                                    $quote->collectTotals()->save();
                                }
                                else{
                                    $returnArray["message"] = __("Invalid Shipping data.");
                                    return $this->getJsonResponse($returnArray);
                                }
                            }
                            $quote->getShippingAddress()->collectShippingRates()->save();
                            $shippingRateGroups = $quote->getShippingAddress()->getGroupedAllShippingRates();
                            foreach($shippingRateGroups as $code => $rates) {
                                $oneShipping = [];
                                $oneShipping["title"] = $this->_helperCatalog->stripTags($this->_helper->getConfigData("carriers/".$code."/title"));
                                foreach($rates as $rate){
                                    $oneMethod = [];
                                    if($rate->getErrorMessage())
                                        $oneMethod["error"] = $rate->getErrorMessage();
                                    $oneMethod["code"]  = $rate->getCode();
                                    $oneMethod["label"] = $rate->getMethodTitle();
                                    $oneMethod["price"] = $this->_helperCatalog->stripTags($this->_priceHelper->currency((float) $rate->getPrice()));
                                    $oneShipping["method"][] = $oneMethod;
                                }
                                $returnArray["shippingMethods"][] = $oneShipping;
                            }
                        }
                        foreach($this->_paymentHelper->getStoreMethods($storeId, $quote) as $method) {
                            $oneMethod          = [];
                            $oneMethod["code"]  = $method->getCode();
                            $oneMethod["title"] = $method->getTitle();
                            $oneMethod["extraInformation"] = "";
                            if(in_array($method->getCode(), ["paypal_standard", "paypal_express"])){
                                $oneMethod["extraInformation"] = __("You will be redirected to the PayPal website.");
                                $config = $this->_paypalConfig->setMethod($method->getCode());
                                $locale = $this->_localeResolver;
                                $oneMethod["title"]    = "";
                                $oneMethod["link"]     = $config->getPaymentMarkWhatIsPaypalUrl($locale);
                                $oneMethod["imageUrl"] = $config->getPaymentMarkImageUrl($locale->getDefaultLocale());
                            }
                            else
                            if(in_array($method->getCode(), ["paypal_express_bml"])){
                                $oneMethod["extraInformation"] = __("You will be redirected to the PayPal website.");
                                $oneMethod["title"]    = "";
                                $oneMethod["link"]     = "https://www.securecheckout.billmelater.com/paycapture-content/fetch?hash=AU826TU8&content=/bmlweb/ppwpsiw.html";
                                $oneMethod["imageUrl"] = "https://www.paypalobjects.com/webstatic/en_US/i/buttons/ppc-acceptance-medium.png";
                            }
                            else
                            if($method->getCode() == "checkmo"){
                                if($method->getPayableTo())
                                    $extraInformationPrefix = __("Make Check payable to:");
                                else
                                    $extraInformationPrefix = __("Send Check to:");
                                $extraInformation = $this->_helper->getConfigData("payment/".$method->getCode()."/mailing_address");
                                if($extraInformation == "")
                                    $extraInformation = __(" xxxxxxx");
                                $oneMethod["extraInformation"] = $extraInformationPrefix.$extraInformation;
                            }
                            else
                            if($method->getCode() == "banktransfer"){
                                $extraInformation = $this->_helper->getConfigData("payment/".$method->getCode()."/instructions");
                                if($extraInformation == "")
                                    $extraInformation = __("Bank Details are xxxxxxx");
                                $oneMethod["extraInformation"] = $extraInformation;
                            }
                            else
                            if($method->getCode() == "cashondelivery"){
                                $extraInformation = $this->_helper->getConfigData("payment/".$method->getCode()."/instructions");
                                if($extraInformation == "")
                                    $extraInformation = __("Pay at the time of delivery");
                                $oneMethod["extraInformation"] = $extraInformation;
                            }
                            else
                            if (in_array($method->getCode(), ["webkul_stripe", "authorizenet"])) {
                                $allowedCc            = [];
                                $allowedCcTypesString = $method->getConfigData("cctypes");
                                $allowedCcTypes       = explode(",", $allowedCcTypesString);
                                $ccTypes              = $this->_ccType->toOptionArray();
                                $types                = [];
                                foreach ($ccTypes as $data) {
                                    if (isset($data["value"]) && isset($data["label"]))
                                        $types[$data["value"]] = $data["label"];
                                }
                                foreach ($allowedCcTypes as $value) {
                                    $eachAllowedCc         = [];
                                    $eachAllowedCc["code"] = $value;
                                    $eachAllowedCc["name"] = $types[$value];
                                    $allowedCc[]           = $eachAllowedCc;
                                }
                                $extraInformation          = $allowedCc;
                                $oneMethod["extraInformation"] = $extraInformation;
                            }
                            $returnArray["paymentMethods"][] = $oneMethod;
                        }
                        $allItems = $quote->getAllVisibleItems();
                        $itemData = [];
                        foreach ($allItems as $item) {
                            $eachItem          = []; 
                            $product           = $item->getProduct();
                            $eachItem['name']  = $item->getName();
                            $eachItem['price'] = $this->_helperCatalog->stripTags($this->_priceHelper->currency((float) $item->getPrice()));
                            $eachItem['qty']   = $item->getQty();
                            $eachItem["image"] = $this->_helperCatalog->getImageUrl($product, $width/2.5, "product_page_image_small");
                            $itemData[]        = $eachItem;
                        }
                        $returnArray['orderReviewData']["items"] = $itemData;
                        $totals = [];
                        $checkoutHelper = $this->_checkoutHelper;
                        if ($quote->isVirtual())
                            $totals = $quote->getBillingAddress()->getTotals();
                        else
                            $totals = $quote->getShippingAddress()->getTotals();
                        if(isset($totals["subtotal"])){
                            $subtotal = $totals["subtotal"];
                            $returnArray['orderReviewData']["subtotal"] = [
                                "title" => $subtotal->getTitle(),
                                "value" => $this->_helperCatalog->stripTags($checkoutHelper->formatPrice($subtotal->getValue())),
                                "unformatedValue" => $subtotal->getValue()
                            ];
                        }
                        if(isset($totals["discount"])){
                            $discount = $totals["discount"];
                            $returnArray['orderReviewData']["discount"] = [
                                "title" => $discount->getTitle(),
                                "value" => $this->_helperCatalog->stripTags($checkoutHelper->formatPrice($discount->getValue())),
                                "unformatedValue" => $discount->getValue()
                            ];
                        }
                        if(isset($totals["tax"])){
                            $tax = $totals["tax"];
                            $returnArray['orderReviewData']["tax"] = [
                                "title" => $tax->getTitle(),
                                "value" => $this->_helperCatalog->stripTags($checkoutHelper->formatPrice($tax->getValue())),
                                "unformatedValue" => $tax->getValue()
                            ];
                        }
                        if(isset($totals["shipping"])){
                            $shipping = $totals["shipping"];
                            $returnArray['orderReviewData']["shipping"] = [
                                "title" => $shipping->getTitle(),
                                "value" => $this->_helperCatalog->stripTags($checkoutHelper->formatPrice($shipping->getValue())),
                                "unformatedValue" => $shipping->getValue()
                            ];
                        }
                        if(isset($totals["grand_total"])){
                            $grandtotal = $totals["grand_total"];
                            $returnArray['orderReviewData']["grandtotal"] = [
                                "title" => $grandtotal->getTitle(),
                                "value" => $this->_helperCatalog->stripTags($checkoutHelper->formatPrice($grandtotal->getValue())),
                                "unformatedValue" => $grandtotal->getValue()
                            ];
                        }
                        $returnArray["deliveryDateFields"] = $this->getDeliveryDateFieldsData($customerId);
                        $returnArray["success"] = true;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch(Exception $e)   {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }
        public function getDeliveryDateFieldsData($customerId)
        {
            $data = [];
            $groupId = 0;
            if ($customerId) {
                $groupId = $this->_customerFactory->create()->load($customerId)->getGroupId();
            }
            if ($this->deliveryHelper->moduleEnabled()) {
    
                if ($this->showByGroup('date_field', $groupId)) {
                    $data['deliverydate_date'] = [
                        'type' => 'date',
                        'label' => __('Delivery Date'),
                        'validation' => [
                            'validate-date' => true
                        ],
                        'notice' => $this->deliveryHelper->getStoreScopeValue('date_field/note'),
                        'visible' => !$this->deliveryHelper->getStoreScopeValue('date_field/enabled_carriers'),
                        'options' => ['amdeliveryconf' => $this->_deliverydate->getDeliveryDateFieldConfig()],
                        'readonly' => 1,
                        'pickerDefaultDateFormat' => $this->_deliverydate->getPickerDateFormat(),
                        'pickerDateTimeFormat' => $this->_deliverydate->getPickerDateFormat(),
                        'outputDateFormat' => DeliverydateConfigProvider::OUTPUT_DATE_FORMAT,
                        'inputDateFormat' => $this->_deliverydate->getInputDateFormat(),
                        'default' => $this->_deliverydate->getDefaultDeliveryDate(),
                        'carriers' => explode(',', $this->deliveryHelper->getStoreScopeValue('date_field/carriers'))
                    ];
                    if ((int)$this->deliveryHelper->getStoreScopeValue('date_field/required') == 1) {
                        $elements['deliverydate_date']['validation']['required-entry'] = true;
                    }
                }
    
                if ($this->deliveryHelper->getStoreScopeValue('time_field/enabled_time')
                    && $this->showByGroup('time_field', $groupId)
                ) {
                    $options = $this->tintervalCollection->toOptionArray();
                    if (!empty($options)) {
                        $data['deliverydate_time'] = [
                            'type' => 'select',
                            'label' =>  __('Delivery Time Interval'),
                            'sortOrder' => 201,
                            'disabled' => false,
                            'validation' =>
                                [
                                    'required-entry' => (bool)$this->deliveryHelper
                                        ->getStoreScopeValue('time_field/required')
                                ],
                            'notice' => $this->deliveryHelper->getStoreScopeValue('time_field/note'),
                            'visible' => !$this->deliveryHelper->getStoreScopeValue('time_field/enabled_carriers'),
                            'caption' =>  __('Please select time interval.'),
                            'options' => $options,
                            'default' => $this->_deliverydate->getDefaultDeliveryTime(),
                            'carriers' => explode(',', $this->deliveryHelper->getStoreScopeValue('time_field/carriers'))
                        ];
                    }
                }
    
                if ($this->deliveryHelper->getStoreScopeValue('comment_field/enabled_comment')
                    && $this->showByGroup('comment_field', $groupId)
                ) {
                    $validation = [
                        'required-entry' => (bool)$this->deliveryHelper->getStoreScopeValue('comment_field/required')
                    ];
                    if ($maxLength = (int)$this->deliveryHelper->getStoreScopeValue('comment_field/maxlength')) {
                        $validation['max_text_length'] = $maxLength;
                    }
    
                    $data['deliverydate_comment'] = [
                        'label' =>  __('Delivery Comments'),
                        'validation' => $validation,
                        'notice' => $this->deliveryHelper->getStoreScopeValue('comment_field/note'),
                        'visible' => !$this->deliveryHelper->getStoreScopeValue('comment_field/enabled_carriers'),
                        'cols' => 5,
                        'rows' => 5,
                    ];
                }
            }
    
            return $data;

        }
        protected function showByGroup($field, $groupId)
        {
            if ($this->deliveryHelper->getStoreScopeValue($field . '/enabled_customer_groups')) {
                $groups = explode(',', $this->deliveryHelper->getStoreScopeValue($field . '/customer_groups'));
                if (!in_array($groupId, $groups)) {
                    return false;
                }
            }
            return true;
        }

    }