<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class ReviewList extends AbstractCustomer    {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["message"]      = "";
            $returnArray["totalCount"]   = 0;
            $returnArray["reviewList"]   = [];
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width       = $wholeData["width"]      ?? 1000;
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $customerId  = $wholeData["customerId"] ?? 0;
                        $pageNumber  = $wholeData["pageNumber"] ?? 1;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $reviews     = $this->_collectionFactory->create()
                            ->addStoreFilter($storeId)
                            ->addCustomerFilter($customerId)
                            ->setDateOrder();
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if($pageNumber >= 1) {
                            $returnArray["totalCount"] = count($reviews);
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $reviews->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
// Creating Review List /////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $reviewList = [];
                        foreach ($reviews as $key=>$review) {
                            $eachReview              = [];
                            $eachReview["date"]      = $this->_customerList->dateFormat($review->getReviewCreatedAt());
                            $eachReview["id"]        = $key;
                            $product                 = $this->_productFactory->create()->load($review->getEntityPkValue());
                            $eachReview["proName"]   = $this->_helperCatalog->stripTags($product->getName());
                            $eachReview["details"]   = $this->_reviewHelper->getDetailHtml($review->getDetail());
                            $eachReview["thumbNail"] = $this->_helperCatalog->getImageUrl($product, $width/3);
                            $eachReview["productId"] = $product->getId();
                            $ratingCollection        = $this->_voteCollection->setReviewFilter($key);
                            $ratingArray = [];
                            foreach ($ratingCollection as $rating) {
                                $eachRating                = [];
                                $eachRating["ratingCode"]  = $this->_helperCatalog->stripTags($rating->getRatingCode());
                                $eachRating["ratingValue"] = number_format($rating->getPercent(), 2, ".", "");
                                $ratingArray[]             = $eachRating;
                            }
                            $eachReview["ratingData"] = $ratingArray;
                            $reviewList[] = $eachReview;
                        }
                        $returnArray["reviewList"] = $reviewList;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }