<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class SaveDeliveryFormData extends AbstractCustomer     {

        public function execute()   {
            $returnArray            = [];
            $returnArray['message'] = '';
            $returnArray['success'] = false;
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId     = $wholeData["storeId"]     ?? 1;
                        $incrementId = $wholeData["incrementId"] ?? 0;
                        $deliveryDate = $wholeData["deliveryDate"]?? '';
                        $intervalId = $wholeData["intervalId"]?? '';
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $order       = $this->_order->loadByIncrementId($incrementId);
                        if ($deliveryDate) {
                            $deliverydate = $this->deliverydateRepository->getByOrder($order->getId());
                            if (!$deliverydate->isCanEditOnFront()) {
                                $returnArray['message'] = __('You are not authorize to update delivery date');
                            }
                            $postData['date'] = $deliveryDate;
                            $postData['tinterval_id'] = $intervalId;
                            $postData['form_key'] = '';
                            $wasDate = $deliverydate->getDate();
                            if ($deliverydate->prepareForSave($postData, $order)) {
                                $deliverydate->validate($order);
                                $this->deliverydateRepository->save($deliverydate);
                                if ($wasDate != $deliverydate->getDate()) {
                                    $this->sendNotification($wasDate, $deliverydate, $order);
                                }
                                $returnArray['message'] = __('Delivery Date has been successfully saved');
                                $returnArray['success'] = true;
                            } else {
                                $returnArray['message'] = __('No data to save');
                            }
                        } else {
                            $returnArray['message'] = __('Invalid Delivery Date');
                        }
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }
            /**
         * Send Email notification to admin
         *
         * @param string $wasDate
         * @param \Amasty\Deliverydate\Model\Deliverydate $deliverydate
         * @param \Magento\Sales\Api\Data\OrderInterface $order
         */
        protected function sendNotification($wasDate, $deliverydate, $order)
        {
            $recipientEmails = explode(',', $this->deliveryHelper->getStoreScopeValue('editable/email'));
            if (empty($recipientEmails)) {
                return;
            }
            $value = $deliverydate->getFormattedDate();
            $deliverydate->setDate($value);

            if ($order->getCustomerIsGuest()) {
                $customerName = $order->getBillingAddress()->getName();
            } else {
                $customerName = $order->getCustomerName();
            }

            $sender = ['email' => $order->getCustomerEmail(), 'name' => $customerName];

            $vars = [
                'delivery' => $deliverydate,
                'was_date' => $this->deliveryHelper->convertDateOutput($wasDate),
                'order' => $order
            ];

            $this->transportBuilder
                ->setTemplateIdentifier('amdeliverydate_admin_email_template')
                ->setTemplateOptions(['area' => 'adminhtml', 'store' => 0])
                ->setTemplateVars($vars)
                ->setFrom($sender)
                ->addTo($recipientEmails);

            $transport = $this->transportBuilder->getTransport();
            $transport->sendMessage();
        }

    }