<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Download;
    use Magento\Downloadable\Model\Link\Purchased\Item;

    class Index extends \Webkul\Mobikul\Controller\ApiController    {

        protected $_downloadableLink;
        protected $_linkPurchasedItem;
        protected $_downloadableHelperFile;
        protected $_downloadableHelperDownload;

        public function __construct(
            Item $linkPurchasedItem,
            \Webkul\Mobikul\Helper\Data $helper,
            \Magento\Store\Model\App\Emulation $emulate,
            \Webkul\Mobikul\Helper\Catalog $helperCatalog,
            \Magento\Framework\App\Action\Context $context,
            \Magento\Downloadable\Model\Link $downloadableLink,
            \Magento\Downloadable\Helper\File $downloadableHelperFile,
            \Magento\Downloadable\Helper\Download $downloadableHelperDownload
        ) {
            $this->_downloadableLink           = $downloadableLink;
            $this->_linkPurchasedItem          = $linkPurchasedItem;
            $this->_downloadableHelperFile     = $downloadableHelperFile;
            $this->_downloadableHelperDownload = $downloadableHelperDownload;
            parent::__construct($helper, $context);
        }

        public function execute()   {
            $wholeData   = $this->getRequest()->getParams();
            $authKey     = $this->getRequest()->getHeader("authKey");
            $authData    = $this->_helper->isAuthorized($authKey);
            if ($authData["code"] == 1) {
                $linkFile          = "";
                $fileName          = "";
                $hash              = $wholeData["hash"];
                $linkPurchasedItem = $this->_linkPurchasedItem->load($hash, "link_hash");
                $downloadsLeft     = $linkPurchasedItem->getNumberOfDownloadsBought() - $linkPurchasedItem->getNumberOfDownloadsUsed();
                $status            = $linkPurchasedItem->getStatus();
                if ($status == Item::LINK_STATUS_AVAILABLE && ($downloadsLeft || $linkPurchasedItem->getNumberOfDownloadsBought() == 0)) {
                    if ($linkPurchasedItem->getLinkType() == \Magento\Downloadable\Helper\Download::LINK_TYPE_FILE) {
                        $linkFile  = $this->_downloadableHelperFile->getFilePath($this->_downloadableLink->getBasePath(), $linkPurchasedItem->getLinkFile());
                        $resourceType = \Magento\Downloadable\Helper\Download::LINK_TYPE_FILE;
                        $helper       = $this->_downloadableHelperDownload;
                        $helper->setResource($linkFile, $resourceType);
                        $fileName     = $helper->getFilename();
                        $contentType  = $helper->getContentType();
                        $this->getResponse()->setHttpResponseCode(200)
                            ->setHeader("Pragma", "public", true)
                            ->setHeader("Cache-Control", "must-revalidate, post-check=0, pre-check=0", true)
                            ->setHeader("Content-type", $contentType, true);
                        if ($fileSize = $helper->getFileSize())
                            $this->getResponse()->setHeader("Content-Length", $fileSize);
                        if ($contentDisposition = $helper->getContentDisposition())
                            $this->getResponse()->setHeader("Content-Disposition", $contentDisposition . "; filename=" . $fileName);
                        $this->getResponse()->clearBody();
                        $this->getResponse()->sendHeaders();
                        $helper->output();
                        $linkPurchasedItem->setNumberOfDownloadsUsed($linkPurchasedItem->getNumberOfDownloadsUsed() + 1);
                        if ($linkPurchasedItem->getNumberOfDownloadsBought() != 0 && !($downloadsLeft - 1))
                            $linkPurchasedItem->setStatus(\Magento\Downloadable\Model\Link\Purchased\Item::LINK_STATUS_EXPIRED);
                        $linkPurchasedItem->save();
                    }
                }
                exit(0);
            } else {
                return $this->getJsonResponse($returnArray, 401, $authData["token"]);
            }
        }

    }