<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Extra;

    class NotificationList extends AbstractMobikul   {

        public function execute()   {
            $returnArray                     = [];
            $returnArray["success"]          = false;
            $returnArray["message"]          = "";
            $returnArray["notificationList"] = [];
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width       = $wholeData["width"]   ?? 1000;
                        $storeId     = $wholeData["storeId"] ?? 1;
                        $mFactor     = $wholeData["mFactor"] ?? 1;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $notificationCollection = $this->_mobikulNotification
                            ->create()
                            ->getCollection()
                            ->addFieldToFilter("status", 1)
                            ->addFieldToFilter("store_id", [["finset"=>[$storeId]], ["finset"=>[0]]])
                            ->setOrder("updated_at", "DESC");
                        $height   = ($width/2) * $mFactor;
                        $width    *= $mFactor;
                        foreach ($notificationCollection as $notification) {
                            $eachNotification                     = [];
                            $eachNotification["id"]               = $notification->getId();
                            $eachNotification["title"]            = $notification->getTitle();
                            $eachNotification["content"]          = implode(" ", array_slice(explode(" ", $notification->getContent()), 0, 3));
                            $eachNotification["notificationType"] = $notification->getType();
                            $basePath = $this->_baseDir.DS."mobikul".DS."notification".DS.$notification->getFilename();
                            if (is_file($basePath)) {
                                $newPath = $this->_baseDir.DS."mobikul".DS."resized".DS."notificationBanner".DS.$width."x".$height.DS.$notification->getFilename();
                                $this->_helperCatalog->resizeNCache($basePath, $newPath, $width, $height);
                                $eachNotification["banner"] = $this->_helper->getUrl("media")."mobikul".DS."resized".DS."notificationBanner".DS.$width."x".$height.DS.$notification->getFilename();
                            }
                            else
                                $eachNotification["banner"] = "";
                            if ($notification->getType() == "category") {
// for category /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                $category = $this->_categoryFactory->create()->load($notification->getProCatId());
                                $eachNotification["categoryName"] = $category->getName();
                                $eachNotification["categoryId"]   = $notification->getProCatId();
                            } elseif ($notification->getType() == "product") {
// for product //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                                $product = $this->_productFactory->create()->load($notification->getProCatId());
                                $eachNotification["productName"]  = $product->getName();
                                $eachNotification["productType"]  = $product->getTypeId();
                                $eachNotification["productId"]    = $notification->getProCatId();
                            }
                            $returnArray["notificationList"][]    = $eachNotification;
                        }
                        $returnArray["success"] = true;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }