<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Sales;

    class Guestview extends \Webkul\Mobikul\Controller\ApiController    {

        protected $_emulate;
        protected $_orderFactory;

        public function __construct(
            \Webkul\Mobikul\Helper\Data $helper,
            \Magento\Store\Model\App\Emulation $emulate,
            \Magento\Framework\App\Action\Context $context,
            \Magento\Sales\Model\OrderFactory $orderFactory
        ) {
            $this->_emulate      = $emulate;
            $this->_orderFactory = $orderFactory;
            parent::__construct($helper, $context);
        }

        public function execute()   {
            $returnArray                 = [];
            $returnArray["message"]      = "";
            $returnArray["success"]      = false;
            try {
                $wholeData           = $this->getRequest()->getPostValue();
                $this->_headers      = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey          = $this->getRequest()->getHeader("authKey");
                    $authData        = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $type        = $wholeData["type"]        ?? "";
                        $email       = $wholeData["email"]       ?? "";
                        $storeId     = $wholeData["storeId"]     ?? 1;
                        $zipCode     = $wholeData["zipCode"]     ?? 0;
                        $lastName    = $wholeData["lastName"]    ?? "";
                        $incrementId = $wholeData["incrementId"] ?? 0;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $errors      = false;
                        $order       = $this->_orderFactory->create();
                        if (!empty($wholeData) && $incrementId && $type) {
                            if (empty($incrementId) || empty($lastName) || empty($type) || empty($storeId) || !in_array($type, ["email", "zip"]) || $type == "email" && empty($email) || $type == "zip" && empty($zipCode)
                            )
                                $errors = true;
                            if (!$errors) {
                                $order = $order->loadByIncrementIdAndStoreId($incrementId, $storeId);
                            }
                            $errors = true;
                            if ($order->getId()) {
                                $billingAddress = $order->getBillingAddress();
                                if (strtolower($lastName) == strtolower($billingAddress->getLastname()) && ($type == "email" && strtolower($email) == strtolower($billingAddress->getEmail()) || $type == "zip" && strtolower($zipCode) == strtolower($billingAddress->getPostcode())))
                                    $errors = false;
                            }
                        }
                        if (!$errors && $order->getId())
                            $returnArray["success"] = true;
                        else
                            $returnArray["message"] = __("You entered incorrect data. Please try again.");
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }