<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Adminhtml\Notification;

    class MassPush extends \Webkul\Mobikul\Controller\Adminhtml\Notification  {

        public function execute()   {
            $resultRedirect   = $this->resultRedirectFactory->create();
            $collection       = $this->_filter->getCollection($this->_collectionFactory->create());
            foreach ($collection->getAllIds() as $key => $notificationId) {
                $model        = $this->_notificationRepository->getById($notificationId);
                $baseTmpPath  = "mobikul/notification/";
                $target       = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA).$baseTmpPath;
                $bannerUrl    = "";
                $androidTopic  = $this->_helper->getConfigData("mobikul/notification/andoridtopic");
                $iosTopic  = $this->_helper->getConfigData("mobikul/notification/iostopic");
                if ($model->getFilename() != "")
                    $bannerUrl = $target.$model->getFilename();
                try{
                    $message = [
                        "id"               => $model->getId(),
                        "body"             => $model->getContent(),
                        "sound"            => "default",
                        "title"            => $model->getTitle(),
                        "message"          => $model->getContent(),
                        "store_id"         => $model->getStoreId(),
                        "banner_url"       => $bannerUrl,
                        "notificationType" => $model->getType()
                    ];
                    if ($model->getType() == "category" && $model->getProCatId() != "") {
// for category /////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $message["categoryName"] = $this->_categoryResource->getAttributeRawValue($model->getProCatId(), "name", 1);
                        $message["categoryId"] = $model->getProCatId();
                    } elseif ($model->getType() == "product" && $model->getProCatId() != "") {
// for product //////////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $message["productName"] = $this->_productResource->getAttributeRawValue($model->getProCatId(), "name", 1);
                        $message["productId"] = $model->getProCatId();
                    }
                  
                    $url     = "https://fcm.googleapis.com/fcm/send";
                    $authKey = $this->_helper->getConfigData("mobikul/notification/apikey");
                    $headers = [
                        "Authorization:key=".$authKey,
                        "Content-Type:application/json"
                    ];
                    // $topic   = $this->_helper->getConfigData("mobikul/notification/topic");
                    // $fields  = [
                    //     "to"                => "/topics/".$topic,
                    //     "data"              => $message,
                    //     "priority"          => "high",
                    //     "notification"      => $message,
                    //     "time_to_live"      => 30,
                    //     "delay_while_idle"  => true,
                    //     "content_available" => true
                    // ];

                    
                    // for android////////////////////////////////////////////////////////////////////
                    $fields = [
                        "to"                => "/topics/".$androidTopic,
                        "data"              => $message,
                        "priority"          => "high",
                        "time_to_live"      => 30,
                        "delay_while_idle"  => true,
                        "content_available" => true
                    ];
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $url);
                    curl_setopt($ch, CURLOPT_POST, true);
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $this->_jsonHelper->jsonEncode($fields));
                    // $result = curl_exec($ch);
                    $androidresult = curl_exec($ch);
                    curl_close($ch);


                    // for ios //////////////////////////////////////////////////////////////////////////////
                    $fields = [
                        "to"                => "/topics/".$iosTopic,
                        "data"              => $message,
                        "priority"          => "high",
                        "time_to_live"      => 30,
                        "notification"      => $message,
                        "delay_while_idle"  => true,
                        "content_available" => true
                    ];
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, $url);
                    curl_setopt($ch, CURLOPT_POST, true);
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                    curl_setopt($ch, CURLOPT_POSTFIELDS, $this->_jsonHelper->jsonEncode($fields));
                    $iosresult = curl_exec($ch);

                    if (!$this->isJson($iosresult) || !$this->isJson($androidresult)) {
                        if (!$this->isJson($iosresult)) {
                            $this->messageManager->addError(__("Sorry something went wrong while sending notification in IOS."));
                        }
                        if (!$this->isJson($androidresult)) {
                            $this->messageManager->addError(__("Sorry something went wrong while sending notification in ANDROID."));
                        }
                        return $resultRedirect->setPath("mobikul/notification/index");
                    }
                } catch (\Exception $e) {
                    $this->messageManager->addError($e->getMessage());
                }
            }
            $this->messageManager->addSuccess(__("Total %1 notification(s) has been pushed successfully.", count($collection)));
            return $resultRedirect->setPath("mobikul/notification/index");
        }

        public function isJson($string) {
            json_decode($string);
            return (json_last_error() == JSON_ERROR_NONE);
        }

        protected function _isAllowed()     {
            return $this->_authorization->isAllowed("Webkul_Mobikul::notification");
        }

    }