<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Catalog;

    class AddToCompare extends AbstractCatalog    {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["success"]      = false;
            $returnArray["message"]      = "";
            try {
                $wholeData      = $this->getRequest()->getPostValue();
                $this->_headers = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey  = $this->getRequest()->getHeader("authKey");
                    $authData = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $productId   = $wholeData["productId"]  ?? 0;
                        $customerId  = $wholeData["customerId"] ?? 0;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        if ($productId && ($this->_customerVisitor->getId() || $customerId != 0)) {
                            try {
                                $product = $this->_productRepository->getById($productId, false, $storeId);
                            } catch (NoSuchEntityException $e) {
                                $product = null;
                            }
                            if ($product) {
                                $item = $this->_compareItemFactory->create();
                                $item->addVisitorId($this->_customerVisitor->getId());
                                if ($customerId != 0)
                                    $item->setCustomerId($customerId);
                                $item->loadByProduct($product);
                                if (!$item->getId()) {
                                    $item->addProductData($product);
                                    $item->save();
                                }
                                $productName = $this->_escaper->escapeHtml($product->getName());
                                $returnArray["message"] = __("You added product %1 to the comparison list.", $productName);
                                $this->_eventManager->dispatch("catalog_product_compare_add_product", ["product"=>$product]);
                            }
                            $this->_compare->calculate();
                        }
                        $returnArray["success"] = true;
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }