<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Catalog;

    class FeaturedProductList extends AbstractCatalog    {

        public function execute()   {
            $returnArray                           = [];
            $returnArray["message"]                = "";
            $returnArray["cartCount"]              = 0;
            $returnArray["totalCount"]             = 0;
            $returnArray["productList"]            = [];
            $returnArray["layeredData"]            = [];
            $returnArray["sortingData"]            = [];
            $returnArray["showSwatchOnCollection"] = false;
            try {
                $wholeData           = $this->getRequest()->getPostValue();
                $this->_headers      = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey         = $this->getRequest()->getHeader("authKey");
                    $authData        = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width       = $wholeData["width"]      ?? 1000;
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $quoteId     = $wholeData["quoteId"]    ?? 0;
                        $sortData    = $wholeData["sortData"]   ?? "[]";
                        $pageNumber  = $wholeData["pageNumber"] ?? 1;
                        $filterData  = $wholeData["filterData"] ?? "[]";
                        $customerId  = $wholeData["customerId"] ?? 0;
                        $sortData    = $this->_jsonHelper->jsonDecode($sortData);
                        $filterData  = $this->_jsonHelper->jsonDecode($filterData);
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
// setting currency /////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $store       = $this->_store;
                        $currency    = $wholeData["currency"] ?? $store->getBaseCurrencyCode();
                        $store->setCurrentCurrencyCode($currency);
// checking is swatch allowed on colletion page /////////////////////////////////////////////////////////////////////////////////
                        $returnArray["showSwatchOnCollection"] = (bool)$this->_helper->getConfigData("catalog/frontend/show_swatches_in_product_list");
// Featured Products ////////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $productCollection = $this->_productCollection->create()
                                ->setStore($storeId)
                                ->addAttributeToSelect("*")
                                ->addAttributeToSelect("as_featured")
                                ->addAttributeToSelect("visibility")
                                ->addStoreFilter()
                                ->addAttributeToFilter("status", ["in"=>$this->_productStatus->getVisibleStatusIds()])
                                ->setVisibility($this->_productVisibility->getVisibleInSiteIds());
                        if($this->_helper->getConfigData("mobikul/configuration/featuredproduct") == 1)
                            $productCollection->getSelect()->order("rand()");
                        else
                            $productCollection->addAttributeToFilter("as_featured", 1);
                        if($this->_helperCatalog->showOutOfStock() == 0)
                            $this->_stockFilter->addInStockFilterToCollection($productCollection);
// Filtering product collection /////////////////////////////////////////////////////////////////////////////////////////////////
                        if(count($filterData) > 0) {
                            for($i=0; $i<count($filterData[0]); ++$i) {
                                if($filterData[0][$i] != "" && $filterData[1][$i] == "price") {
                                    $priceRange    = explode("-", $filterData[0][$i]);
                                    $to            = $priceRange[1];
                                    $from          = $priceRange[0];
                                    $currencyRate  = $productCollection->getCurrencyRate();
                                    $fromRange     = ($from - (.01 / 2)) / $currencyRate;
                                    $toRange       = ($to - (.01 / 2)) / $currencyRate;
                                    $select        = $productCollection->getSelect();
                                    $isFlatEnabled = $this->_productResourceCollection->isEnabledFlat();
                                    if($isFlatEnabled){
                                        if($from !== "")
                                            $select->where("price_index.price".">=".$fromRange);
                                        if($to !== "")
                                            $select->where("price_index.price"."<".$toRange);
                                    }
                                    else{
                                        if($from !== "")
                                            $select->where("price_index.min_price".">=".$fromRange);
                                        if($to !== "")
                                            $select->where("price_index.min_price"."<".$toRange);
                                    }
                                } elseif($filterData[0][$i] != "" && $filterData[1][$i] == "cat") {
                                    $categoryToFilter = $this->_category->create()->load($filterData[0][$i]);
                                    $productCollection->setStoreId($storeId)->addCategoryFilter($categoryToFilter);
                                } else {
                                    $attribute = $this->_eavConfig->getAttribute("catalog_product", $filterData[1][$i]);
                                    $attributeModel = $this->_layerAttribute->create()->setAttributeModel($attribute);
                                    $filterAtr  = $this->_layerFilterAttributeResource->create();
                                    $connection = $filterAtr->getConnection();
                                    $tableAlias = $attribute->getAttributeCode()."_idx";
                                    $conditions = [
                                        "{$tableAlias}.entity_id = e.entity_id",
                                        $connection->quoteInto("{$tableAlias}.attribute_id = ?", $attribute->getAttributeId()),
                                        $connection->quoteInto("{$tableAlias}.store_id = ?", $productCollection->getStoreId()),
                                        $connection->quoteInto("{$tableAlias}.value = ?", $filterData[0][$i]),
                                    ];
                                    $productCollection->getSelect()->join([$tableAlias=>$filterAtr->getMainTable()],implode(" AND ",$conditions),[]);
                                }
                            }
                        }
// Sorting product collection ///////////////////////////////////////////////////////////////////////////////////////////////////
                        if(count($sortData) > 0) {
                            $sortBy = $sortData[0];
                            if($sortData[1] == 0)
                                $productCollection->setOrder($sortBy, "ASC");
                            else
                                $productCollection->setOrder($sortBy, "DESC");
                        }
                        else
                            $productCollection->setOrder("position", "ASC");
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if($pageNumber >= 1) {
                            $returnArray["totalCount"] = $productCollection->getSize();
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $productCollection->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
// Creating product collection //////////////////////////////////////////////////////////////////////////////////////////////////
                        $productList = [];
                        foreach ($productCollection as $eachProduct) {
                            $eachProduct   = $this->_productFactory->create()->load($eachProduct->getId());
                            $productList[] = $this->_helperCatalog->getOneProductRelevantData($eachProduct, $storeId, $width, $customerId);
                        }
                        $returnArray["productList"] = $productList;
// Creating layered attribute collection ////////////////////////////////////////////////////////////////////////////////////////
                        $this->_mobikulLayer->_customCollection = $productCollection;
                        $this->_mobikulLayerPrice->_customCollection = $productCollection;
                        $layeredData = [];
                        $doPrice     = true;
                        if(count($filterData) > 0) {
                            if(in_array("price", $filterData[1]))
                                $doPrice = false;
                        }
                        $filters = $this->_filterableAttributes->getList();
                        foreach ($filters as $filter) {
                            if ($filter->getFrontendInput() == "price") {
                                if ($doPrice) {
                                    $priceFilterModel = $this->_filterPriceDataprovider->create();
                                    if ($priceFilterModel) {
                                        $each            = [];
                                        $each["code"]    = $filter->getAttributeCode();
                                        $each["label"]   = $filter->getFrontendLabel();
                                        $each["options"] = $this->_helperCatalog->getPriceFilter($priceFilterModel, $storeId);
                                        $layeredData[]   = $each;
                                    }
                                }
                            } else {
                                $doAttribute = true;
                                if (count($filterData) > 0) {
                                    if (in_array($filter->getAttributeCode(), $filterData[1]))
                                        $doAttribute = false;
                                }
                                if ($doAttribute) {
                                    $attributeFilterModel = $this->_layerAttribute->create()->setAttributeModel($filter);
                                    if ($attributeFilterModel->getItemsCount()) {
                                        $each            = [];
                                        $each["code"]    = $filter->getAttributeCode();
                                        $each["label"]   = $filter->getFrontendLabel();
                                        $each["options"] = $this->_helperCatalog->getAttributeFilter($attributeFilterModel, $filter);
                                        $layeredData[]   =  $each;
                                    }
                                }
                            }
                        }
                        $returnArray["layeredData"] = $layeredData;
// Creating sort attribute collection ///////////////////////////////////////////////////////////////////////////////////////////
                        $sortingData       = [];
                        $toolbar           = $this->_toolbar;
                        foreach($toolbar->getAvailableOrders() as $key=>$order) {
                            if (in_array($key, ["position", "name", "price"])) {
                                $each          = [];
                                $each["code"]  = $key;
                                $each["label"] = $order;
                                $sortingData[] = $each;
                            }
                        }
                        $returnArray["success"]     = true;
                        $returnArray["sortingData"] = $sortingData;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = __($e->getMessage());
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }