<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Checkout;
    use Magento\Store\Model\App\Emulation;
    use Magento\Framework\App\Action\Action;
    use Magento\Framework\App\Action\Context;
    use Webkul\Mobikul\Helper\Data as HelperData;
    use Magento\Checkout\Model\Cart as CustomerCart;
    use Magento\Sales\Model\Order\Payment\Transaction;
    use Magento\Customer\Api\AccountManagementInterface;
    use Magento\Sales\Model\Order\Email\Sender\OrderSender;
    use Magento\Sales\Model\Order\Email\Sender\InvoiceSender;
    use Magento\Quote\Model\Quote\Address\ToOrder as ToOrderConverter;
    use Magento\Quote\Model\Quote\Item\ToOrderItem as ToOrderItemConverter;
    use Magento\Quote\Model\Quote\Payment\ToOrderPayment as ToOrderPaymentConverter;
    use Magento\Quote\Model\Quote\Address\ToOrderAddress as ToOrderAddressConverter;

    abstract class AbstractCheckout extends \Webkul\Mobikul\Controller\ApiController    {

        protected $_store;
        protected $_ccType;
        protected $_country;
        protected $_emulate;
        protected $_escaper;
        protected $_wishlist;
        protected $_jsonHelper;
        protected $_stockState;
        protected $_orderSender;
        protected $_priceHelper;
        protected $_deviceToken;
        protected $_cartFactory;
        protected $_customerUrl;
        protected $_catalogLink;
        protected $_salesHelper;
        protected $_orderFactory;
        protected $_quoteFactory;
        protected $_coreRegistry;
        protected $_storeManager;
        protected $_customerForm;
        protected $_paypalConfig;
        protected $_helperCatalog;
        protected $_addressHelper;
        protected $_invoiceSender;
        protected $_productOption;
        protected $_couponFactory;
        protected $_dbTransaction;
        protected $_paymentHelper;
        protected $_checkoutHelper;
        protected $_localeResolver;
        protected $_productFactory;
        protected $_quoteValidator;
        protected $_invoiceService;
        protected $_relatedProducts;
        protected $addressInterface;
        protected $_customerFactory;
        protected $_customerAddress;
        protected $_quoteRepository;
        protected $_checkoutSession;
        protected $_quoteManagement;
        protected $_regionCollection;
        protected $_dataObjectHelper;
        protected $_orderEmailSender;
        protected $_requestInfoFilter;
        protected $_accountManagement;
        protected $_countryCollection;
        protected $_objectCopyService;
        protected $_productVisibility;
        protected $_customerRepository;
        protected $_transactionBuilder;
        protected $_orderCustomermanager;
        protected $_downloadableConfiguration;
        protected $_deliverydate;
        /**
         * @var Data
         */
        protected $deliveryHelper;

        /**
         * @var \Amasty\Deliverydate\Model\ResourceModel\Tinterval\Collection
         */
        protected $tintervalCollection;

        public function __construct(
            Context $context,
            Emulation $emulate,
            HelperData $helper,
            OrderSender $orderSender,
            InvoiceSender $invoiceSender,
            \Magento\Store\Model\Store $store,
            \Magento\Framework\Escaper $escaper,
            ToOrderConverter $quoteAddressToOrder,
            \Magento\Customer\Model\Url $customerUrl,
            \Magento\Directory\Model\Country $country,
            \Magento\Framework\Registry $coreRegistry,
            ToOrderItemConverter $quoteItemToOrderItem,
            \Magento\Wishlist\Model\Wishlist $wishlist,
            \Magento\Sales\Helper\Reorder $salesHelper,
            \Magento\Customer\Model\Form $customerForm,
            \Magento\Paypal\Model\Config $paypalConfig,
            \Magento\Payment\Helper\Data $paymentHelper,
            \Magento\Payment\Model\Source\Cctype $ccType,
            \Magento\Checkout\Helper\Data $checkoutHelper,
            AccountManagementInterface $accountManagement,
            \Webkul\Mobikul\Helper\Catalog $helperCatalog,
            \Webkul\Mobikul\Model\DeviceToken $deviceToken,
            \Magento\Quote\Model\QuoteFactory $quoteFactory,
            \Magento\Sales\Model\OrderFactory $orderFactory,
            \Magento\Framework\Json\Helper\Data $jsonHelper,
            \Magento\Customer\Helper\Address $addressHelper,
            Transaction\BuilderInterface $transactionBuilder,
            \Magento\Checkout\Model\Session $checkoutSession,
            \Magento\Checkout\Model\CartFactory $cartFactory,
            \Magento\Customer\Model\Session $customerSession,
            \Magento\Framework\DB\Transaction $dbTransaction,
            \Magento\Customer\Model\Address $customerAddress,
            \Magento\Framework\Locale\Resolver $localeResolver,
            \Magento\Quote\Model\QuoteValidator $quoteValidator,
            ToOrderAddressConverter $quoteAddressToOrderAddress,
            ToOrderPaymentConverter $quotePaymentToOrderPayment,
            \Magento\Framework\Pricing\Helper\Data $priceHelper,
            \Magento\Catalog\Model\Product\Option $productOption,
            \Magento\SalesRule\Model\CouponFactory $couponFactory,
            \Magento\Catalog\Model\ProductFactory $productFactory,
            \Magento\Quote\Model\Quote\Address\Rate $shippingRate,
            \Magento\Framework\DataObject\Copy $objectCopyService,
            \Magento\Quote\Model\QuoteManagement $quoteManagement,
            \Magento\Catalog\Model\Product\LinkFactory $catalogLink,
            \Magento\Store\Model\StoreManagerInterface $storeManager,
            \Magento\Customer\Model\CustomerFactory $customerFactory,
            \Magento\Framework\Api\DataObjectHelper $dataObjectHelper,
            \Magento\Quote\Api\Data\AddressInterface $addressInterface,
            \Magento\Quote\Model\CustomerManagement $customerManagement,
            \Magento\Sales\Model\Service\InvoiceService $invoiceService,
            \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
            \Magento\Catalog\Model\Product\Visibility $productVisibility,
            \Magento\CatalogInventory\Api\StockStateInterface $stockState,
            \Magento\Quote\Model\Quote\Item\RelatedProducts $relatedProducts,
            \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
            \Magento\Customer\Api\CustomerRepositoryInterface $customerRepository,
            \Magento\Sales\Model\Order\Email\Sender\OrderSender $orderEmailSender,
            \Magento\Sales\Api\OrderCustomerManagementInterface $orderCustomermanager,
            \Magento\Directory\Model\ResourceModel\Country\Collection $countryCollection,
            \Magento\Directory\Model\ResourceModel\Region\CollectionFactory $regionCollection,
            \Magento\Downloadable\Helper\Catalog\Product\Configuration $downloadableConfiguration,
            \Magento\Checkout\Api\Data\TotalsInformationInterface $totalsInformationInterface, 
            \Magento\Checkout\Model\TotalsInformationManagement $totalsInformationManagement,
            \Amasty\Deliverydate\Model\DeliverydateConfigProvider $deliverydate,
            \Amasty\Deliverydate\Helper\Data $deliveryHelper,
            \Amasty\Deliverydate\Model\ResourceModel\Tinterval\Collection $tintervalCollection
        ) {
            $this->_store                      = $store;
            $this->_ccType                     = $ccType;
            $this->_country                    = $country;
            $this->_emulate                    = $emulate;
            $this->_escaper                    = $escaper;
            $this->_wishlist                   = $wishlist;
            $this->_jsonHelper                 = $jsonHelper;
            $this->_stockState                 = $stockState;
            $this->_salesHelper                = $salesHelper;
            $this->_catalogLink                = $catalogLink;
            $this->_priceHelper                = $priceHelper;
            $this->_cartFactory                = $cartFactory;
            $this->_customerUrl                = $customerUrl;
            $this->_deviceToken                = $deviceToken;
            $this->_orderSender                = $orderSender;
            $this->_paypalConfig               = $paypalConfig;
            $this->_customerForm               = $customerForm;
            $this->_coreRegistry               = $coreRegistry;
            $this->_shippingRate               = $shippingRate;
            $this->_orderFactory               = $orderFactory;
            $this->_storeManager               = $storeManager;
            $this->_quoteFactory               = $quoteFactory;
            $this->_paymentHelper              = $paymentHelper;
            $this->_productOption              = $productOption;
            $this->_addressHelper              = $addressHelper;
            $this->_helperCatalog              = $helperCatalog;
            $this->_couponFactory              = $couponFactory;
            $this->_invoiceSender              = $invoiceSender;
            $this->_stockRegistry              = $stockRegistry;
            $this->_dbTransaction              = $dbTransaction;
            $this->_checkoutHelper             = $checkoutHelper;
            $this->_quoteValidator             = $quoteValidator;
            $this->_localeResolver             = $localeResolver;
            $this->_productFactory             = $productFactory;
            $this->_invoiceService             = $invoiceService;
            $this->_customerAddress            = $customerAddress;
            $this->_quoteManagement            = $quoteManagement;
            $this->_relatedProducts            = $relatedProducts;
            $this->_quoteRepository            = $quoteRepository;
            $this->_customerFactory            = $customerFactory;
            $this->addressInterface            = $addressInterface;
            $this->_checkoutSession            = $checkoutSession;
            $this->_customerSession            = $customerSession;
            $this->_orderEmailSender           = $orderEmailSender;
            $this->_regionCollection           = $regionCollection;
            $this->_dataObjectHelper           = $dataObjectHelper;
            $this->_accountManagement          = $accountManagement;
            $this->_productVisibility          = $productVisibility;
            $this->_objectCopyService          = $objectCopyService;
            $this->_countryCollection          = $countryCollection;
            $this->_customerRepository         = $customerRepository;
            $this->_transactionBuilder         = $transactionBuilder;
            $this->_customerManagement         = $customerManagement;
            $this->_quoteAddressToOrder        = $quoteAddressToOrder;
            $this->_orderCustomermanager       = $orderCustomermanager;
            $this->_quoteItemToOrderItem       = $quoteItemToOrderItem;
            $this->_downloadableConfiguration  = $downloadableConfiguration;
            $this->_quotePaymentToOrderPayment = $quotePaymentToOrderPayment;
            $this->_quoteAddressToOrderAddress = $quoteAddressToOrderAddress;
            $this->_deliverydate = $deliverydate;
            $this->_totalsInformationInterface = $totalsInformationInterface;
            $this->_totalsInformationManagement = $totalsInformationManagement;
            $this->deliveryHelper      = $deliveryHelper;
            $this->tintervalCollection = $tintervalCollection;
            parent::__construct($helper, $context);
        }

        protected function _customerEmailExists($email, $websiteId = null)  {
            $customer = $this->_customerFactory->create();
            if ($websiteId)
                $customer->setWebsiteId($websiteId);
            $customer->loadByEmail($email);
            if ($customer->getId())
                return $customer;
            return false;
        }

        protected function _validateCustomerData($data)     {
            $storeId      = $data["storeId"];
            $customerData = [];
            $customer     = null;
            $customerForm = $this->_customerForm->setFormCode("checkout_register");
            $quote        = new \Magento\Framework\DataObject();
            if ($data["customerId"] != 0) {
                $customerId = $data["customerId"];
                $quoteCollection = $this->_quoteFactory->create()->getCollection()
                    ->addFieldToFilter("is_active", 1)
                    ->addFieldToFilter("customer_id", $customerId)
                    ->addOrder("updated_at", "DESC");
                $quote = $quoteCollection->getFirstItem();
            }
            if ($data["quoteId"] != 0) {
                $quoteId = $data["quoteId"];
                $quote   = $this->_quoteFactory->create()->setStoreId($storeId)->load($quoteId);
            }
            if ($quote->getCustomerId()) {
                $customer = $quote->getCustomer();
                $customer = $this->_customerFactory->create()->load($customer->getId());
                $customerForm->setEntity($customer);
                $customerData = $customer->getData();
            } else {
                $customer = $this->_customerFactory->create();
                $customerForm->setEntity($customer);
                $newAddress  = [];
                $billingData = $data["billingData"];
                $billingData = $this->_jsonHelper->jsonDecode($billingData);
                if (isset($billingData["newAddress"])) {
                    if (!empty($billingData["newAddress"]))
                        $newAddress = $billingData["newAddress"];
                }
                $customerData = array(
                    "lastname"   => $newAddress["lastName"],
                    "firstname"  => $newAddress["firstName"],
                    "dob"        => $newAddress["dob"]        ?? "",
                    "email"      => $newAddress["email"]      ?? "",
                    "prefix"     => $newAddress["prefix"]     ?? "",
                    "suffix"     => $newAddress["suffix"]     ?? "",
                    "taxvat"     => $newAddress["taxvat"]     ?? "",
                    "gender"     => $newAddress["gender"]     ?? "",
                    "middlename" => $newAddress["middleName"] ?? ""
                );
            }
            $customerErrors = true;
            if ($customerErrors !== true)
                return ["error"=>1, "message"=>implode(", ", $customerErrors)];
            if ($quote->getCustomerId())
                return true;
            if ($quote->getCheckoutMethod() == "register") {
                $customerForm->compactData($customerData);
                $customer->setPassword($data["password"]);
                $customer->setConfirmation($data["confirmPassword"]);
                $customer->setPasswordConfirmation($data["confirmPassword"]);
                $result = $customer->validate();
                if (true !== $result && is_array($result))
                    return ["error"=>-1, "message"=>implode(", ", $result)];
            }
            if ($quote->getCheckoutMethod() == "register") {
                $quote->setPasswordHash($customer->encryptPassword($customer->getPassword()));
                $quote->setCustomer($customer);
            }
            $quote->getBillingAddress()->setEmail($customer->getEmail());
            $this->_objectCopyService->copyFieldsetToTarget("customer_account", "to_quote", $customer, $quote);
            return true;
        }

    }