<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class RescheduleDeliveryFormData extends AbstractCustomer     {

        public function execute()   {
            $returnArray                    = [];

            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $storeId     = $wholeData["storeId"]     ?? 1;
                        $incrementId = $wholeData["incrementId"] ?? 0;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $order       = $this->_order->loadByIncrementId($incrementId);
                        $returnArray["title"] = __('Edit Delivery Date For The Order #%1', $order->getIncrementId());
                        $returnArray['orderInfoTitle'] = __('Order Information');
                        $returnArray['orderTitle'] = __('Order #');
                        $returnArray['incrementId'] = $order->getRealOrderId();
                        $returnArray['emailTitle'] = __('Order #');
                        $returnArray['email'] = $order->getCustomerEmail();
                        $returnArray['deliveryDateTitle'] = __('Current Delivery Date');
                        
//delivery date data//////////////////////////////////////////////////////////////////////////////////////////////
                        $deliveryDate = $this->deliveryDateFactory->create();
                        $this->deliverydateResourceModel->load($deliveryDate, $order->getId(), 'order_id');
                        $returnArray['timeIntervalId'] = $deliveryDate->getTintervalId();
                        $returnArray['deliveryDate'] = $deliveryDate->getFormattedDate();
                        $returnArray['deliveryDateFormTitle'] = __('Delivery Date');
                        $returnArray['deliveryDateFormLabel'] = __('New Delivery Date');
                        $returnArray['BackLinkLabel'] = __('Back to Order # %1', $order->getRealOrderId());
                        $returnArray['deliveryDateConfiguration'] = $this->getCalendarJsonConfig();
                        $returnArray['deliveryDateObject'] = $this->getAmastyCalendarJsonConfig();
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

        /**
         * @return string json
         */
        public function getCalendarJsonConfig()
        {
            return [
                'minDate' => $this->deliveryHelper->getMinDays(),
                'maxDate' => $this->deliveryHelper->getStoreScopeValue('general/max_days'),
                'dateFormat' => $this->getCalendarDateFormat(),
            ];
        }

        /**
         * @return string
         */
        protected function getCalendarDateFormat()
        {
            $format = $this->deliveryHelper->getStoreScopeValue('date_field/format');
            return preg_replace(['/D/s','/M/s'], ['d','m'], $format);

        }

        public function getAmastyCalendarJsonConfig()
        {
            return $this->configProvider->getDeliveryDateFieldConfig();
        }

    }