<?php
    /**
    * Webkul Software.
    *
    * @category  Webkul
    * @package   Webkul_Mobikul
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Controller\Customer;

    class Wishlist extends AbstractCustomer  {

        public function execute()   {
            $returnArray                 = [];
            $returnArray["success"]      = false;
            $returnArray["message"]      = "";
            $returnArray["wishList"]     = [];
            $returnArray["totalCount"]   = 0;
            try {
                $wholeData       = $this->getRequest()->getPostValue();
                $this->_headers  = $this->getRequest()->getHeaders();
                $this->_helper->log(__CLASS__, "logClass", $wholeData);
                $this->_helper->log($wholeData, "logParams", $wholeData);
                $this->_helper->log($this->_headers, "logHeaders", $wholeData);
                if ($wholeData) {
                    $authKey     = $this->getRequest()->getHeader("authKey");
                    $authData    = $this->_helper->isAuthorized($authKey);
                    if ($authData["code"] == 1) {
                        $width       = $wholeData["width"]      ?? 1000;
                        $storeId     = $wholeData["storeId"]    ?? 1;
                        $quoteId     = $wholeData["quoteId"]    ?? 1;
                        $customerId  = $wholeData["customerId"] ?? 0;
                        $pageNumber  = $wholeData["pageNumber"] ?? 1;
                        $environment = $this->_emulate->startEnvironmentEmulation($storeId);
                        $wishlist    = $this->_wishlistProvider->loadByCustomerId($customerId, true);
                        $wishListCollection = $wishlist->getItemCollection();
// Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                        if($pageNumber >= 1) {
                            $returnArray["totalCount"] = $wishListCollection->getSize();
                            $pageSize = $this->_helperCatalog->getPageSize();
                            $wishListCollection->setPageSize($pageSize)->setCurPage($pageNumber);
                        }
                        
                        
                        $quoteCollection = $this->_objectManager->create("\Magento\Quote\Model\Quote")
                            ->getCollection()
                            ->addFieldToFilter("customer_id", $customerId)
                            ->addFieldToFilter("is_active", 1)
                            ->addOrder("updated_at", "DESC");
                        if ($quoteCollection->getFirstItem()->getId() > 0) {
                            $this->_objectManager->create("\Magento\Checkout\Model\Session")->setQuoteId($quoteCollection->getFirstItem()->getId());
                            $returnArray["freeShippingLabel"] = strip_tags($this->_objectManager->create("\Aheadworks\Freeshippinglabel\Block\Label")->getLabelModel()->getMessage());
                        }
// Creating Wish List ///////////////////////////////////////////////////////////////////////////////////////////////////////////
                        $wishList = [];
                        foreach ($wishListCollection as $item) {
                            $eachWishData = [];
                            $eachWishData["id"]          = $item->getId();
                            $product                     = $this->_productFactory->create()->load($item->getProductId());
                            $eachWishData["sku"]         = $product->getSku();
                            $eachWishData["qty"]         = $item->getQty() * 1;
                            $eachWishData["name"]        = $product->getName();
                            $eachWishData["price"]       = $this->_helperCatalog->stripTags($this->_pricingHelper->currency($product->getFinalPrice()));
                            $eachWishData["productId"]   = $product->getId();
                            $eachWishData["thumbNail"]   = $this->_helperCatalog->getImageUrl($product, $width/3);
                            $eachWishData["description"] = $item->getDescription();
                            $options                     = $this->_productConfigurationHelper->getOptions($item);
                            $eachWishData["options"]     = [];
                            if (count($options) > 0){
                                foreach ($options as $option){
                                    $eachOption                = [];
                                    $eachOption["label"]       = html_entity_decode($option["label"]);
                                    if (is_array($option["value"]))
                                        $eachOption["value"]   = $option["value"]; 
                                    else
                                        $eachOption["value"][] = $option["value"]; 
                                    $eachWishData["options"][] = $eachOption;
                                }
                            }
                            $reviews = $this->_reviewCollection
                                    // ->addStoreFilter($storeId)
                                    ->addEntityFilter("product", $product->getId())
                                    ->addStatusFilter(\Magento\Review\Model\Review::STATUS_APPROVED)
                                    ->setDateOrder()
                                    ->addRateVotes();
                            $ratings = [];
                            if (count($reviews) > 0) {
                                foreach ($reviews->getItems() as $review) {
                                    foreach ($review->getRatingVotes() as $vote)
                                        $ratings[] = $vote->getPercent();
                                }
                            }
                            if (count($ratings) > 0)
                                $rating = number_format((5 * (array_sum($ratings) / count($ratings))) / 100, 2, ".", "");
                            else
                                $rating = 0;
                            $eachWishData["rating"] = $rating;
                            $wishList[] = $eachWishData;
                            
                        }
                        $returnArray["success"]  = true;
                        $returnArray["wishList"] = $wishList;
                        $this->_emulate->stopEnvironmentEmulation($environment);
                        $this->_helper->log($returnArray, "logResponse", $wholeData);
                        return $this->getJsonResponse($returnArray);
                    } else {
                        return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                    }
                } else {
                    $returnArray["message"]      = __("Invalid Request");
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                }
            } catch (\Exception $e) {
                $returnArray["message"] = $e->getMessage();
                $this->_helper->printLog($returnArray, 1);
                return $this->getJsonResponse($returnArray);
            }
        }

    }