<?php
/**
* Webkul Software.
*
* @category  Webkul
* @package   Webkul_Mobikul
* @author    Webkul
* @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
* @license   https://store.webkul.com/license.html
*/

namespace Webkul\Mobikul\Controller\Extra;

class CustomCollection extends AbstractMobikul
{
    public function execute()
    {
        $returnArray                 = [];
        $returnArray["message"]      = "";
        $returnArray["cartCount"]    = 0;
        $returnArray["totalCount"]   = 0;
        $returnArray["productList"]  = [];
        $returnArray["success"]  = false;
        try {
            $wholeData       = $this->getRequest()->getPostValue();
            $this->_headers  = $this->getRequest()->getHeaders();
            $this->_helper->log(__CLASS__, "logClass", $wholeData);
            $this->_helper->log($wholeData, "logParams", $wholeData);
            $this->_helper->log($this->_headers, "logHeaders", $wholeData);
            if ($wholeData) {
                $authKey     = $this->getRequest()->getHeader("authKey");
                $authData    = $this->_helper->isAuthorized($authKey);
                if ($authData["code"] == 1) {
                    $width             = $wholeData["width"]          ?? 1000;
                    $storeId           = $wholeData["storeId"]        ?? 19;
                    $quoteId           = $wholeData["quoteId"]        ?? 0;
                    $sortData          = $wholeData["sortData"]       ?? "[]";
                    $customerId        = $wholeData["customerId"]     ?? 0;
                    $pageNumber        = $wholeData["pageNumber"]     ?? 1;
                    $filterData        = $wholeData["filterData"]     ?? "[]";
                    $notificationId    = $wholeData["notificationId"] ?? 0;
                    $landingPageId     = $wholeData["landingPageId"]  ?? 0;
                    $amasty            = $wholeData["amasty"]         ?? 0;
                    $sortData          = $this->_jsonHelper->jsonDecode($sortData);
                    $filterData        = $this->_jsonHelper->jsonDecode($filterData);
                    $environment       = $this->_emulate->startEnvironmentEmulation($storeId);
                    $productCollection = $this->_productCollection;
                    $notification      = $this->_mobikulNotification->create()->load($notificationId);
                    $customFilterData  = unserialize($notification->getFilterData());
                    if ($amasty > 0) {
                        if ($landingPageId > 0) {
                            $productCollection = $this->getAmastyProductCollection($landingPageId, $storeId);
                        } else {
                            $returnArray["message"] = __("Invalid page Id");
                            $this->_helper->log($returnArray, "logResponse", $wholeData);
                        }
                    } else {
                        if ($notification->getCollectionType() == "product_attribute") {
                            $productCollection->setStore($storeId)
                                ->addAttributeToSelect("*")
                                ->addAttributeToSelect("as_featured")
                                ->addAttributeToSelect("visibility")
                                ->addStoreFilter()
                                ->addAttributeToFilter("status", ["in"=>$this->_productStatus->getVisibleStatusIds()])
                                ->setVisibility($this->_productVisibility->getVisibleInSiteIds());
                            foreach ($customFilterData as $key => $filterValue) {
                                if ($key == "category_ids") {
                                    foreach (explode(",", $filterValue) as $value)
                                        $productCollection->addCategoryFilter($this->_categoryFactory->create()->load($value));
                                } else {
                                    $productCollection->addAttributeToSelect($key);
                                    $productCollection->addAttributeToFilter($key, array("in" => $filterValue));
                                }
                            }
                        } elseif ($notification->getCollectionType() == "product_ids") {
                            $productCollection->setStore($storeId)
                                ->addAttributeToSelect("*")
                                ->addAttributeToSelect("as_featured")
                                ->addAttributeToSelect("visibility")
                                ->addStoreFilter()
                                ->addAttributeToFilter("status", ["in"=>$this->_productStatus->getVisibleStatusIds()])
                                ->setVisibility($this->_productVisibility->getVisibleInSiteIds());
                            $productCollection->addAttributeToFilter("entity_id", array("in" => explode(",", $customFilterData)));
                        } elseif ($notification->getCollectionType() == "product_new") {
                            $todayStartOfDayDate = $this->_localeDate->date()->setTime(0, 0, 0)->format("Y-m-d H:i:s");
                            $todayEndOfDayDate   = $this->_localeDate->date()->setTime(23, 59, 59)->format("Y-m-d H:i:s");
                            $productCollection->setVisibility($visibleCatalogIds)
                                ->addMinimalPrice()
                                ->addFinalPrice()
                                ->addAttributeToSelect("*")
                                ->addStoreFilter()
                                ->addAttributeToFilter("news_from_date", ["or"=>[
                                        0=>["date"=>true, "to"=>$todayEndOfDayDate],
                                        1=>["is"=>new \Zend_Db_Expr("null")]]
                                    ], "left")
                                ->addAttributeToFilter("news_to_date", ["or"=>[
                                        0=>["date"=>true, "from"=>$todayStartOfDayDate],
                                        1=>["is"=>new \Zend_Db_Expr("null")]]
                                    ], "left")
                                ->addAttributeToFilter([
                                        ["attribute"=>"news_from_date", "is"=>new \Zend_Db_Expr("not null")],
                                        ["attribute"=>"news_to_date", "is"=>new \Zend_Db_Expr("not null")]
                                    ]);
                            $returnArray["totalCount"] = $customFilterData;
                            if ($pageNumber >= 1)
                                $productCollection->setPageSize($customFilterData)->setCurPage($pageNumber);
                        }
                    }
                    // Filtering product collection /////////////////////////////////////////////////////////////////////////////////////////////////
                    if(count($filterData) > 0) {
                        for($i=0; $i<count($filterData[0]); ++$i) {
                            if($filterData[0][$i] != "" && $filterData[1][$i] == "price") {
                                $minPossiblePrice = .01;
                                $currencyRate     = $productCollection->getCurrencyRate();
                                $priceRange       = explode("-", $filterData[0][$i]);
                                $from             = $priceRange[0];
                                $to               = $priceRange[1];
                                $fromRange        = ($from - ($minPossiblePrice / 2)) / $currencyRate;
                                $toRange          = ($to - ($minPossiblePrice / 2)) / $currencyRate;
                                $select           = $productCollection->getSelect();
                                if($from !== "")
                                    $select->where("price_index.min_price".">=".$fromRange);
                                if($to !== "")
                                    $select->where("price_index.min_price"."<".$toRange);
                            } elseif($filterData[0][$i] != "" && $filterData[1][$i] == "cat") {
                                $categoryToFilter = $this->_categoryFactory->create()->load($filterData[0][$i]);
                                $productCollection->setStoreId($storeId)->addCategoryFilter($categoryToFilter);
                            } else {
                                $attribute  = $this->_eavConfig->getAttribute("catalog_product", $filterData[1][$i]);
                                $this->_filterAttribute->setAttributeModel($attribute);
                                $filterAtr  = $this->_layerFilterAttribute;
                                $connection = $filterAtr->getConnection();
                                $tableAlias = $attribute->getAttributeCode()."_idx";
                                $conditions = [
                                    "{$tableAlias}.entity_id = e.entity_id",
                                    $connection->quoteInto("{$tableAlias}.attribute_id = ?", $attribute->getAttributeId()),
                                    $connection->quoteInto("{$tableAlias}.store_id = ?", $productCollection->getStoreId()),
                                    $connection->quoteInto("{$tableAlias}.value = ?", $filterData[0][$i]),
                                ];
                                $productCollection->getSelect()->join([$tableAlias=>$filterAtr->getMainTable()],implode(" AND ",$conditions),[]);
                            }
                        }
                    }
                    // Applying pagination //////////////////////////////////////////////////////////////////////////////////////////////////////////
                    if ($notification->getCollectionType() != "product_new" && $pageNumber >= 1) {
                        $returnArray["totalCount"] = $productCollection->getSize();
                        $pageSize = $this->_helperCatalog->getPageSize();
                        $productCollection->setPageSize($pageSize)->setCurPage($pageNumber);
                    }
                    // Sorting product collection ///////////////////////////////////////////////////////////////////////////////////////////////////
                    if (count($sortData) > 0) {
                        $sortBy = $sortData[0];
                        if($sortData[1] == 0)
                            $productCollection->setOrder($sortBy, "ASC");
                        else
                            $productCollection->setOrder($sortBy, "DESC");
                    }
                    elseif($notification->getCollectionType() == "product_new"){
                        $productCollection->addAttributeToSort("news_from_date", "DESC");
                    }
                    foreach ($productCollection as $eachProduct) {
                        $eachProduct = $this->_productFactory->create()->load($eachProduct->getId());
                        if($eachProduct->isAvailable() || (bool)$this->_helper->getConfigData("cataloginventory/options/show_out_of_stock"))
                            $returnArray["productList"][] = $this->_helperCatalog->getOneProductRelevantData($eachProduct, $storeId, $width, $customerId);
                    }
                    // Creating layered attribute collection ////////////////////////////////////////////////////////////////////////////////////////
                    $layeredData = [];
                    $filters = $this->_filterableAttributes->getList();
                    foreach ($filters as $filter) {
                        $doAttribute = true;
                        if (count($filterData) > 0) {
                            if (in_array($filter->getAttributeCode(), $filterData[1]))
                                $doAttribute = false;
                        }
                        if ($doAttribute) {
                            $attributeFilterModel = $this->_filterAttribute->setAttributeModel($filter);
                            if ($attributeFilterModel->getItemsCount()) {
                                $each = array();
                                $each["label"]   = $filter->getFrontendLabel();
                                $each["code"]    = $filter->getAttributeCode();
                                $each["options"] = $this->_helperCatalog->getAttributeFilter($attributeFilterModel, $filter);
                                $layeredData[]   =  $each;
                            }
                        }
                    }
                    $returnArray["layeredData"] = $layeredData;
                    // Creating sort attribute collection ///////////////////////////////////////////////////////////////////////////////////////////
                    $sortingData       = [];
                    $toolbar           = $this->_toolbar;
                    foreach($toolbar->getAvailableOrders() as $key=>$order) {
                        $each          = [];
                        $each["code"]  = $key;
                        $each["label"] = $order;
                        $sortingData[] = $each;
                    }
                    $returnArray["sortingData"] = $sortingData;
                    if ($customerId != 0) {
                        $quote = $this->_quote->getCollection()
                            ->addFieldToFilter("customer_id", $customerId)
                            ->addFieldToFilter("is_active", 1)
                            ->addOrder("updated_at", "DESC")
                            ->getFirstItem();
                        $quote->collectTotals()->save();
                        $returnArray["cartCount"] = $quote->getItemsQty() * 1;
                    }
                    if ($quoteId != 0) {
                        $returnArray["cartCount"] = $this->_quote->setStoreId($storeId)->load($quoteId)->getItemsQty() * 1;
                    }
                    $returnArray["success"] = true;
                    $this->_emulate->stopEnvironmentEmulation($environment);
                    $this->_helper->log($returnArray, "logResponse", $wholeData);
                    return $this->getJsonResponse($returnArray);
                } else {
                    return $this->getJsonResponse($returnArray, 401, $authData["token"]);
                }
            } else {
                $returnArray["message"]      = __("Invalid Request");
                $this->_helper->log($returnArray, "logResponse", $wholeData);
                return $this->getJsonResponse($returnArray);
            }
        } catch (\Exception $e) {
            $returnArray["message"] = __($e->getMessage());
            $this->_helper->printLog($returnArray, 1);
            return $this->getJsonResponse($returnArray);
        }
    }

    public function getAmastyProductCollection($pageId, $storeId)
    {
        $landingPageCollection = $this->_amastyPageFactory->create()->getCollection()
            ->addFieldToFilter('page_id', ['eq' => $pageId])
            ->addFieldToSelect('page_id', 'store_id');
        $connection = $this->_connection->getConnection(\Magento\Framework\App\ResourceConnection::DEFAULT_CONNECTION);
        $pageProductIndexTable = $connection->getTableName("amasty_xlanding_page_product_index");
        $landingPageCollection->getSelect()->join(
            $pageProductIndexTable.' as ppit',
            'main_table.page_id = ppit.page_id',
            ['product_id']
        )->where("ppit.store_id = ".$storeId);
        $productIds = [];
        foreach ($landingPageCollection as $landingPage) {
            $productIds[] = $landingPage->getProductId();
        }
        $productCollection = $this->_productFactory->create()->getCollection()
            ->addFieldToFilter('entity_id', ['in' => $productIds]);
        return $productCollection;
    }
}