<?php
    /**
    * Webkul Software.
    *
    * @category Webkul
    *
    * @author    Webkul
    * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
    * @license   https://store.webkul.com/license.html
    */

    namespace Webkul\Mobikul\Helper;

    class Data extends \Magento\Framework\App\Helper\AbstractHelper     {

        protected $_encrypted;
        protected $_storeManager;
        protected $_directoryList;
        protected $_sessionManager;

        public function __construct(
            \Magento\Framework\App\Helper\Context $context,
            \Magento\Store\Model\StoreManagerInterface $storeManager,
            \Magento\Config\Model\Config\Backend\Encrypted $encrypted,
            \Magento\Framework\Filesystem\DirectoryList $directoryList,
            \Magento\Framework\Session\SessionManagerInterface $sessionManager
        ) {
            $this->_encrypted      = $encrypted;
            $this->_storeManager   = $storeManager;
            $this->_directoryList  = $directoryList;
            $this->_sessionManager = $sessionManager;
            parent::__construct($context);
        }

        public function getUrl($dir)    {
            return $this->_storeManager->getStore()->getBaseUrl($dir);
        }

        public function isAuthorized($authKey)   {
            $authData          = [];
            $authData["code"]  = 2;
            $authData["token"] = "";
            $apiKey       = $this->getPassword();
            $apiUsername  = $this->getConfigData("mobikul/configuration/apiusername");
            $sessionToken = $this->_sessionManager->getSessionId();
            $H1 = md5($apiUsername.":".$apiKey);
            $H2 = md5($H1.":".$sessionToken);
            if($authKey == $H2)
                $authData["code"] = 1;
            else
                $authData["token"] = $sessionToken;
                $authData["code"] = 1;
            return $authData;
        }

        public function log($data, $key, $wholeData) {
            $flag = $wholeData[$key] ?? 0;
            $this->printLog($data, $flag);
        }

        public function printLog($data, $flag=1, $filename="mobikul.log"){
            if($flag == 1)  {
                $path   = $this->_directoryList->getPath("var");
                $logger = new \Zend\Log\Logger();
                if (!file_exists($path."/log/"))
                    mkdir($path."/log/", 0777, true);
                $logger->addWriter(new \Zend\Log\Writer\Stream($path."/log/".$filename));
                if (is_array($data) || is_object($data))
                    $data = print_r($data, true);
                $logger->info($data);
            }
        }

        public function validate($wholeData, $key)    {
            if(isset($wholeData[$key]) && $wholeData[$key] != "")
                return $wholeData[$key];
            else
                return false;
        }

        public function getConfigData($path, $scope=\Magento\Store\Model\ScopeInterface::SCOPE_STORE) {
            return $this->scopeConfig->getValue($path, $scope);
        }

        public function canReorder(\Magento\Sales\Model\Order $order)   {
            if (!$this->getConfigData("sales/reorder/allow"))
                return 0;
            if (1)
                return $order->canReorder();
            else
                return 1;
        }

        public function getPassword()   {
            return $this->_encrypted->processValue(
                $this->scopeConfig->getValue("mobikul/configuration/apikey", \Magento\Store\Model\ScopeInterface::SCOPE_STORE)
            );
        }

    }
