<?php
/**
 * Webkul Software.
 *
 * @category  Webkul
 * @package   Webkul_Mobikul
 * @author    Webkul
 * @copyright Copyright (c) 2010-2018 Webkul Software Private Limited (https://webkul.com)
 * @license   https://store.webkul.com/license.html
 */

namespace Webkul\Mobikul\Ui\Component\Listing\Columns;

use Magento\Store\Model\StoreManagerInterface;
use Webkul\Mobikul\Helper\Barcode as HelperBarcode;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Framework\View\Element\UiComponent\ContextInterface;

class BarCodeImage extends \Magento\Ui\Component\Listing\Columns\Column
{
    protected $_product;
    protected $_baseDir;
    protected $_storeManager;
    protected $_helperBarcode;

    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        StoreManagerInterface $storeManager,
        array $components = [],
        array $data = [],
        \Magento\Framework\Filesystem\DirectoryList $dir,
        HelperBarcode $helperBarcode,
        \Magento\Catalog\Model\ProductFactory $product
    ) {
        parent::__construct($context, $uiComponentFactory, $components, $data);
        $this->_baseDir       = $dir->getPath("media");
        $this->_storeManager  = $storeManager;
        $this->_helperBarcode = $helperBarcode;
        $this->_product       = $product;
    }

    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource["data"]["items"])) {
            $obManager = \Magento\Framework\App\ObjectManager::getInstance();
            $target    = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA);
            $fieldName = $this->getData("name");
            $ioFile    = $obManager->get("\Magento\Framework\Filesystem\Io\File");
            foreach ($dataSource["data"]["items"] as $key => $item) {
                $barcode = $this->_product->create()->load($item['entity_id'])->getBarcode();
                $basePath = $this->_baseDir."/"."mobikul"."/"."product"."/".$item["entity_id"]."/";
                $fileName = str_replace(" ", "_", $barcode).".png";
                if (!is_file($basePath.$fileName)) {
                    $ioFile->mkdir($basePath, 0777);
                    $this->_helperBarcode->generatebarcode($basePath.$fileName, $barcode, 20, "horizontal", "code128", false, 1);
                }
                $dataSource["data"]["items"][$key][$fieldName."_src"]  = $target."mobikul"."/"."product"."/".$item["entity_id"]."/".$fileName;
                $dataSource["data"]["items"][$key][$fieldName."_alt"]  = $barcode;
            }
        }
        return $dataSource;
    }
}
