<?php
/**
 * Copyright © 2017 Mageside. All rights reserved.
 * See MS-LICENSE.txt for license details.
 */

namespace Mageside\BackupToDropbox\Helper;

use Magento\Customer\Api\GroupManagementInterface;
use Magento\Framework\App\Helper\AbstractHelper;

class UiFormHelper extends AbstractHelper
{
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    protected $_scopeConfig;

    /**
     * @var \Magento\Framework\Math\Random
     */
    protected $_mathRandom;

    /**
     * @var GroupManagementInterface
     */
    protected $_groupManagement;

    /**
     * @var \Magento\Framework\Unserialize\Unserialize
     */
    protected $_unserialize;

    /**
     * UiFormHelper constructor.
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Math\Random $mathRandom
     * @param GroupManagementInterface $groupManagement
     * @param \Magento\Framework\Unserialize\Unserialize $unserialize
     */
    public function __construct(
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Math\Random $mathRandom,
        GroupManagementInterface $groupManagement,
        \Magento\Framework\Unserialize\Unserialize $unserialize
    ) {
        $this->_scopeConfig = $scopeConfig;
        $this->_mathRandom = $mathRandom;
        $this->_groupManagement = $groupManagement;
        $this->_unserialize = $unserialize;
    }

    /**
     * @param $value
     * @return array|string
     */

    public function getValueIgnoreList($value)
    {
        if ($this->isEncodedArrayFieldValue($value)) {
            $value = $this->decodeArrayFieldValue($value);
        }
        $value = $this->serializeValue($value);
        return $value;
    }

    /**
     * @param $value
     * @return bool
     */
    protected function isEncodedArrayFieldValue($value)
    {
        if (!is_array($value)) {
            return false;
        }
        unset($value['__empty']);
        foreach ($value as $field => $row) {
            if (!is_array($row)
                || !('table_ignore_list' == $field)
                || !('folder_ignore_list' == $field)) {
                return false;
            }
        }
        return true;
    }

    /**
     * @param array $value
     * @return array
     */
    protected function decodeArrayFieldValue(array $value)
    {
        $result = [];
        unset($value['__empty']);
        foreach ($value as $row) {
            if (!is_array($row)
                || !array_key_exists('table_ignore_list', $row)
                || !array_key_exists('folder_ignore_list', $row)) {
                continue;
            }
            if (array_key_exists('table_ignore_list', $row)) {
                $result['table_ignore_list'] = $row['table_ignore_list'];
            }
            if (array_key_exists('folder_ignore_list', $row)) {
                $result['folder_ignore_list'] = $row['folder_ignore_list'];
            }
        }
        return $result;
    }

    /**
     * @param $value
     * @return string
     */
    protected function serializeValue($value)
    {
        if (is_numeric($value)) {
            return (string) $value;
        } elseif (is_array($value)) {
            unset($value['__empty']);
            $data = [];
            $i = 1;
            foreach ($value as $paths) {
                foreach ($paths as $ignore_list => $item) {
                    if ($i>1) {
                        if (!array_key_exists($item, $data[$ignore_list])) {
                            $data[$ignore_list][] = $item;
                            $i++;
                        }
                    } else {
                        $data[$ignore_list][] = $item;
                        $i++;
                    }
                }
            }
            return serialize($data);
        } else {
            return '';
        }
    }

    /**
     * @param $value
     * @return array|mixed
     */
    public function makeArrayFieldValue($value)
    {
        $value = $this->unserializeValue($value);
        if (!$this->isEncodedArrayFieldValue($value)) {
            $value = $this->encodeArrayFieldValue($value);
        }
        return $value;
    }

    /**
     * @param $value
     * @return array|mixed
     */
    public function unserializeValue($value)
    {
        if (is_numeric($value)) {
            return [$this->getAllCustomersGroupId() => $this->fixQty($value)];
        } elseif (is_string($value) && !empty($value)) {
            return $this->_unserialize->unserialize($value);
        } else {
            return [];
        }
    }

    /**
     * @param array $value
     * @return array
     */
    protected function encodeArrayFieldValue(array $value)
    {
        $result = [];
        foreach ($value as $list => $itemList) {
            foreach ($itemList as $item) {
                $result[] = [$list => $item];
            }
        }
        return $result;
    }
}
